// ConsoleMain.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file ConsoleMain.cs
 * Provide main console functions.*/

using System;
using System.IO;
using System.Timers;
using System.Xml;
using Mono.Unix;
using System.Reflection;

namespace Grubng
{
	
	/// <summary>
	/// Provide main console functions.
	/// </summary>
	internal sealed class ConsoleMain : IDisposable
	{
		
		/// <summary>
		/// Object from Threads class. Provide functions to upload, crawl and upload workunits.
		/// </summary>
		Threads workthread;
		/// <summary>
		/// Object from Config class. Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Object from RemoteControl class. Provide functions for web interface.
		/// </summary>
		RemoteControl rControl;
		/// <summary>
		/// Number of crawler to show in details
		/// </summary>
		int crawler;
		/// <summary>
		/// Status of selected crawler
		/// </summary>
		int status;
		/// <summary>
		/// Workunit time timer
		/// </summary>
		Timer worktime;
		/// <summary>
		/// Status of crawlers.
		/// </summary>
		int crawlerstat;
		/// <summary>
		/// Program uptime
		/// </summary>
		TimeSpan uptimespan;
		/// <summary>
		/// Amount of working crawlers.
		/// </summary>
		int working;
		/// <summary>
		/// If true start program in auto mode
		/// </summary>
		bool automode;
		/// <summary>
		/// Array of translated crawler statuses
		/// </summary>
		string[] crawlerstatus;
		/// <summary>
		/// Array of translated crawler messages
		/// </summary>
		string[] crawlermessage;
		/// <summary>
		/// Check if resources are disposed.
		/// </summary>
		bool disposedres;
		/// <summary>
		/// Amount of connection errors which appear in this session.
		/// </summary>
		int connectionerrors;
		/// <summary>
		/// Amount of seconds which program must wait before connect again to servers.
		/// </summary>
		int connectiondelay;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public ConsoleMain()
		{
			this.conf = new Config();
			this.crawlerstatus = new string[11]{Catalog.GetString("Stopped"),
				Catalog.GetString("Paused"),
				Catalog.GetString("In Queue"),
				Catalog.GetString("Downloading"),
				Catalog.GetString("Download Error"),
				Catalog.GetString("Crawling"),
				Catalog.GetString("Uploading"),
				Catalog.GetString("Upload Error"),
				Catalog.GetString("Waiting"),
				Catalog.GetString("Invalid arc file"),
				Catalog.GetString("Program error")
			};
			this.crawlermessage = new string[24]{Catalog.GetString("Waiting in queue for download new workunit"),
				Catalog.GetString("Waiting in queue for upload workunit"),
				Catalog.GetString("Invalid login. Check your setting for username and password."),
				Catalog.GetString("Request timeout."),
				Catalog.GetString("Problems with connection to Internet."),
				Catalog.GetString("Unknown error"),
				Catalog.GetString("Done"),
				Catalog.GetString("File checked and compressed, ready to upload."),
				Catalog.GetString("Workunit uploaded"),
				Catalog.GetString("Invalid workunit, you must try crawl it again"),
				Catalog.GetString("Timeout reached when uploading. Please check your internet connection"),
				Catalog.GetString("Cannot connect to server. Try upload workunit later."),
				Catalog.GetString("Internal Server Error. Try download workunit later."),
				Catalog.GetString("No server response. Try upload workunit again."),
				Catalog.GetString("Program error appear and crawler was stopped."),
				Catalog.GetString("Internal Server Error. Try upload .arc file later."),
				Catalog.GetString("Sitemap uploaded."),
				Catalog.GetString("Internal Server Error. Sitemap cannot be uploaded."),
				Catalog.GetString("Generate sitemap."),
				Catalog.GetString("Upload .arc files to this server is temporary disabled. Please try later."),
				Catalog.GetString("You was sent this file earlier. You cannot send this same .arc file twice."),
				Catalog.GetString("Someone was sent this same URL's earlier."),
				Catalog.GetString("No workunits available at this moment. Please try later."),
				Catalog.GetString("Upload sitemaps to this server is disabled.")
			};
			this.status = -1;
			this.connectionerrors = 0;
			this.workthread = new Threads();
			this.workthread.Running = false;
			if (Config.ReadConfig("/configuration/other/enableremote", this.conf.FilePath) == "Y")
			{
				this.rControl = new RemoteControl(this.workthread);
				this.rControl.Start();
				this.rControl.WebEventHandler += new EventHandler<WebEventArgs>(OnWebMessage);
			}
			this.worktime = new System.Timers.Timer();
			this.worktime.Elapsed += new ElapsedEventHandler(OnWorkElapsed);
			this.worktime.Interval = 1000;
			this.worktime.Start();
		}
		
		/// <summary>
		/// Function show main menu, start crawling, show settings or quit from program.
		/// </summary>
		public void Start()
		{
			if (disposedres)
			{
				throw new ObjectDisposedException(GetType().Name);
			}
			string auto = Config.ReadConfig("/configuration/work/automatic", this.conf.FilePath);
			//Start program in manual mode
			if (auto == "N")
			{
				Console.WriteLine("1)" + Catalog.GetString("Start"));
				Console.WriteLine("2)" + Catalog.GetString("Start in auto mode"));
				Console.WriteLine("3)" + Catalog.GetString("Settings"));
				Console.WriteLine("4)" + Catalog.GetString("Show license"));
				Console.WriteLine("5)" + Catalog.GetString("Show help"));
				Console.WriteLine("6)" + Catalog.GetString("Show info about program"));
				Console.WriteLine("7)" + Catalog.GetString("Create sitemap"));
				Console.WriteLine("Q)" + Catalog.GetString("Quit"));
				char answer = Convert.ToChar(Console.Read());
				Console.Clear();
				//Quit from program
				if (answer == 'q')
				{
					return;
				}
				//Start crawling
				if (answer == '1')
				{
					Console.WriteLine(Catalog.GetString("Start crawling..."));
					this.Run();
				}
				//Start crawling in auto mode
				if (answer == '2')
				{
					Console.WriteLine(Catalog.GetString("Start crawling in auto mode..."));
					this.automode = true;
					this.connectiondelay = 0;
					this.Run();
				}
				//Set program preferences
				if (answer == '3')
				{
					ConsolePref pref = new ConsolePref();
					pref.Start();
					pref = null;
				}
				//Show program license
				if (answer == '4')
				{
					ConsoleMain.ShowFile("COPYING");
				}
				//Show README.txt
				if (answer == '5')
				{
					ConsoleMain.ShowFile("README.txt");
				}
				//Show info about program
				if (answer == '6')
				{
					ConsoleMain.ShowAbout();
				}
				//Create sitemap
				if (answer == '7')
				{
					ConsoleMain.CreateSitemap();
				}
			}
			//Start program in automatic mode
			else
			{
				Console.WriteLine(Catalog.GetString("Start crawling in auto mode..."));				
				this.automode = true;
				this.workthread.Running = true;
				this.Run();
			}
		}
		
		/// <summary>
		/// Main function - crawling and show crawling menu. It start crawling, show menu to user and quit from crawling.
		/// </summary>
		public void Run()
		{
			if (disposedres)
			{
				throw new ObjectDisposedException(GetType().Name);
			}
			this.workthread.TimeOut = 0;
			this.workthread.Start();
			char answer = 'a';
			string answer2;
			bool stoped;
			while (answer != 'q')
			{
				answer = ConsoleMain.Menu();
				Console.Clear();
				//Resume/pause crawling
				if (answer == 'r')
				{
					stoped = false;
					for (int i = 0; i < this.workthread.Crawlers; i++)
					{
						if ((this.workthread.GetField(i, "working") == 0) || (this.workthread.GetField(i, "working") == 2))
						{
							stoped = true;
							break;
						}
					}
					this.workthread.Running = false;
					//Resume crawling
					if (stoped)
					{
						Console.WriteLine(Catalog.GetString("Start crawling..."));
						this.workthread.WriteStats();
						this.workthread.TimeOut = 0;
						this.workthread.Start();
					}
					//Pause crawling
					else
					{
						Console.WriteLine(Catalog.GetString("Pause"));
						for (int i = 0; i < this.workthread.Crawlers; i++)
						{
							this.workthread.SetField(i, "working", 2);
						}
					}
				}
				//Show details about crawlers
				if (answer == 'd')
				{
					double tmp = 0;
					for (int i = 0; i < this.workthread.Crawlers; i++)
					{
						if (this.workthread.GetField(i, "status") == 6)
						{
							tmp = this.workthread.UploadProgress();
						}
						if (this.workthread.GetField(i, "status") == 5)
						{
							tmp = (double)this.workthread.GetField(i, "curiter") / (double)(this.workthread.GetField(i, "iter") - 1) * 100;
						}
						if (tmp < 0)
						{
							tmp = 0;
						}
						if (tmp > 100)
						{
							tmp = 100;
						}
						if (this.workthread.GetField(i, "status") == 0)
						{
							tmp = 0;
						}
						if ((tmp < int.MinValue) || (tmp > int.MaxValue))
						{
							tmp = 0;
						}
						Console.WriteLine("{0} [{1} %] {2} {3} {4} {5}", (i + 1).ToString(), 
						                  Convert.ToInt32(tmp).ToString(),  
						                  this.crawlerstatus[this.workthread.GetField(i, "status")], 
						                  this.workthread.GetStringField(i, "workfilename"), 
						                  this.workthread.GetStringField(i, "downloaddate"), 
						                  this.workthread.CountDiskUsage(i));
					}
					Console.WriteLine("{0} [1-{1}] {2}, {3}", Catalog.GetString("Press"), this.workthread.Crawlers.ToString(), Catalog.GetString("for show info about selected crawler"), Catalog.GetString("Q for back to crawling menu."));
					answer2 = Console.ReadLine();
					//Show details about selected crawler
					if ((answer2 != "q") && (answer2 != "Q"))
					{
						try
						{
							this.crawler = (Convert.ToInt32(answer2) - 1);
							this.workthread.messageEmit += new EventHandler(OnMessage);
							int result = Console.Read();
							result++;
							this.workthread.messageEmit -= new EventHandler(OnMessage);
						}
						catch (FormatException)
						{
							Console.WriteLine(Catalog.GetString("Wrong option"));
						}
					}
					Console.Clear();
				}
				//Show stats about crawling
				if (answer == 's')
				{
					string[] time = this.uptimespan.ToString().Split('.');
					if (this.uptimespan.Days == 1)
					{
						time[0] = time[0] + " " + Catalog.GetString("day") + " " + time[1];
					}
					if (this.uptimespan.Days > 1)
					{
						time[0] = time[0] + " " + Catalog.GetString("days") + " " + time[1];
					}
					Console.WriteLine(Catalog.GetString("Uptime:") + " " + time[0]);
					Console.WriteLine("{0} {1}/{2}", Catalog.GetString("Working crawlers:"), this.working.ToString(), this.workthread.Crawlers.ToString());
					double diskusage = 0;
					for (int i = 0; i < this.workthread.Crawlers; i++)
					{
						diskusage += this.workthread.CountDiskUsage(i);
					}
					Console.WriteLine("{0} {1}", Catalog.GetString("Disk Usage (MB):"), diskusage.ToString());
					Console.WriteLine("{0, 40}", Catalog.GetString("Statistics:"));
					Console.WriteLine("{0}{1, 50}", Catalog.GetString("Current:"), Catalog.GetString("Overall:"));
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("Workunits send:"), 
					                  this.workthread.GetField(0, "workunitsend").ToString(), 
					                  this.workthread.GetField(0, "overallworkunit").ToString());
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("Invalid workunits:"), 
					                  this.workthread.GetField(0, "invalidarc").ToString(), 
					                  this.workthread.GetField(0, "overallinvalidworkunit").ToString());
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("Links crawled:"), 
					                  this.workthread.GetField(0, "crawledurls").ToString(), 
					                  this.workthread.GetField(0, "overallcrawled").ToString());
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("Success:"), 
					                  this.workthread.GetField(0, "validurls").ToString(), 
					                  this.workthread.GetField(0, "overallvalid").ToString());
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("Failures:"), 
					                  this.workthread.GetField(0, "invalidurls").ToString(), 
					                  this.workthread.GetField(0, "overallinvalid").ToString());
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("Errors:"), 
					                  this.workthread.GetField(0, "errorurls").ToString(), 
					                  this.workthread.GetField(0, "overallerror").ToString());
					double tmp = Math.Round(this.workthread.GetDoubleField("uptimereceived"), 1);
					double tmp2 = Math.Round(this.workthread.GetDoubleField("oveallreceived"), 1);
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("MB Received:"), tmp.ToString(), tmp2.ToString());
					tmp = Math.Round(this.workthread.GetDoubleField("uptimesend"), 1);
					tmp2 = Math.Round(this.workthread.GetDoubleField("overallsent"), 1);
					Console.WriteLine("{0} {1}{0, 30} {2}", Catalog.GetString("MB Send:"), tmp.ToString(), tmp2.ToString());
					Console.WriteLine();
				}
				//Show program license
				if (answer == 'l')
				{
					ConsoleMain.ShowFile("COPYING");
				}
				//Show README.txt
				if (answer == 'h')
				{
					ConsoleMain.ShowFile("README.txt");
				}
				//Show info about program
				if (answer == 'a')
				{
					ConsoleMain.ShowAbout();
				}
				//Upload sitemap on server
				if (answer == 'u')
				{
					ConsoleMain.CreateSitemap();
				}
			}
			//Quit from program
			this.worktime.Stop();
			this.worktime.Dispose();
			Console.Write(Catalog.GetString("Stopping all crawlers..."));
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				this.workthread.SetField(i,"working", 0);
			}
			int quitdelay = 25;
			while (!this.workthread.CrawlersStopped())
			{
				System.Threading.Thread.Sleep(1000);
				quitdelay --;
				if (quitdelay == 0)
				{
					this.workthread.WriteStats();
					MainClass.DeletePIDFile();
					System.Diagnostics.Process proc = System.Diagnostics.Process.GetCurrentProcess();
					proc.Kill();
				}
			}
			this.workthread.WriteStats();
			Console.WriteLine(Catalog.GetString("Done"));
		}
		
		/// <summary>
		/// Function show crawling menu. Show menut to user and wait for key pressed.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Char"/> key pressed.
		/// </returns>
		static char Menu()
		{
			Console.WriteLine("R)" + Catalog.GetString("Start/Pause"));
			Console.WriteLine("D)" + Catalog.GetString("Details"));
			Console.WriteLine("S)" + Catalog.GetString("Stats"));
			Console.WriteLine("L)" + Catalog.GetString("Show license"));
			Console.WriteLine("H)" + Catalog.GetString("Show help"));
			Console.WriteLine("A)" + Catalog.GetString("Show info about program"));
			Console.WriteLine("C)" + Catalog.GetString("Create sitemap"));
			Console.WriteLine("Q)" + Catalog.GetString("Quit"));
			char answer = (char)Console.Read();
			Console.WriteLine();
			return Char.ToLower(answer);
		}
		
		/// <summary>
		/// Function show content of file.
		/// </summary>
		/// <param name="filename">
		/// A <see cref="System.String"/> name of file to show.
		/// </param>
		static void ShowFile(string filename)
		{
			System.IO.FileStream file = System.IO.File.OpenRead(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, filename));
			System.IO.StreamReader stream = new System.IO.StreamReader(file);
			string[] content = stream.ReadToEnd().Split('\n');
			stream.Close();
			stream = null;
			file.Close();
			file = null;
			int lines = 0;
			char answer;
			foreach (string line in content)
			{
				Console.WriteLine(line);
				lines++;
				if (lines == 19)
				{
					Console.WriteLine(Environment.NewLine + Catalog.GetString("==Press any key for more or Q for quit=="));
					lines = 0;
					answer = (char)Console.Read();
					Console.WriteLine();
					if ((answer == 'q') || (answer == 'Q'))
					{
						return;
					}
				}
			}
		}
		
		/// <summary>
		/// Function show info about program.
		/// </summary>
		static void ShowAbout()
		{
			Assembly asm = Assembly.GetExecutingAssembly();
			Console.WriteLine(asm.GetName().Name + " " + asm.GetName().Version.ToString());
			Console.WriteLine(Catalog.GetString("Grub Next Generation client"));
			object[] attributes = asm.GetCustomAttributes(typeof(AssemblyCopyrightAttribute), false);
			Console.WriteLine(((AssemblyCopyrightAttribute)attributes[0]).Copyright);
			Console.WriteLine("http://grub.org");
			Console.WriteLine(Catalog.GetString("Authors"));
			ConsoleMain.ShowFile("AUTHORS");
			Console.WriteLine();
		}
		
		/// <summary>
		/// Function update informations about crawler. Raised when crawler thread emits message event.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnMessage(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			string tdate = DateTime.Now.ToString("HH:mm:ss");
			int crawlstatus = this.workthread.GetField(this.crawler, "status");
			int message = this.workthread.GetField(this.crawler, "message");
			string servermessage = this.workthread.GetStringField(this.crawler, "servermessage");
			if (this.status != crawlstatus)
			{
				Console.WriteLine(tdate + " " + this.crawlerstatus[crawlstatus]);
				this.status = crawlstatus;
			}
			if (message != -1)
			{
				Console.WriteLine(tdate + " " + this.crawlermessage[message]);
				this.workthread.SetField(this.crawler, "message", 0);
			}
			if (servermessage.Length > 0)
			{
				Console.WriteLine(tdate + " " + Catalog.GetString("Server response:") + " " + servermessage);
				this.workthread.SetField(this.crawler, "servermessage", 0);
			}
			if (this.status == 5)
			{
				string cururl = this.workthread.GetStringField(this.crawler, "cururl");
				int curstatus = this.workthread.GetField(this.crawler, "curstatus");
				if (cururl.Length > 0)
				{
					Console.Write("{0} {1}/{2} {3} ", tdate, this.workthread.GetField(this.crawler, "curiter").ToString(), 
					              (this.workthread.GetField(this.crawler, "iter") - 1).ToString(), cururl);
					this.workthread.SetField(this.crawler, "cururl", 0);
				}
				if (curstatus != 0)
				{
					Console.WriteLine("{0} {1}]", Catalog.GetString("[Status:"), curstatus.ToString());
					this.workthread.SetField(this.crawler, "curstatus", 0);
				}
			}
		}
		
		/// <summary>
		/// Timer for check crawlers status, check internet connection. Raised every second when program works.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="ElapsedEventArgs"/> unused
		/// </param>
		void OnWorkElapsed(object sender, ElapsedEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			TimeSpan tmpspan = new TimeSpan(0, 0, 1);
			this.uptimespan += tmpspan;
			int stoped = 0;
			this.working = 0;
			int paused = 0;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{		
				switch (this.workthread.GetField(i, "working"))
				{
				case 0:
					stoped++;
					break;
				case 1:
					this.working++;
					break;
				case 2:
					paused++;
					break;
				}
			}
			if (paused != this.workthread.Crawlers)
			{
				this.CheckNet();
			}
			if (this.connectiondelay > 0)
			{
				this.connectiondelay--;
			}
			if ((stoped > 0) && (this.automode))
			{
				if (this.workthread.ConnectionError)
				{
					this.connectionerrors ++;
					this.connectiondelay = 30 * this.connectionerrors;
					this.workthread.ConnectionError = false;
					this.workthread.Running = false;
				}
				if ((this.connectiondelay <= 0) && (!this.workthread.Running))
				{
					this.workthread.Running = true;
					this.workthread.Start();
				}
			}
			string info = Catalog.GetString("[INFO: All crawlers");
			if ((this.working == this.workthread.Crawlers) && (this.crawlerstat != 1))
			{
				Console.WriteLine(info + " " + Catalog.GetString("works.]"));
				this.crawlerstat = 1;
			}
			if ((paused == this.workthread.Crawlers) && (this.crawlerstat != 2))
			{
				Console.WriteLine(info + " " + Catalog.GetString("paused.]"));
				this.crawlerstat = 2;
			}
			if ((stoped == this.workthread.Crawlers) && (this.crawlerstat != 0))
			{
				Console.WriteLine(info + " " + Catalog.GetString("stoped.]"));
				this.crawlerstat = 0;
			}
		}
		
		/// <summary>
		/// Function check internet connection. If there is more than 49 request timeouts in row, function ping net
		/// for check internet connection. If fail, pause all crawlers and show information to user about problems with
		/// connection.
		/// </summary>
		void CheckNet()
		{
			bool check = true;
			if (this.workthread.TimeOut > 49)
			{
				System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.WebRequest.Create("http://grub.org/");
				request.Timeout = 5000;
				request.ProtocolVersion = System.Net.HttpVersion.Version11;
				request.UserAgent = "GrubNG";
				request.KeepAlive = false;
				request.Proxy = null;
				request.AllowAutoRedirect = false;
				try
				{
					System.Net.HttpWebResponse response = (System.Net.HttpWebResponse)request.GetResponse();
					response.Close();
				}
				catch (System.Net.WebException e)
				{
					int error = (int)e.Status;
					if (error == 1)
					{
						check = false;
						for (int i = 0; i < this.workthread.Crawlers; i++)
						{
							this.workthread.SetField(i, "working", 2);
						}
						Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
						                  Catalog.GetString("Couldn't connect to several sites. ") + 
						                  Catalog.GetString("Possibly your computer isn't connected with the Internet anymore. ") + 
						                  Catalog.GetString("Please check the Internet connection.") + "]");
					}
				}
				if (check)
				{
					this.workthread.TimeOut = 0;
				}
			}
		}
		
		/// <summary>
		/// Function show menu for create and upload sitemap on server.
		/// </summary>
		static void CreateSitemap()
		{
			Console.WriteLine("1)" + Catalog.GetString("Create empty sitemap"));
			Console.WriteLine("2)" + Catalog.GetString("Open existing sitemap"));
			Console.WriteLine("3)" + Catalog.GetString("Create sitemap from URL"));
			Console.WriteLine("Q)" + Catalog.GetString("Quit"));
			char answer = (char)Console.Read();
			Console.WriteLine();
			if ((answer == 'q') || (answer == 'Q'))
			{
				return;
			}
			string file = String.Empty;
			Config conf = new Config();
			string filepath = Config.ReadConfig("/configuration/work/workdir", conf.FilePath);
			conf = null;
			Guid guid = Guid.NewGuid();
			string[] extension;
			int index;
			if (answer == '1')
			{
				file = filepath + guid.ToString() + ".xml";
			}
			if (answer == '2')
			{
				Console.WriteLine(Catalog.GetString("Enter name or path to sitemap file") + " " + 
				                  Catalog.GetString("(must be .xml or .xml.gz file).") + " " +
				                  Catalog.GetString("Leave it empty for exit."));
				file = Console.ReadLine();
				if (String.IsNullOrEmpty(file))
				{
					return;
				}
				bool checkfile = System.IO.File.Exists(file);
				if (!checkfile)
				{
					Console.WriteLine(Catalog.GetString("[INFO:") + " " +
					                  Catalog.GetString("File not exist.") + "]");
					return;
				}
				extension = file.Split('.');
				index = extension.Length - 1;
				if ((extension[index] != "xml") && (extension[index] != "gz"))
				{
					Console.WriteLine(Catalog.GetString("[INFO:") + " " +
					                  Catalog.GetString("Invalid filename.") + "]");
					return;
				}
			}
			if (answer == '3')
			{
				Console.Write(Catalog.GetString("URL for create sitemap (with http://):"));
				string url = Console.ReadLine();
				//Check correctness of inserted url
				if (!System.Text.RegularExpressions.Regex.IsMatch(url, 
				                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
				                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
				{
					Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
					                  Catalog.GetString("Invalid url") + "]");
					return;
				}
				//Downloading page
				Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
				                  Catalog.GetString("Downloading page") + "]");
				file = filepath + guid.ToString() + ".arc";
				string[] hostname = url.Split('/');
				Crawler grubng = new Crawler();
				grubng.SetPath();
				System.Collections.Generic.List<string> headers = new System.Collections.Generic.List<string>();
				headers.Add("User-Agent: GrubNG");
				headers.Add("Accept: text/html");
				FileStream workunit = File.Create(file);
				FileStream tmpsitemap = File.Create(file + ".tmp");
				int[] httpresult = grubng.Crawling2(url, hostname[2], ref workunit, "HTTP/1.0", headers, ref tmpsitemap, true);
				workunit.Close();
				workunit = null;
				tmpsitemap.Close();
				tmpsitemap = null;
				grubng = null;
				System.IO.File.Delete(file);
				if (httpresult[0] != 200)
				{
					Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
						Catalog.GetString("Error during downloading page. HTTP answer code:") + " " + 
							httpresult[0].ToString() + "]");
					return;
				}
				//Generate sitemap from temporary file
				Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
				                  Catalog.GetString("Generate sitemap") + "]");
				file = filepath + guid.ToString() + ".arc.xml";
			}
			Sitemap sitemap = new Sitemap(file);
			if (answer == '3')
			{
				sitemap.CreateSitemap(false);
			}
			//Editing created sitemap
			extension = file.Split('.');
			index = extension.Length - 1;
			if (extension[index] == "gz")
			{
				FileStream filestr = File.OpenRead(file);
				file = file.Substring(0, (file.Length - 3));
				byte[] buffer = new byte[1024];
				FileStream unfile = File.Create(file);
				System.IO.Compression.GZipStream gzstream = new System.IO.Compression.GZipStream(filestr,
				                                                                                 System.IO.Compression.CompressionMode.Decompress);
				int bytesRead = 0;
				while (true)
				{
					bytesRead = gzstream.Read(buffer, 0, buffer.Length);
					if (bytesRead == 0)
					{
						break;
					}
					unfile.Write(buffer, 0, bytesRead);
				}
				gzstream.Close();
				gzstream.Dispose();;
				buffer = null;
				unfile.Close();
				unfile.Dispose();
				filestr.Close();
				filestr.Dispose();
				File.Delete(file + ".gz");
			}
			int curindex = 0;
			int amount = 0;
			System.Collections.Generic.List<string> urllist = new System.Collections.Generic.List<string>();
			if (answer != '1')
			{
				XmlDocument doc = new XmlDocument();
				doc.Load(file);
				XmlNode node = doc.SelectSingleNode("/");
				XmlElement urlmap = (XmlElement)node.ChildNodes[1];
				amount = urlmap.GetElementsByTagName("url").Count;
				for (int i = 0; i < amount; i++)
				{
					urllist.Add(urlmap.GetElementsByTagName("url")[i].FirstChild.InnerText);
				}
				doc = null;
			}
			string answer2 = "t";
			int linknumber;
			int maxlinks;
			while (answer2 != "s")
			{
				//Show 10 links from sitemap
				if (curindex < amount)
				{
					if ((curindex + 10) > amount)
					{
						maxlinks = amount;
					}
					else
					{
						maxlinks = curindex + 10;
					}
					Console.WriteLine();
					for (int i = curindex; i < maxlinks; i++)
					{
						Console.WriteLine((i + 1).ToString() + ")" + urllist[i]);
					}
				}
				Console.WriteLine();
				Console.WriteLine(Catalog.GetString("For edit or delete, enter number of selected link"));
				Console.WriteLine((amount + 1).ToString() + ")" + Catalog.GetString("Add new link"));
				if (curindex > 9)
				{
					Console.WriteLine("P)" + Catalog.GetString("Previous 10 links"));
				}
				if (curindex < amount)
				{
					Console.WriteLine("N)" + Catalog.GetString("Next 10 links"));
				}
				Console.WriteLine("S)" + Catalog.GetString("Send sitemap on server"));
				Console.WriteLine("A)" + Catalog.GetString("Save sitemap"));
				Console.WriteLine("Q)" + Catalog.GetString("Quit"));
				answer2 = Console.ReadLine();
				try
				{
					linknumber = Convert.ToInt32(answer2);
				}
				catch (FormatException)
				{
					linknumber = -1;
				}
				linknumber --;
				//Edit selected link
				if ((linknumber > -1) && (linknumber <= amount))
				{
					Console.WriteLine(Catalog.GetString("Enter new link (with http://). If you want delete this link, type:") + " " +
						"delete " + 
							Catalog.GetString("Hit Enter if you don't want change this link."));
					if (linknumber < amount)
					{
						Console.WriteLine(Catalog.GetString("Current link:") + " " + urllist[linknumber]);
					}
					else
					{
						Console.WriteLine(Catalog.GetString("New link"));
					}
					answer2 = Console.ReadLine();
					if (!String.IsNullOrEmpty(answer2))
					{
						if (answer2 != "delete")
						{
							if (!System.Text.RegularExpressions.Regex.IsMatch(answer2, 
							                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
							                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
							{
								Console.WriteLine("Invalid url");
							}
							else
							{
								if (linknumber < amount)
								{
									urllist[linknumber] = answer2;
								}
								else
								{
									urllist.Add(answer2);
									amount ++;
								}
							}
						}
						else
						{
							urllist.RemoveAt(linknumber);
							amount --;
						}
					}
				}
				//Save sitemap
				if ((answer2 == "A") || (answer2 == "a"))
				{
					string tmpmap = file.Substring(0, (file.Length - 3)) + "tmp";
					FileStream filestr = File.Create(tmpmap);
					StreamWriter mapwriter = new StreamWriter(filestr);
					foreach (string siteurl in urllist)
					{
						mapwriter.WriteLine(siteurl);
					}
					mapwriter.Dispose();
					filestr.Dispose();
					File.Delete(file);
					sitemap.CreateSitemap(false);
				}
				//Set previous 10 links from sitemap to show
				if ((answer2 == "P") || (answer2 == "p"))
				{
					curindex -= 10;
					if (curindex < 0)
					{
						curindex = 0;
					}
				}
				//Set next 10 links from sitemap to show
				if ((answer2 == "N") || (answer2 == "n"))
				{
					curindex += 10;
					if (curindex >= amount)
					{
						curindex = amount;
					}
				}
				//Set send sitemap on server
				if (answer2 == "S")
				{
					if (amount == 0)
					{
						Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
						                  Catalog.GetString("You can't send empty sitemap file") + "]");
						return;
					}
					answer2 = "s";
				}
				//Quit from editing sitemap
				if ((answer2 == "Q") || (answer2 == "q"))
				{
					return;
				}
			}
			//Sending sitemap on server
			Console.WriteLine(Catalog.GetString("[INFO:") + " " + Catalog.GetString("Checking propriety of sitemap file") + "]");
			bool valid = sitemap.CheckSitemap();
			if (!valid)
			{
				Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
				                  Catalog.GetString("Invalid sitemap file") + "]");
				return;
			}
			Console.WriteLine(Catalog.GetString("[INFO:") + " " + Catalog.GetString("Uploading") + "]");
			int[] status = sitemap.UploadSitemap(false);
			if (status[0] == 417)
			{
				status = sitemap.UploadSitemap(true);
			}
			sitemap = null;
			string message = Catalog.GetString("[INFO:") + " ";
			switch (status[0])
			{
			case 0:
				message += Catalog.GetString("Upload fail. Reason: Cannot connect to server.");
				break;
			case 200:
				message += Catalog.GetString("Sitemap uploaded.");
				break;
			case 401:
				message += Catalog.GetString("Upload fail. Reason: Invalid login.");
				break;
			case 408:
				message += Catalog.GetString("Upload fail. Reason: Connection Timeout.");
				break;
			case 500:
				message += Catalog.GetString("Upload fail. Reason: Internal Server Error.");
				break;
			case 503:
				message += Catalog.GetString("Upload fail. Reason: Cannot connect to server.");
				break;
			default:
				message += status.ToString();
				break;
			}
			message += "]";
			Console.WriteLine(message);
		}
		
		/// <summary>
		/// Function update configuration file or quit from program. Raised when program received message from
		/// web interface.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="WebEventArgs"/> type of even, "quit" or "config"
		/// </param>
		void OnWebMessage(object sender, WebEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args.Type == "quit")
			{
				//FIXME: not working properly
				Console.WriteLine("q");
			}
			if (args.Type == "config")
			{
				this.workthread.SetPrefs(false);
			}
		}
		
		/// <summary>
		/// Function dispose unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			if (!disposedres)
			{
				this.worktime.Dispose();
				this.workthread.Dispose();
				if (this.rControl != null)
				{
					this.rControl.Dispose();
				}
				disposedres = true;
			}
		}
	}
}
