// ConsolePref.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file ConsolePref.cs
 * Provide console functions for set program preferences (Unix version).*/

using System;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Provide console functions for set program preferences.
	/// </summary>
	internal sealed class ConsolePref
	{
		/// <summary>
		/// Object from Config class. 
		/// Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Grub username.
		/// </summary>
		string username;
		/// <summary>
		/// Grub password.
		/// </summary>
		string password;
		/// <summary>
		/// Program working directory.
		/// </summary>
		string workdir;
		/// <summary>
		/// Address of server with workunits.
		/// </summary>
		string workserv;
		/// <summary>
		/// Amount of working crawlers.
		/// </summary>
		string crawlers;
		/// <summary>
		/// Proxy status.
		/// </summary>
		string proxystatus;
		/// <summary>
		/// Address of proxy server.
		/// </summary>
		string proxyaddress;
		/// <summary>
		/// Proxy server port
		/// </summary>
		string proxyport;
		/// <summary>
		/// Proxy server login
		/// </summary>
		string proxylogin;
		/// <summary>
		/// Proxy password
		/// </summary>
		string proxypass;
		/// <summary>
		/// Run program in auto mode
		/// </summary>
		string auto;
		/// <summary>
		/// Start program minimized
		/// </summary>
		string runhidden;
		/// <summary>
		/// Clear stats
		/// </summary>
		char clearstats;
		/// <summary>
		/// Address of server with updates
		/// </summary>
		string updateserver;
		/// <summary>
		/// Checking for updates
		/// </summary>
		string checkupdates;
		/// <summary>
		/// Max download speed
		/// </summary>
		string downloadspeed;
		/// <summary>
		/// Max upload speed
		/// </summary>
		string uploadspeed;
		/// <summary>
		/// Address of server with sitemaps
		/// </summary>
		string mapserver;
		/// <summary>
		/// Delete sitemaps after upload
		/// </summary>
		string deletesitemap;
		/// <summary>
		/// Compression method
		/// </summary>
		string compression;
		/// <summary>
		/// Enable/disable creating sitemaps during crawling
		/// </summary>
		string createsitemaps;
		/// <summary>
		/// Enable/disable remote control of this crawler
		/// </summary>
		string enableremote;
		/// <summary>
		/// Remote control port
		/// </summary>
		string remoteport;
		/// <summary>
		/// Check for unstable updates of client
		/// </summary>
		string unstable;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public ConsolePref()
		{
			this.conf = new Config();
			string[] nodes = new string[] {
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/workdir",
				"/configuration/work/workserver",
				"/configuration/work/crawlers",
				"/configuration/work/proxystatus",
				"/configuration/work/proxyaddress",
				"/configuration/work/proxyport",
				"/configuration/work/proxylogin",
				"/configuration/work/proxypass",
				"/configuration/work/automatic",
				"/configuration/other/runhidden",
				"/configuration/other/updateserver",
				"/configuration/other/checkupdates",
				"/configuration/work/downloadrate",
				"/configuration/work/uploadrate",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"/configuration/work/createsitemaps",
				"/configuration/work/compression",
				"/configuration/other/enableremote",
				"/configuration/other/remoteport",
				"/configuration/other/unstable"
			};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			this.username = values[0];
			this.password = values[1];
			this.workdir = values[2];
			this.workserv = values[3];
			this.crawlers = values[4];
			this.proxystatus = values[5];
			this.proxyaddress = values[6];
			this.proxyport = values[7];
			this.proxylogin = values[8];
			this.proxypass = values[9];
			this.auto = values[10];
			this.runhidden = values[11];
			this.clearstats = 'N';
			this.updateserver = values[12];
			this.checkupdates = values[13];
			this.downloadspeed = values[14];
			this.uploadspeed = values[15];
			this.mapserver = values[16];
			this.deletesitemap = values[17];
			this.createsitemaps = values[18];
			if (values[19] == ".gz")
			{
				this.compression = "GZip";
			}
			else
			{
				this.compression = "LZMA";
			}
			this.enableremote = values[20];
			this.remoteport = values[21];
			this.unstable = values[22];
			nodes = null;
			values = null;
		}
		
		/// <summary>
		/// Show preferences menu to user.
		/// </summary>
		public void Start()
		{
			char answer = 'a';
			char answer2 = 'a';
			while ((answer != 'q') && (answer != 'Q'))
			{
				Console.WriteLine("1)" + Catalog.GetString("Login"));
				Console.WriteLine("2)" + Catalog.GetString("Directories"));
				Console.WriteLine("3)" + Catalog.GetString("Net"));
				Console.WriteLine("4)" + Catalog.GetString("Proxy"));
				Console.WriteLine("5)" + Catalog.GetString("Misc"));
				Console.WriteLine("6)" + Catalog.GetString("Updater"));
				Console.WriteLine("7)" + Catalog.GetString("Sitemaps"));
				Console.WriteLine("A)" + Catalog.GetString("Save"));
				Console.WriteLine("S)" + Catalog.GetString("Save and quit"));
				Console.WriteLine("Q)" + Catalog.GetString("Quit"));
				answer = (char)Console.Read();
				Console.Clear();
				//Show preferences about login
				if (answer == '1')
				{
					string[] options = {Catalog.GetString("Username:"), Catalog.GetString("Password:")};
					string[] values = {this.username, this.password};
					answer2 = ConsolePref.ShowOptions(options, values);
					switch (answer2)
					{
						//Edit username
					case '1':
						this.username = ConsolePref.EditOptions(Catalog.GetString("Username:"), this.username, "text", 0, 0);
						break;
						//Edit password
					case '2':
						this.password = ConsolePref.EditOptions(Catalog.GetString("Password:"), this.password, "text", 0, 0);
						break;
					default:
						break;
					}
				}
				//Show preferences about directories
				if (answer == '2')
				{
					string[] options = {Catalog.GetString("Work directory:")};
					string[] values = {this.workdir};
					answer2 = ConsolePref.ShowOptions(options, values);
					//Edit work directory
					if (answer2 == '1')
					{
						this.workdir = ConsolePref.EditOptions(Catalog.GetString("Work directory:"), this.workdir, "text", 0, 0);
					}
				}
				//Show preferences about network
				if (answer == '3')
				{
					string[] options = {Catalog.GetString("Workunit server:"), 
						Catalog.GetString("Working crawlers:"),
						Catalog.GetString("Max download speed (in KB/s):"),
						Catalog.GetString("Max upload speed (in KB/s):"),
						Catalog.GetString("Compression method:"),
						Catalog.GetString("Create sitemaps during crawling:")};
					string[] values = {this.workserv, this.crawlers, this.downloadspeed, this.uploadspeed, this.compression, 
						this.createsitemaps};
					if (this.createsitemaps == "Y")
					{
						values[5] = Catalog.GetString("Yes");
					}
					else
					{
						values[5] = Catalog.GetString("No");
					}
					answer2 = ConsolePref.ShowOptions(options, values);
					bool option;
					switch (answer2)
					{
						//Edit server with workunits address
					case '1':
						this.workserv = ConsolePref.EditOptions(Catalog.GetString("Workunit server:"), this.workserv, "url", 0, 0);
						break;
						//Edit amount of working crawlers
					case '2':
						this.crawlers = ConsolePref.EditOptions(Catalog.GetString("Working crawlers:"), this.crawlers, "number", 1, 50);
						break;
						//Edit max download speed
					case '3':
						this.downloadspeed = ConsolePref.EditOptions(Catalog.GetString("Max download speed (in KB/s):"), this.downloadspeed, "number", 4, 512);
						break;
						//Edit max upload speed
					case '4':
						this.uploadspeed = ConsolePref.EditOptions(Catalog.GetString("Max upload speed (in KB/s):"), this.uploadspeed, "number", 4, 512);
						break;
						//Edit compression method
					case '5':
						option = ConsolePref.SetOptions(Catalog.GetString("Compression method (GZip or LZMA):"));
						if (option)
						{
							if (this.compression == "GZip")
							{
								this.compression = "LZMA";
							}
							else
							{
								this.compression = "GZip";
							}
						}
						break;
						//Enable/disable creating sitemaps during crawling
					case '6':
						option = ConsolePref.SetOptions(Catalog.GetString("Create sitemaps during crawling:"));
						if (option)
						{
							if (this.createsitemaps == "Y")
							{
								this.createsitemaps = "N";
							}
							else
							{
								this.createsitemaps = "Y";
							}
						}
						break;
					default:
						break;
					} 
				}
				//Show preferences about proxy server
				if (answer == '4')
				{
					string[] options = {Catalog.GetString("Direct connect to internet"),
						Catalog.GetString("Use system default proxy settings"),
						Catalog.GetString("Set custom proxy preferences"),
						Catalog.GetString("HTTP proxy server:"),
						Catalog.GetString("Port:"),
						Catalog.GetString("Login:"),
						Catalog.GetString("Password:")};
					string[] values = new string[7];
					if (this.proxystatus == "none")
					{
						values = new string[7]{Catalog.GetString("Yes"), 
							Catalog.GetString("No"), 
							Catalog.GetString("No"), 
							String.Empty, 
							String.Empty,
							String.Empty,
							String.Empty};
					}
					if (this.proxystatus == "default")
					{
						values = new string[7]{Catalog.GetString("No"), 
							Catalog.GetString("Yes"), 
							Catalog.GetString("No"), 
							String.Empty, 
							String.Empty,
							String.Empty,
							String.Empty};
					}
					if (this.proxystatus == "custom")
					{
						values = new string[7]{Catalog.GetString("No"), 
							Catalog.GetString("No"), 
							Catalog.GetString("Yes"), 
							this.proxyaddress, 
							this.proxyport,
							this.proxylogin,
							this.proxypass};
					}
					answer2 = ConsolePref.ShowOptions(options, values);
					bool option;
					switch (answer2)
					{
						//Set direct connect to net
					case '1':
						option = ConsolePref.SetOptions(Catalog.GetString("Direct connect to internet"));
						if (option)
						{
							this.proxystatus = "none";
						}
						break;
						//Set system default proxy settings
					case '2':
						option = ConsolePref.SetOptions(Catalog.GetString("Use system default proxy settings"));
						if (option)
						{
							this.proxystatus = "default";
						}
						break;
						//Set custom settings for proxy
					case '3':
						option = ConsolePref.SetOptions(Catalog.GetString("Set custom proxy preferences"));
						if (option)
						{
							this.proxystatus = "custom";
						}
						break;
						//Set proxy address
					case '4':
						this.proxyaddress = ConsolePref.EditOptions(Catalog.GetString("HTTP proxy server:"), this.proxyaddress, "url", 0, 0);
						break;
						//Set proxy port
					case '5':
						this.proxyport = ConsolePref.EditOptions(Catalog.GetString("Port:"), proxyport, "number", 1, 10000);
						break;
						//Set proxy login
					case '6':
						this.proxylogin = ConsolePref.EditOptions(Catalog.GetString("Login:"), this.proxylogin, "text", 0, 0);
						break;
						//Set proxy password
					case '7':
						this.proxypass = ConsolePref.EditOptions(Catalog.GetString("Password:"), this.proxylogin, "text", 0, 0);
						break;
					default:
						break;
					}
				}
				//Show miscellaneous preferences
				if (answer == '5')
				{
					string[] options = {
						Catalog.GetString("Run program in automatic mode"),
						Catalog.GetString("Run program minimized"),
						Catalog.GetString("Clear stats"),
						Catalog.GetString("Enable remote control"),
						Catalog.GetString("Remote control port:")
					};
					string[] values = new string[options.Length];
					if (this.auto == "Y")
					{
						values[0] = Catalog.GetString("Yes");
					}
					else
					{
						values[0] = Catalog.GetString("No");
					}
					if (this.runhidden == "Y")
					{
						values[1] = Catalog.GetString("Yes");
					}
					else
					{
						values[2] = Catalog.GetString("No");
					}
					if (this.clearstats == 'N')
					{
						values[2] = Catalog.GetString("No");
					}
					else
					{
						values[2] = Catalog.GetString("Yes");
					}
					if (this.enableremote == "Y")
					{
						values[3] = Catalog.GetString("Yes");
					}
					else
					{
						values[3] = Catalog.GetString("No");
					}
					values[4] = this.remoteport;
					answer2 = ConsolePref.ShowOptions(options, values);
					bool option;
					switch (answer2)
					{
						//Start program in automatic mode
					case '1':
						option = ConsolePref.SetOptions(Catalog.GetString("Run program in automatic mode"));
						if (option)
						{
							if (this.auto == "Y")
							{
								this.auto = "N";
							}
							else
							{
								this.auto = "Y";
							}
						}
						break;
						//Run program minimized
					case '2':
						option = ConsolePref.SetOptions(Catalog.GetString("Run program minimized"));
						if (option)
						{
							if (this.runhidden == "Y")
							{
								this.runhidden = "N";
							}
							else
							{
								this.runhidden = "Y";
							}
						}
						break;
						//Clear program stats
					case '3':
						option = ConsolePref.SetOptions(Catalog.GetString("Run program in automatic mode"));
						if (option)
						{
							if (this.clearstats == 'Y')
							{
								this.clearstats = 'N';
							}
							else
							{
								this.clearstats = 'Y';
							}
						}
						break;
						//Enable/disable remote control
					case '4':
						option = ConsolePref.SetOptions(Catalog.GetString("Enable remote control"));
						if (option)
						{
							if (this.enableremote == "Y")
							{
								this.enableremote = "N";
							}
							else
							{
								this.enableremote = "Y";
							}
						}
						break;
						//Set remote control port
					case '5':
						this.remoteport = ConsolePref.EditOptions(Catalog.GetString("Remote control port:"), this.remoteport, "number", 1024, 65535);
						break;
					default:
						break;
					}
				}
				//Show autoupdater preferences
				if (answer == '6')
				{
					System.Collections.Generic.List<string> values = new System.Collections.Generic.List<string>();
					System.Collections.Generic.List<string> options = new System.Collections.Generic.List<string>();
					options.Add(Catalog.GetString("Server with updates:"));
					options.Add(Catalog.GetString("Enable Autoupdater"));
					values.Add(this.updateserver);
					if (this.checkupdates == "N")
					{
						values.Add(Catalog.GetString("No"));
					}
					else
					{
						values.Add(Catalog.GetString("Yes"));
						options.Add(Catalog.GetString("Check for"));
						if (this.unstable == "N")
						{
							values.Add(Catalog.GetString("Stable version"));
						}
						else
						{
							values.Add(Catalog.GetString("Unstable version"));
						}
					}
					answer2 = ConsolePref.ShowOptions(options.ToArray(), values.ToArray());
					bool option;
					switch (answer2)
					{
						//Set address for server with updates
					case '1':
						this.updateserver = ConsolePref.EditOptions(Catalog.GetString("Server with updates:"), this.updateserver, "url", 0, 0);
						break;
						//Enable/Disable autoupdater
					case '2':
						option = ConsolePref.SetOptions(Catalog.GetString("Enable Autoupdater"));
						if (option)
						{
							if (this.checkupdates == "Y")
							{
								this.checkupdates = "N";
							}
							else
							{
								this.checkupdates = "Y";
							}
						}
						break;
						//Check for stable/unstable versions of program
					case '3':
						option = ConsolePref.SetOptions(Catalog.GetString("Check for stable/unstable"));
						if (option)
						{
							if (this.unstable == "Y")
							{
								this.unstable = "N";
							}
							else
							{
								this.unstable = "Y";
							}
						}
						break;
					default:
						break;				
					}
				}
				//Show upload sitemaps preferences
				if (answer == '7')
				{
					string[] options = {Catalog.GetString("Sitemaps server:"),
						Catalog.GetString("Delete sitemap after upload")};
					string[] values = new string[2];
					values[0] = this.mapserver;
					if (this.deletesitemap == "N")
					{
						values[1] = Catalog.GetString("No");
					}
					else
					{
						values[1] = Catalog.GetString("Yes");
					}
					answer2 = ConsolePref.ShowOptions(options, values);
					bool option;
					switch (answer2)
					{
						//Set address for server with sitemaps
					case '1':
						this.mapserver = ConsolePref.EditOptions(Catalog.GetString("Sitemaps server:"), this.mapserver, "url", 0, 0);
						break;
						//Enable/Disable delete sitemap after upload
					case '2':
						option = ConsolePref.SetOptions(Catalog.GetString("Delete sitemap after upload"));
						if (option)
						{
							if (this.deletesitemap == "Y")
							{
								this.deletesitemap = "N";
							}
							else
							{
								this.deletesitemap = "Y";
							}
						}
						break;
					default:
						break;				
					}
				}
				//Save configuration to grub.xml file
				if ((answer == 'A') || (answer == 'a'))
				{
					this.SavePrefs();
				}
				//Save configuration to grub.xml file and exit
				if ((answer == 'S') || (answer == 's'))
				{
					this.SavePrefs();
					answer = 'q';
				}
			}
		}
		
		/// <summary>
		/// Show options available in this preferences submenu.
		/// </summary>
		/// <param name="options">
		/// A <see cref="System.String"/> array with options name
		/// </param>
		/// <param name="values">
		/// A <see cref="System.String"/> array with options value
		/// </param>
		/// <returns>
		/// A <see cref="System.Char"/> key pressed by user
		/// </returns>
		static char ShowOptions(string[] options, string[] values)
		{
			for (int i = 0; i < options.Length; i++)
			{
				Console.WriteLine("{0}){1} {2}", (i + 1).ToString(), options[i], values[i]);
			}
			Console.Write(Catalog.GetString("Select option to edit or press Q for back to preferences menu."));
			char answer = (char)Console.Read();
			Console.WriteLine();
			return answer;
		}
		
		/// <summary>
		/// Function to edit selected option. Show option to edit and check for empty fields.
		/// </summary>
		/// <param name="option">
		/// A <see cref="System.String"/> option name
		/// </param>
		/// <param name="val">
		/// A <see cref="System.String"/> option value
		/// </param>
		/// <param name="type">
		/// A <see cref="System.String"/> option value type (text, url, number)
		/// </param>
		/// <param name="startrange">
		/// A <see cref="System.Int32"/> minimal number for value (only for number values, other set 0 here)
		/// </param>
		/// <param name="endrange">
		/// A <see cref="System.Int32"/> maximum number for value (only for number values, other set 0 here)
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> if option changed, return new value, if not, return old value
		/// </returns>
		static string EditOptions(string option, string val, string type, int startrange, int endrange)
		{
			Console.Write(Catalog.GetString("New value of") + " " + option + " ");
			string answer = Console.ReadLine();
			//Check if string is empty
			if (String.IsNullOrEmpty(answer))
			{
				Console.WriteLine(Catalog.GetString("Fill empty fields"));
				return val;
			}
			//Check for valid url
			if (type == "url")
			{
				System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^(ht|f)tp(s?)\:\/\/[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$");
				if (!rx.IsMatch(answer))
				{
					Console.WriteLine(Catalog.GetString("Invalid url"));
					return val;
				}
			}
			//Check for numbers
			if (type == "number")
			{
				System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^[1-9][0-9]*$");
				if (!rx.IsMatch(answer))
				{
					Console.WriteLine(Catalog.GetString("Value must be a number."));
					return val;
				}
				if (Convert.ToInt32(answer) < startrange)
				{
					Console.WriteLine(Catalog.GetString("Value too small."));
					return val;
				}
				if (Convert.ToInt32(answer) > endrange)
				{
					Console.WriteLine(Catalog.GetString("Value too big."));
					return val;
				}
			}
			return answer;
		}
			
		/// <summary>
		/// Function to switch between options (similar to GUI radiobutton).
		/// </summary>
		/// <param name="option">
		/// A <see cref="System.String"/> option name
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if option change status, otherwise false
		/// </returns>
		static bool SetOptions(string option)
		{
			Console.WriteLine(Catalog.GetString("You want change setting:") + " " + 
			                  option + " [y/n]");
			char answer = (char)Console.Read();
			Console.WriteLine();
			if ((answer == 'y') || (answer == 'Y'))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		/// <summary>
		/// Function save all preferences to grub.xml file.
		/// </summary>
		void SavePrefs()
		{
			Console.Write(Catalog.GetString("Saving program preferences..."));
			string[] xmlnodes = new string[] {
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/workdir",
				"/configuration/work/workserver",
				"/configuration/work/crawlers",
				"/configuration/work/downloadrate",
				"/configuration/work/uploadrate",
				"/configuration/work/compression",
				"/configuration/work/proxystatus",
				"/configuration/work/proxyaddress",
				"/configuration/work/proxyport",
				"/configuration/work/proxylogin",
				"/configuration/work/proxypass",
				"/configuration/work/automatic",
				"/configuration/other/runhidden",
				"/configuration/other/updateserver",
				"/configuration/other/checkupdates",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"/configuration/work/createsitemaps",
				"/configuration/other/enableremote",
				"/configuration/other/remoteport",
				"/configuration/other/unstable"
			};
			string[] xmlvalues = new string[xmlnodes.Length];
			xmlvalues[0] = this.username;
			xmlvalues[1] = this.password;
			xmlvalues[2] = this.workdir;
			xmlvalues[3] = this.workserv;
			xmlvalues[4] = this.crawlers;
			xmlvalues[5] = this.downloadspeed;
			xmlvalues[6] = this.uploadspeed;
			if (this.compression == "GZip")
			{
				xmlvalues[7] = ".gz";
			}
			else
			{
				xmlvalues[7] = ".lzma";
			}
			xmlvalues[8] = this.proxystatus;
			if ((this.proxystatus == "none") || (this.proxystatus == "default"))
			{
				xmlvalues[9] = String.Empty;
				xmlvalues[10] = String.Empty;
				xmlvalues[11] = String.Empty;
				xmlvalues[12] = String.Empty;
			}
			else
			{
				xmlvalues[9] = this.proxyaddress;
				xmlvalues[10] = this.proxyport;
				xmlvalues[11] = this.proxylogin;
				xmlvalues[12] = this.proxypass;
			}
			xmlvalues[13] = this.auto;
			xmlvalues[14] = this.runhidden;
			xmlvalues[15] = this.updateserver;
			xmlvalues[16] = this.checkupdates;
			xmlvalues[17] = this.mapserver;
			xmlvalues[18] = this.deletesitemap;
			xmlvalues[19] = this.createsitemaps;
			xmlvalues[20] = this.enableremote;
			xmlvalues[21] = this.remoteport;
			xmlvalues[22] = this.unstable;
			Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
			if (this.clearstats == 'Y')
			{
				xmlnodes = new string[] {
					"/configuration/statistics/worksend",
					"/configuration/statistics/urlcrawled",
					"/configuration/statistics/success",
					"/configuration/statistics/failure",
					"/configuration/statistics/errors",
					"/configuration/statistics/mbreceived",
					"/configuration/statistics/mbsend",
					"/configuration/statistics/invalidarc"
				};
				xmlvalues = new string[] {"0", "0", "0", "0", "0", "0", "0", "0"};
				Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
			}
			Console.WriteLine(Catalog.GetString("Done"));
		}
	}
}
