// Daemon.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Daemon.cs
 * Provide main daemon functions.*/

using System;
using System.Timers;

namespace Grubng
{
	
	/// <summary>
	/// Provide main daemon functions.
	/// </summary>
	internal sealed class Daemon : IDisposable
	{
		
		/// <summary>
		/// Object from threads class. Provide functions to upload, crawl and upload workunits.
		/// </summary>
		Threads workthread;
		/// <summary>
		/// Object from Config class. Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Object from RemoteControl class. Provide functions for web interface.
		/// </summary>
		RemoteControl rControl;
		/// <summary>
		/// Workunit time timer
		/// </summary>
		Timer worktime;
		/// <summary>
		/// Program uptime
		/// </summary>
		TimeSpan uptimespan;
		/// <summary>
		/// Amount of working crawlers
		/// </summary>
		int working;
		/// <summary>
		/// Check if resources are disposed.
		/// </summary>
		bool disposedres;
		/// <summary>
		/// Amount of connection errors which appear in this session.
		/// </summary>
		int connectionerrors;
		/// <summary>
		/// Amount of seconds which program must wait before connect again to servers.
		/// </summary>
		int connectiondelay;
		/// <summary>
		/// If true, program works, otherwise close program
		/// </summary>
		bool works;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public Daemon()
		{
			this.conf = new Config();
			if (!System.IO.File.Exists(this.conf.FilePath))
			{
				return;
			}
			MainClass.CreatePIDFile();
			this.connectionerrors = 0;
			this.connectiondelay = 0;
			this.workthread = new Threads();
			this.worktime = new System.Timers.Timer();
			this.worktime.Elapsed += new ElapsedEventHandler(OnWorkElapsed);
			this.worktime.Interval = 1000;
		}
		
		/// <summary>
		/// Function start crawling.
		/// </summary>
		public void Start()
		{
			if (disposedres)
			{
				throw new ObjectDisposedException(GetType().Name);
			}
			this.workthread.Running = true;
			this.workthread.Start();
			if (Config.ReadConfig("/configuration/other/enableremote", this.conf.FilePath) == "Y")
			{
				this.rControl = new RemoteControl(this.workthread);
				this.rControl.Start();
				this.rControl.WebEventHandler += new EventHandler<WebEventArgs>(OnWebMessage);
			}
			else
			{
				this.Stop();
				return;
			}
			this.worktime.Start();
			this.works = true;
			while (works)
			{
				System.Threading.Thread.Sleep(1000);
			}
			this.Stop();
		}
		
		/// <summary>
		/// Function stop crawling and quit from program.
		/// </summary>
		public void Stop()
		{
			if (disposedres)
			{
				throw new ObjectDisposedException(GetType().Name);
			}
			this.worktime.Stop();
			this.worktime.Dispose();
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				this.workthread.SetField(i, "working", 0);
			}
			int quitdelay = 25;
			while (!this.workthread.CrawlersStopped())
			{
				System.Threading.Thread.Sleep(1000);
				quitdelay --;
				if (quitdelay == 0)
				{
					this.workthread.WriteStats();
					MainClass.DeletePIDFile();
					System.Diagnostics.Process proc = System.Diagnostics.Process.GetCurrentProcess();
					proc.Kill();
				}
			}
			this.workthread.WriteStats();
		}
		
		/// <summary>
		/// Timer for check crawlers status, check internet connection. Raised every second when program works.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="ElapsedEventArgs"/> unused
		/// </param>
		void OnWorkElapsed(object sender, ElapsedEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			TimeSpan tmpspan = new TimeSpan(0, 0, 1);
			this.uptimespan += tmpspan;
			int stoped = 0;
			this.working = 0;
			int paused = 0;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{		
				switch (this.workthread.GetField(i, "working"))
				{
				case 0:
					stoped++;
					break;
				case 1:
					this.working++;
					break;
				case 2:
					paused++;
					break;
				}
			}
			if (paused != this.workthread.Crawlers)
			{
				this.CheckNet();
			}
			else
			{
				for (int i = 0; i < this.workthread.Crawlers; i++)
				{
					this.workthread.SetField(i, "working", 0);
				}
				this.workthread.Running = false;
			}
			if (this.connectiondelay > 0)
			{
				this.connectiondelay--;
			}
			else
			{
				if (this.workthread.ConnectionError)
				{
					this.connectionerrors ++;
					this.connectiondelay = 30 * this.connectionerrors;
					this.workthread.ConnectionError = false;
					this.workthread.Running = false;
				}
				else
				{
					if (!this.workthread.Running)
					{
						this.workthread.Running = true;
						this.workthread.Start();
					}
				}
			}
		}
		
		/// <summary>
		/// Function check internet connection. If there is more than 49 request timeout in row, function ping net
		/// for check internet connection. If fail, pause all crawlers.
		/// </summary>
		void CheckNet()
		{
			bool check = true;
			if (this.workthread.TimeOut > 49)
			{
				System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.WebRequest.Create("http://grub.org/");
				request.Timeout = 5000;
				request.ProtocolVersion = System.Net.HttpVersion.Version11;
				request.UserAgent = "GrubNG";
				request.KeepAlive = false;
				request.Proxy = null;
				request.AllowAutoRedirect = false;
				try
				{
					System.Net.HttpWebResponse response = (System.Net.HttpWebResponse)request.GetResponse();
					response.Close();
				}
				catch (System.Net.WebException e)
				{
					int error = (int)e.Status;
					if (error == 1)
					{
						check = false;
						for (int i = 0; i < this.workthread.Crawlers; i++)
						{
							this.workthread.SetField(i, "working", 2);
						}
					}
				}
				if (check)
				{
					this.workthread.TimeOut = 0;
				}
			}
		}
		
		/// <summary>
		/// Function update configuration file or quit from program. Raised when program received message from
		/// web interface.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="WebEventArgs"/> type of even, "quit" or "config"
		/// </param>
		void OnWebMessage(object sender, WebEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args.Type == "quit")
			{
				this.works = false;
			}
			if (args.Type == "config")
			{
				this.workthread.SetPrefs(false);
			}
		}
		
		/// <summary>
		/// Function dispose unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			if (!disposedres)
			{
				this.worktime.Dispose();
				this.workthread.Dispose();
				this.rControl.Dispose();
				disposedres = true;
			}
		}
	}
}
