// MainWindow.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file MainWindow.cs
 * Provide GTK Window for program.
 * @fn virtual void Grubng::MainWindow::Build()
 * Create part of GUI and show it to user*/

using System;
using System.IO;
using System.Reflection;
using Gtk;
using Mono.Unix;

namespace Grubng 
{
	
	
	internal sealed partial class MainWindow : Gtk.Window 
	{
		
		/// <summary>
		/// Object from Config class. Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Object from threads class. Provide functions to upload, crawl and upload workunits.
		/// </summary>
		Threads workthread;
		/// <summary>
		/// Object from RemoteControl class. Provide functions for web interface.
		/// </summary>
		RemoteControl rControl;
		/// <summary>
		/// Program GTK Icon
		/// </summary>
		StatusIcon icon;
		/// <summary>
		/// Workunit time timer
		/// </summary>
		System.Timers.Timer worktime;
		/// <summary>
		/// Stop status image
		/// </summary>
		Gdk.Pixbuf stopimg;
		/// <summary>
		/// GTK TreeStore to store informations about crawlers
		/// </summary>
		TreeStore store;
		/// <summary>
		/// List of GTK TreeIter of crawlers
		/// </summary>
		System.Collections.Generic.List<TreeIter> iter1;
		/// <summary>
		/// Program uptime
		/// </summary>
		TimeSpan uptimespan;
		/// <summary>
		/// Work status image
		/// </summary>
		Gdk.Pixbuf workimg;
		/// <summary>
		/// List of current numbers of url in crawler
		/// </summary>
		System.Collections.Generic.List<int> curiter;
		/// <summary>
		/// Amount of working crawlers
		/// </summary>
		int workingcrawlers;
		/// <summary>
		/// Crawlers amount
		/// </summary>
		int crawlers;
		/// <summary>
		/// If true program start exit procedure
		/// </summary>
		bool quitprogram;
		/// <summary>
		/// Array of translated crawler statuses
		/// </summary>
		string[] crawlerstatus;
		/// <summary>
		/// Amount of connection errors which appear in this session.
		/// </summary>
		int connectionerrors;
		/// <summary>
		/// Amount of seconds which program must wait before connect again to servers.
		/// </summary>
		int connectiondelay;
		/// <summary>
		/// Amount of max working crawlers.
		/// </summary>
		int maxcrawlers;
		/// <summary>
		/// Max download rate.
		/// </summary>
		int maxdownload;
		/// <summary>
		/// Max upload rate.
		/// </summary>
		int maxupload;
		/// <summary>
		/// Pause status image
		/// </summary>
		Gdk.Pixbuf pauseimg;
		/// <summary>
		/// Amount of second of wait on quit from program.
		/// </summary>
		int quitdelay;
		
		/// <summary>
		/// Standard class constructor
		/// </summary>
		public MainWindow() : base(Gtk.WindowType.Toplevel) 
		{
			this.Build();
			this.BuildGUI();
			this.stopimg = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                     System.IO.Path.DirectorySeparatorChar + "icon-grub-err.jpg"));
			this.workimg = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                     System.IO.Path.DirectorySeparatorChar + "icon-grub-run.jpg"));
			this.pauseimg = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
				                                                              System.IO.Path.DirectorySeparatorChar + "icon-grub-pause.jpg"));
			this.imgstatus.Pixbuf = stopimg;
			this.icon = new StatusIcon(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                  System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.icon.Visible = true;
			this.quitdelay = 25;
			this.icon.Activate += new EventHandler(OnIconActivate);
			this.icon.PopupMenu += new PopupMenuHandler(OnIconMenu);
			this.conf = new Config();
			//Set values for options
			this.ReadOptions();
			this.notebook.CurrentPage = 2;
			this.crawlerstatus = new string[11]{Catalog.GetString("Stopped"),
				Catalog.GetString("Paused"),
				Catalog.GetString("In Queue"),
				Catalog.GetString("Downloading"),
				Catalog.GetString("Download Error"),
				Catalog.GetString("Crawling"),
				Catalog.GetString("Uploading"),
				Catalog.GetString("Upload Error"),
				Catalog.GetString("Waiting"),
				Catalog.GetString("Invalid arc file"),
				Catalog.GetString("Program error")
			};
			this.workthread = new Threads();
			this.lblsend2.Text = this.workthread.GetField(0, "overallworkunit").ToString();
			this.lblinvalidarc2.Text = this.workthread.GetField(0, "overallinvalidworkunit").ToString();
			this.lblurls2.Text = this.workthread.GetField(0, "overallcrawled").ToString();
			this.lblvalid2.Text = this.workthread.GetField(0, "overallvalid").ToString();
			this.lblinvalid2.Text = this.workthread.GetField(0, "overallinvalid").ToString();
			this.lblerrors2.Text = this.workthread.GetField(0, "overallerror").ToString();
			if (Config.ReadConfig("/configuration/other/enableremote", this.conf.FilePath) == "Y")
			{
				this.rControl = new RemoteControl(this.workthread);
				this.rControl.Start();
				this.rControl.WebEventHandler += new EventHandler<WebEventArgs>(OnWebMessage);
			}
			this.worktime = new System.Timers.Timer();
			this.worktime.Elapsed += new System.Timers.ElapsedEventHandler(OnWorkElapsed);
			this.worktime.Interval = 1000;
			this.worktime.Start();
			this.store = new TreeStore(typeof (int), typeof (string), typeof (int), typeof (string), 
			                           typeof (string), typeof (double));
			this.iter1 = new System.Collections.Generic.List<TreeIter>();
			this.curiter = new System.Collections.Generic.List<int>();
			//Check for amount of crawlers
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				this.iter1.Add(this.store.AppendValues((i + 1), Catalog.GetString("Stopped"), 
				                                       0, String.Empty, String.Empty, 0.0));
				this.curiter.Add(0);
			}
			this.treedet.Model = store;
			this.lblcurrent.Text = "0/" + this.workthread.Crawlers.ToString();
			this.crawlers = this.workthread.Crawlers;
			this.connectiondelay = 0;
			this.connectionerrors = 0;
			this.icon.Tooltip = Catalog.GetString("Working crawlers:") + " " + 
				"0/" + this.workthread.Crawlers.ToString() + Environment.NewLine +
					Catalog.GetString("Disk Usage (MB):") + " 0" + Environment.NewLine +
					Catalog.GetString("Workunits send:") + " 0" + Environment.NewLine +
					Catalog.GetString("Links crawled:") + " 0";
			//Check for automatic mode
			if (Config.ReadConfig("/configuration/work/automatic", this.conf.FilePath) == "Y")
			{
				this.cbtnauto.Active = true;
				this.workthread.Running = true;
				this.btnrun.Click();
			}
		}
		
		/// <summary>
		/// Raised when window is closed, hide window instead.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="a">
		/// A <see cref="DeleteEventArgs"/> unused
		/// </param>
		void OnDeleteEvent (object sender, DeleteEventArgs a)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (this.HideOnDelete())
			{
				a.RetVal = true;
			}
		}

		/// <summary>
		/// Raised when button close clicked, close all application.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnQuitActivated (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.workthread.WriteStats();
			this.quitprogram = true;
			this.GdkWindow.Cursor = new Gdk.Cursor(Gdk.CursorType.Watch);
		}

		/// <summary>
		/// Start or pause web crawling. Raised when button Run/Pause clicked. Function start new thread for crawling.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnBtnrunClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			bool stoped = false;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				if ((this.workthread.GetField(i, "working") == 0) || (this.workthread.GetField(i, "working") == 2))
				{
					stoped = true;
					break;
				}
			}
			if (stoped)
			{
				this.UpdateButton(true);
				this.workthread.WriteStats();
				this.workthread.TimeOut = 0;
				this.workthread.Start();
			}
			else
			{
				this.UpdateButton(false);
				for (int i = 0; i < this.workthread.Crawlers; i++)
				{
					this.workthread.SetField(i, "working", 2);
				}
			}
		}

		/// <summary>
		/// Show about dialog (information about program). Raised when About menu item selected.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnAboutActivated (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			AboutDialog about = new AboutDialog();
			int result = about.Run();
			if (result < 0)
			{
				about.Destroy();
				about.Dispose();
			}
		}
		
		/// <summary>
		/// Show or hide window by click mouse on tray icon.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnIconActivate(object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.Visible)
			{
				this.Hide();
			}
			else
			{
				this.Show();
			}
			if (this.icon.Blinking)
			{
				this.icon.Blinking = false;
			}
		}

		/// <summary>
		/// Start automatic mode. Raises when Automatic button (or item in tray menu) toggled
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Object which send event
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnAutomaticToggled (object sender, System.EventArgs e)
		{
			if (e != null)
			{
				e = null;
			}
			int stoped = 0;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				if (this.workthread.GetField(i, "working") == 0)
				{
					stoped++;
				}
			}
			if (this.cbtnauto.Active)
			{
				//Start crawling
				if (stoped == this.workthread.Crawlers)
				{
					this.workthread.Running = true;
					this.btnrun.Click();
				}
			}
			else
			{
				this.workthread.Running = false;
			}
		}

		/// <summary>
		/// Show preferences dialog. Raised when menu entry Preferences is seleted
		/// or button btnmoreoptions clicked
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnPreferencesActivated (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			PrefDialog pref = new PrefDialog();
			pref.ButtonAvailable = this.workthread.CrawlersStopped();
			int result = 0;
			bool cleared = false;
			string enableremote;
			while (result == 0)
			{
				result = pref.Run();
				cleared = pref.Cleared;
				if ((result == -5) || (result == 0))
				{
					this.workthread.SetPrefs(false);
					enableremote = Config.ReadConfig("/configuration/other/enableremote", this.conf.FilePath);
					if ((this.rControl == null) && (enableremote == "Y"))
					{
						this.rControl = new RemoteControl(this.workthread);
						this.rControl.Start();
						this.rControl.WebEventHandler += new EventHandler<WebEventArgs>(OnWebMessage);
					}
					if ((this.rControl != null) && (enableremote == "N"))
					{
						this.rControl.Stop();
						this.rControl.WebEventHandler -= OnWebMessage;
						this.rControl = null;
					}
				}
				if (result == 1)
				{
					result = 0;
				}
			}
			if (cleared)
			{
				this.workthread.SetField(0, "crawlerstats", 0);
				this.lblsend.Text = "0";
				this.lblurls.Text = "0";
				this.lblvalid.Text = "0";
				this.lblinvalid.Text = "0";
				this.lblerrors.Text = "0";
				this.lblreceived.Text = "0";
				this.lblsendmb.Text = "0";
			}
			pref.Dispose();
		}

		/// <summary>
		/// Show README.txt file. Raised when menu option Help is selected.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnHelpActivated (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			HelpDialog help = new HelpDialog("README.txt");
			int result = help.Run();
			if (result < 0)
			{
				help.Destroy();
				help.Dispose();
			}
		}
		
		/// <summary>
		/// Show crawling options. Raised when button Options clicked.
		/// Block button Options, make available buttons Stats and Details.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnBtnoptionsClicked(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.notebook.CurrentPage = 1;
			this.btnoptions.Sensitive = false;
			this.btndetails.Sensitive = true;
			this.btnstats.Sensitive = true;
		}

		/// <summary>
		/// Show details about crawling (list of crawlers). Raised when button Details is clicked.
		/// Block button Details, make available buttons Stats and Options.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnBtndetailsClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.notebook.CurrentPage = 0;
			this.btndetails.Sensitive = false;
			this.btnstats.Sensitive = true;
			this.btnoptions.Sensitive = true;
		}
		
		/// <summary>
		/// Show stats about crawling. Raised when button Stats is clicked. Block
		/// button Stats, make available buttons Details and Options.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> Unused
		/// </param>
		void OnBtnstatsClicked (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.notebook.CurrentPage = 2;
			this.btndetails.Sensitive = true;
			this.btnoptions.Sensitive = true;
			this.btnstats.Sensitive = false;
		}
		
		/// <summary>
		/// Timer for update gui, check net connection, close program. Raised every
		/// second when program works.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="ElapsedEventArgs"/> Unused
		/// </param>
		void OnWorkElapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			TimeSpan tmpspan = new TimeSpan(0, 0, 1);
			this.uptimespan += tmpspan;
			string[] time = this.uptimespan.ToString().Split('.');
			if (this.uptimespan.Days == 1)
			{
				time[0] = time[0] + " " + Catalog.GetString("day") + " " + time[1];
			}
			if (this.uptimespan.Days > 1)
			{
				time[0] = time[0] + " " + Catalog.GetString("days") + " " + time[1];
			}
			int stoped = 0;
			int working = 0;
			int paused = 0;
			double mbused = 0;
			double[] diskusage = new double[this.workthread.Crawlers];
			bool changedsettings = false;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{		
				switch (this.workthread.GetField(i, "working"))
				{
				case 0:
					stoped++;
					break;
				case 1:
					working++;
					break;
				case 2:
					paused++;
					break;
				}
				diskusage[i] = this.workthread.CountDiskUsage(i);
				mbused += diskusage[i];
			}
			if (paused != this.workthread.Crawlers)
			{
				this.CheckNet();
			}
			if ((working > 0) && (paused == 0))
			{
				Application.Invoke(delegate
				                   {
					if (this.imgstatus.Pixbuf != this.workimg)
					{
						this.imgstatus.Pixbuf = this.workimg;
					}
					if (this.btnrun.Label != Catalog.GetString("Pause"))
					{
						this.UpdateButton(true);
					}
				});
			}
			if (paused > 0)
			{
				Application.Invoke(delegate
				                   {
					if (this.imgstatus.Pixbuf != this.pauseimg)
					{
						this.imgstatus.Pixbuf = this.pauseimg;
					}
					if (this.btnrun.Label != Catalog.GetString("Start"))
					{
						this.UpdateButton(false);
					}
				});
			}
			Application.Invoke(delegate 
			                   {
				this.lbluptime.Text = time[0];
				if (Convert.ToInt32(this.lblsend.Text) != this.workthread.GetField(0, "workunitsend"))
				{
					this.lblsend.Text = this.workthread.GetField(0, "workunitsend").ToString();
					this.lblsend2.Text = this.workthread.GetField(0, "overallworkunit").ToString();
				}
				if (Convert.ToInt32(this.lblinvalidarc.Text) != this.workthread.GetField(0, "invalidarc"))
				{
					this.lblinvalidarc.Text = this.workthread.GetField(0, "invalidarc").ToString();
					this.lblinvalidarc2.Text = this.workthread.GetField(0, "overallinvalidworkunit").ToString();
				}
				if (Convert.ToDouble(this.lbldiskusage.Text) != mbused)
				{
					this.lbldiskusage.Text = mbused.ToString();
				}
				if (Convert.ToInt32(this.lblurls.Text) != this.workthread.GetField(0, "crawledurls"))
				{
					this.lblurls.Text = this.workthread.GetField(0, "crawledurls").ToString();
					this.lblvalid.Text = this.workthread.GetField(0, "validurls").ToString();
					this.lblinvalid.Text = this.workthread.GetField(0, "invalidurls").ToString();
					this.lblerrors.Text = this.workthread.GetField(0, "errorurls").ToString();
					this.lblurls2.Text = this.workthread.GetField(0, "overallcrawled").ToString();
					this.lblvalid2.Text = this.workthread.GetField(0, "overallvalid").ToString();
					this.lblinvalid2.Text = this.workthread.GetField(0, "overallinvalid").ToString();
					this.lblerrors2.Text = this.workthread.GetField(0, "overallerror").ToString();
				}
				double tmp = Math.Round(this.workthread.GetDoubleField("uptimereceived"), 1);
				this.lblreceived.Text = tmp.ToString();
				tmp = Math.Round(this.workthread.GetDoubleField("uptimesend"), 1);
				this.lblsendmb.Text = tmp.ToString();
				tmp = Math.Round(this.workthread.GetDoubleField("overallreceived"), 1);
				this.lblreceived2.Text = tmp.ToString();
				tmp = Math.Round(this.workthread.GetDoubleField("overallsent"), 1);
				this.lblsendmb2.Text = tmp.ToString();
				if ((this.crawlers != this.workthread.MaxCrawlers) || (this.workingcrawlers != working))
				{
					this.lblcurrent.Text = working.ToString() + "/" + this.workthread.MaxCrawlers.ToString();
					this.crawlers = this.workthread.MaxCrawlers;
					this.workingcrawlers = working;
				}
				this.icon.Tooltip = Catalog.GetString("Working crawlers:") + " " + 
						working.ToString() + "/" + this.workthread.MaxCrawlers.ToString() + Environment.NewLine +
						Catalog.GetString("Disk Usage (MB):") + " " + mbused.ToString() + Environment.NewLine +
						Catalog.GetString("Workunits send:") + " " + this.workthread.GetField(0, "workunitsend").ToString() + Environment.NewLine +
						Catalog.GetString("Links crawled:") + " " + this.workthread.GetField(0, "crawledurls").ToString();
				string temp = null;
				tmp = 0;
				if ((this.iter1.Count > this.workthread.Crawlers) && (this.workthread.Crawlers == this.workthread.MaxCrawlers))
				{
					int deletelines = this.iter1.Count - this.workthread.Crawlers;
					TreeIter tempiter;
					for (int i = 0; i < deletelines; i++)
					{
						tempiter = this.iter1[(this.iter1.Count - 1)];
						this.store.Remove(ref tempiter);
						this.iter1.RemoveAt((this.iter1.Count - 1));
						this.curiter.RemoveAt((this.curiter.Count - 1));
					}
				}
				for (int i = 0; i < this.workthread.Crawlers; i++)
				{
					temp = this.store.GetValue(iter1[i], 1).ToString();
					if (temp != this.crawlerstatus[this.workthread.GetField(i, "status")])
					{
						this.store.SetValue(iter1[i], 1, this.crawlerstatus[this.workthread.GetField(i, "status")]);
					}
					if (this.workthread.GetField(i, "status") == 6)
					{
						tmp = this.workthread.UploadProgress();
						if (tmp < 0)
						{
							tmp = 0;
						}
						if (tmp > 100)
						{
							tmp = 100;
						}
						if ((tmp < int.MinValue) || (tmp > int.MaxValue))
						{
							tmp = 0;
						}
						this.store.SetValue(iter1[i], 2, Convert.ToInt32(tmp));
					}
					if (this.curiter[i] != this.workthread.GetField(i, "curiter"))
					{
						if (this.workthread.GetField(i, "status") == 5)
						{
							tmp = (double)this.workthread.GetField(i, "curiter") / (double)(this.workthread.GetField(i, "iter") - 1) * 100;
						}
						if (tmp < 0)
						{
							tmp = 0;
						}
						if (tmp > 100)
						{
							tmp = 100;
						}
						this.store.SetValue(iter1[i], 2, Convert.ToInt32(tmp));
						this.curiter[i] = this.workthread.GetField(i, "curiter");
					}
					if (this.workthread.GetField(i, "status") == 0)
					{
						this.store.SetValue(iter1[i], 2, 0);
					}
					temp = this.store.GetValue(iter1[i], 3).ToString();
					if (temp != this.workthread.GetStringField(i, "workfilename"))
					{
						this.store.SetValue(iter1[i], 3, this.workthread.GetStringField(i, "workfilename"));
					}
					temp = this.store.GetValue(iter1[i], 4).ToString();
					if (temp != this.workthread.GetStringField(i, "downloaddate"))
					{
						this.store.SetValue(iter1[i], 4, this.workthread.GetStringField(i, "downloaddate"));
					}
					temp = this.store.GetValue(iter1[i], 5).ToString();
					if (temp != diskusage[i].ToString())
					{
						this.store.SetValue(iter1[i], 5, Math.Round(diskusage[i], 1));
					}
				}
				if (this.sbtnmaxcrawlers.ValueAsInt != this.maxcrawlers)
				{
					this.maxcrawlers = this.sbtnmaxcrawlers.ValueAsInt;
					changedsettings = true;
				}
				if (this.sbtnmaxdownload.ValueAsInt != this.maxdownload)
				{
					this.maxdownload = this.sbtnmaxdownload.ValueAsInt;
					changedsettings = true;
				}
				if (this.sbtnmaxupload.ValueAsInt != this.maxupload)
				{
					this.maxupload = this.sbtnmaxupload.ValueAsInt;
					changedsettings = true;
				}
				if (changedsettings)
				{
					string[] xmlvalues = new string[] { 
						this.sbtnmaxcrawlers.ValueAsInt.ToString(),
						this.sbtnmaxdownload.ValueAsInt.ToString(),
						this.sbtnmaxupload.ValueAsInt.ToString()
					};
					string[] xmlnodes = new string[] {
						"/configuration/work/crawlers",
						"/configuration/work/downloadrate",
						"/configuration/work/uploadrate"
					};
					Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
					changedsettings = false;
					this.workthread.SetPrefs(false);
					this.RecountCrawlers();
				}
			});
			if (this.connectiondelay > 0)
			{
				this.connectiondelay--;
			}
			if (stoped > 0)
			{
				if (!this.cbtnauto.Active)
				{
					if (working == 0)
					{
						Application.Invoke(delegate
						                   {
							this.imgstatus.Pixbuf = this.stopimg;
							this.UpdateButton(false);
							if (!this.Visible)
							{
								this.icon.Blinking = true;
							}
						});
					}
				}
				else
				{
					if (this.workthread.ConnectionError)
					{
						this.connectionerrors ++;
						this.connectiondelay = 30 * this.connectionerrors;
						this.workthread.ConnectionError = false;
						this.workthread.Running = false;
					}
					if ((this.connectiondelay <= 0) && (!this.workthread.Running))
					{
						this.workthread.Running = true;
						this.workthread.Start();
					}
				}
			}
			if (this.quitprogram)
			{
				if (!this.workthread.CrawlersStopped())
				{
					if (this.cbtnauto.Active)
					{
						this.cbtnauto.Active = false;
					}
					if (this.btnrun.Label != Catalog.GetString("Start"))
					{
						this.imgstatus.Pixbuf = this.pauseimg;
						this.UpdateButton(false);
					}
					for (int i = 0; i < this.workthread.Crawlers; i++)
					{
						this.workthread.SetField(i, "working", 0);
					}
					this.quitdelay --;
					if (this.quitdelay == 0)
					{
						this.workthread.WriteStats();
						MainClass.DeletePIDFile();
						System.Diagnostics.Process proc = System.Diagnostics.Process.GetCurrentProcess();
						proc.Kill();
					}
				}
				else
				{
					this.worktime.Stop();
					this.workthread.WriteStats();
					Application.Quit();
				}
			}
		}
		
		/// <summary>
		/// Function update text and image on btnrun.
		/// </summary>
		/// <param name="working">
		/// A <see cref="System.Boolean"/> if true, set button text and image to Pause, otherwise to Run
		/// </param>
		void UpdateButton(bool working)
		{
			string btnimage;
			string btntext;
			if (!working)
			{
				btnimage = "run.png";
				btntext = Catalog.GetString("Start");
			}
			else
			{
				btnimage = "pause.png";
				btntext = Catalog.GetString("Pause");
			}
			this.btnrun.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
			                                                     System.IO.Path.DirectorySeparatorChar + btnimage));
			this.btnrun.ImagePosition = PositionType.Left;
			this.btnrun.Label = btntext;
			this.btnrun.ShowAll();
		}
				
		/// <summary>
		/// Function show details about selected crawler. Raised when user double click on row or hit enter.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="Gtk.RowActivatedArgs"/> args.Path - iter of clicked row
		/// </param>
		void OnTreedetRowActivated (object o, Gtk.RowActivatedArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			TreeIter iter = new TreeIter();
			bool res = this.store.GetIter(out iter, args.Path);
			if (res)
			{
			}
			int crawler = Convert.ToInt32(this.store.GetValue(iter, 0)) - 1;
			CrawlerInfo info = new CrawlerInfo(crawler, this.workthread);
			int result = 0;
			while (result == 0)
			{
				result = info.Run();
			}
			info.Destroy();
			info.Dispose();
		}
		
		/// <summary>
		/// Function check internet connection. If there is more than 49 request timeout in row, function ping net
		/// for check internet connection. If fail, pause all crawlers and show information to user about problems with 
		/// connection.
		/// </summary>
		void CheckNet()
		{
			bool check = true;
			if (this.workthread.TimeOut > 49)
			{
				System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.WebRequest.Create("http://grub.org/");
				request.Timeout = 5000;
				request.ProtocolVersion = System.Net.HttpVersion.Version11;
				request.UserAgent = "GrubNG";
				request.KeepAlive = false;
				request.Proxy = null;
				request.AllowAutoRedirect = false;
				try
				{
					System.Net.HttpWebResponse response = (System.Net.HttpWebResponse)request.GetResponse();
					response.Close();
				}
				catch (System.Net.WebException e)
				{
					int error = (int)e.Status;
					if (error == 1)
					{
						check = false;
						int result;
						for (int i = 0; i < this.workthread.Crawlers; i++)
						{
							this.workthread.SetField(i, "working", 2);
						}
						Application.Invoke(delegate {
							this.imgstatus.Pixbuf = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "icon-grub-pause.jpg"));
							MessageDialog md = new MessageDialog(this, 
							                                     DialogFlags.DestroyWithParent,
							                                     MessageType.Error, 
							                                     ButtonsType.Close, 
							                                     Catalog.GetString("Couldn't connect to several sites. ") + Catalog.GetString("Possibly your computer isn't connected with the Internet anymore. ") + Catalog.GetString("Please check the Internet connection."));
							result = md.Run();
							if (result < 0)
							{
								md.Destroy();
								md.Dispose();
							}
						});
					}
				}
				if (check)
				{
					this.workthread.TimeOut = 0;
				}
			}
		}

		/// <summary>
		/// Function show context menu to user. Raised when right mouse button is clicked.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="Gtk.ButtonReleaseEventArgs"/> give information about which button is pressed
		/// </param>
		[GLib.ConnectBefore]
		void OnTreedetButtonReleaseEvent (object o, Gtk.ButtonReleaseEventArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			if (args.Event.Button == 3)
			{
				Menu menu = new Menu();
				MenuItem run = new MenuItem(Catalog.GetString("Run/Pause"));
				run.Activated += new EventHandler(OnRunActivated);
				run.Show();
				menu.Append(run);
				MenuItem details = new MenuItem(Catalog.GetString("Details"));
				details.Activated += new EventHandler(OnDetActivated);
				details.Show();
				menu.Append(details);
				menu.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
				menu.Dispose();
			}
		}
		
		/// <summary>
		/// Function show details about selected crawler. Raised when selected option details from context menu.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnDetActivated (object o, EventArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			if (args != null)
			{
				args = null;
			}
			TreePath path;
			TreeViewColumn column;
			this.treedet.GetCursor(out path, out column);
			TreeIter iter = new TreeIter();
			bool result2 = this.store.GetIter(out iter, path);
			if (result2)
			{
				int crawler = Convert.ToInt32(this.store.GetValue(iter, 0)) - 1;
				CrawlerInfo info = new CrawlerInfo(crawler, this.workthread);
				int result = 0;
				while (result == 0)
				{
					result = info.Run();
				}
				info.Destroy();
				info.Dispose();
			}
		}
		
		/// <summary>
		/// Function run/pause selected crawler. Raised when user select
		/// option run from context menu.
		/// </summary>
		/// <param name="o">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/>
		/// </param>
		void OnRunActivated (object o, EventArgs args)
		{
			if (o != null)
			{
				o = null;
			}
			if (args != null)
			{
				args = null;
			}
			TreePath path;
			TreeViewColumn column;
			this.treedet.GetCursor(out path, out column);
			TreeIter iter = new TreeIter();
			bool result = this.store.GetIter(out iter, path);
			if (result)
			{
				int crawler = Convert.ToInt32(this.store.GetValue(iter, 0)) - 1;
				int status = this.workthread.GetField(crawler, "working");
				switch (status)
				{
				case 0:
					this.workthread.StartCrawler(crawler);
					break;
				case 1:
					this.workthread.SetField(crawler, "working", 2);
					break;
				case 2:
					this.workthread.SetField(crawler, "working", 1);
					break;
				default:
					break;
				}
			}
		}
		
		/// <summary>
		/// Function sort list of crawlers ascending or descending. Raised when column header clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> GTK.TreeViewColumn which header is clicked
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnColumnClick(object sender, EventArgs e)
		{
			if (e != null)
			{
				e = null;
			}
			TreeViewColumn column = (TreeViewColumn)sender;
			if (column.SortOrder == SortType.Descending)
			{
				column.SortOrder = SortType.Ascending;
			}
			else
			{
				column.SortOrder = SortType.Descending;
			}
		}
		
		/// <summary>
		/// Function show popup menu for tray icon. Raised on right click on icon
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="PopupMenuArgs"/> unused
		/// </param>
		void OnIconMenu(object sender, PopupMenuArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			if (!this.Visible)
			{
				Menu menu = new Menu();
				MenuItem run = new MenuItem(Catalog.GetString("Run/Pause"));
				run.Activated += new EventHandler(OnBtnrunClicked);
				run.Show();
				menu.Append(run);
				CheckMenuItem automode = new CheckMenuItem(Catalog.GetString("Automatic"));
				automode.Active = this.cbtnauto.Active;
				automode.Toggled += new EventHandler(OnAutomaticToggled);
				automode.Show();
				menu.Append(automode);
				SeparatorMenuItem separator = new SeparatorMenuItem();
				separator.Show();
				menu.Append(separator);
				MenuItem show = new MenuItem(Catalog.GetString("Show window"));
				show.Activated += new EventHandler(OnIconActivate);
				show.Show();
				menu.Append(show);
				MenuItem sitemap = new MenuItem(Catalog.GetString("Create sitemap"));
				sitemap.Activated += new EventHandler(OnCreateSitemapActionActivated);
				sitemap.Show();
				menu.Append(sitemap);
				MenuItem about = new MenuItem(Catalog.GetString("About"));
				about.Activated += new EventHandler(OnAboutActivated);
				about.Show();
				menu.Append(about);
				SeparatorMenuItem separator2 = new SeparatorMenuItem();
				separator2.Show();
				menu.Append(separator2);
				MenuItem exit = new MenuItem(Catalog.GetString("Quit"));
				exit.Activated += new EventHandler(OnQuitActivated);
				exit.Show();
				menu.Append(exit);
				menu.Popup(null, null, null, 3, Gtk.Global.CurrentEventTime);
				menu.Dispose();
			}
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose() function.
		/// </summary>
		public override void Dispose()
		{
			this.File.Dispose();
			this.quit.Dispose();
			this.Help.Dispose();
			this.help.Dispose();
			this.about.Dispose();
			this.Options.Dispose();
			this.preferences.Dispose();
			this.OtherAction.Dispose();
			this.CreateSitemapAction.Dispose();
			this.vbox1.Dispose();
			this.hbox1.Dispose();
			this.hbuttonbox1.Dispose();
			this.cbtnauto.Dispose();
			this.hbox3.Dispose();
			this.vbuttonbox3.Dispose();
			this.notebook.Dispose();
			this.GtkScrolledWindow.Dispose();
			this.treedet.Dispose();
			this.hbox2.Dispose();
			this.label5.Dispose();
			this.imgstatus.Dispose();
			this.image1.Dispose();
			this.image8.Dispose();
			this.image9.Dispose();
			this.label1.Dispose();
			this.label4.Dispose();
			this.label8.Dispose();
			this.lblcurrent.Dispose();
			this.lbldiskusage.Dispose();
			this.hseparator1.Dispose();
			this.label6.Dispose();
			this.hbox4.Dispose();
			this.label7.Dispose();
			this.label10.Dispose();
			this.hbox10.Dispose();
			this.table2.Dispose();
			this.image10.Dispose();
			this.image103.Dispose();
			this.image12.Dispose();
			this.image13.Dispose();
			this.image232.Dispose();
			this.image490.Dispose();
			this.image53.Dispose();
			this.image2.Dispose();
			this.image3.Dispose();
			this.label11.Dispose();
			this.label12.Dispose();
			this.label14.Dispose();
			this.label16.Dispose();
			this.label17.Dispose();
			this.label21.Dispose();
			this.label13.Dispose();
			this.label29.Dispose();
			this.lblerrors.Dispose();
			this.lblinvalid.Dispose();
			this.lblinvalidarc.Dispose();
			this.lblreceived.Dispose();
			this.lblsend.Dispose();
			this.lblsendmb.Dispose();
			this.lblurls.Dispose();
			this.lblvalid.Dispose();
			this.image104.Dispose();
			this.image11.Dispose();
			this.image14.Dispose();
			this.image15.Dispose();
			this.image233.Dispose();
			this.image491.Dispose();
			this.image56.Dispose();
			this.label13.Dispose();
			this.label15.Dispose();
			this.label18.Dispose();
			this.label19.Dispose();
			this.label20.Dispose();
			this.label22.Dispose();
			this.label23.Dispose();
			this.lblerrors2.Dispose();
			this.lblinvalid2.Dispose();
			this.lblinvalidarc2.Dispose();
			this.lblreceived2.Dispose();
			this.lblsend2.Dispose();
			this.lblsendmb2.Dispose();
			this.lblurls2.Dispose();
			this.lblvalid2.Dispose();
			this.label2.Dispose();
			this.sbar.Dispose();
			this.workthread.Dispose();
			this.icon.Dispose();
			this.stopimg.Dispose();
			this.store.Dispose();
			this.workimg.Dispose();
			this.btndetails.Dispose();
			this.btnstats.Dispose();
			this.btnrun.Dispose();
			this.btnquit.Dispose();
			this.lbluptime.Dispose();
			this.hsmaxcrawlers.Dispose();
			this.hsmaxdownload.Dispose();
			this.hsmaxupload.Dispose();
			this.sbtnmaxcrawlers.Dispose();
			this.sbtnmaxdownload.Dispose();
			this.sbtnmaxupload.Dispose();
			if (this.rControl != null)
			{
				this.rControl.Dispose();
			}
			this.worktime.Dispose();
			this.pauseimg.Dispose();
			this.btnoptions.Dispose();
			this.UIManager.Dispose();
			this.menubar.Dispose();
			this.vbox4.Dispose();
			this.table6.Dispose();
			this.label24.Dispose();
			this.label26.Dispose();
			this.label27.Dispose();
			this.btnmoreoptions.Dispose();
			this.label3.Dispose();
			this.vbox5.Dispose();
			this.table4.Dispose();
			this.label25.Dispose();
			this.table5.Dispose();
			this.label28.Dispose();
			base.Dispose();
		}

		/// <summary>
		/// Start create sitemap. Raised when menu entry CreateSitemap clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused
		/// </param>
		void OnCreateSitemapActionActivated (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			SitemapURLDialog urldlg = new SitemapURLDialog();
			int result = 0;
			while (result != -6)
			{
				result = urldlg.Run();
				if (result == -4)
				{
					result = -6;
				}
			}
			urldlg.Destroy();
			urldlg.Dispose();
		}
		
		/// <summary>
		/// Function read options from config file and set it on GUI
		/// </summary>
		void ReadOptions()
		{
			string[] nodes = new string[] {
				"/configuration/work/crawlers",
				"/configuration/work/downloadrate",
				"/configuration/work/uploadrate"
			};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			nodes = null;
			this.hsmaxcrawlers.Value = Convert.ToDouble(values[0]);
			this.sbtnmaxcrawlers.Value = this.hsmaxcrawlers.Value;
			this.maxcrawlers = this.sbtnmaxcrawlers.ValueAsInt;
			this.hsmaxdownload.Value = Convert.ToDouble(values[1]);
			this.sbtnmaxdownload.Value = this.hsmaxdownload.Value;
			this.maxdownload = this.sbtnmaxdownload.ValueAsInt;
			this.hsmaxupload.Value = Convert.ToDouble(values[2]);
			this.sbtnmaxupload.Value = this.hsmaxupload.Value;
			this.maxupload = this.sbtnmaxupload.ValueAsInt;
		}
		
		/// <summary>
		/// Function update configuration file or quit from program. Raised when program received message from
		/// web interface.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="WebEventArgs"/> type of even, "quit" or "config"
		/// </param>
		void OnWebMessage(object sender, WebEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args.Type == "quit")
			{
				this.OnQuitActivated(null, EventArgs.Empty);
			}
			if (args.Type == "config")
			{
				this.ReadOptions();
				this.workthread.SetPrefs(false);
				this.RecountCrawlers();
			}
		}
		
		/// <summary>
		/// Function synchronize amount of crawlers
		/// </summary>
		void RecountCrawlers()
		{
			if (this.iter1.Count < this.workthread.Crawlers)
			{
				//Check for amount of crawlers
				int tempcrawlers = this.workthread.Crawlers - this.iter1.Count;
				for (int i = 0; i < tempcrawlers; i++)
				{
					this.iter1.Add(this.store.AppendValues((this.iter1.Count + 1), 
					                                       Catalog.GetString("Stopped"), 0, 
					                                       String.Empty, String.Empty, 0.0));
					this.curiter.Add(0);
				}
				if (this.workthread.Running)
				{
					this.workthread.Start();
				}
			}
			if (this.iter1.Count > this.workthread.Crawlers)
			{
				int tempcrawlers = this.iter1.Count - this.workthread.Crawlers;
				TreeIter tempiter;
				for (int i = 0; i < tempcrawlers; i++)
				{
					tempiter = this.iter1[(this.iter1.Count - 1)];
					this.store.Remove(ref tempiter);
					this.iter1.RemoveAt((this.iter1.Count - 1));
					this.curiter.RemoveAt((this.curiter.Count - 1));
				}
			}
		}
	}
}
