// SitemapDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file SitemapDialog.cs
 * Provide GTK Dialog for show informations about uploading sitemap.
 * @fn virtual void Grubng::SitemapDialog::Build()
 * Create part of GUI and show it to user*/

using Gtk;
using Mono.Unix;
using System;
using System.Threading;
using System.Timers;

namespace Grubng
{
	
	/// <summary>
	/// Provide GTK Dialog for show informations about uploading sitemap.
	/// </summary>
	internal sealed partial class SitemapDialog : Gtk.Dialog
	{
		/// <summary>
		/// GTK Button for close dialog.
		/// </summary>
		Button btnok;
		/// <summary>
		/// Upload thread.
		/// </summary>
		Thread uploadthread;
		/// <summary>
		/// Upload time timer.
		/// </summary>
		System.Timers.Timer uploadtime;
		/// <summary>
		/// Object from Sitemap class. Provide functions for manipulate sitemap file.
		/// </summary>
		Sitemap sitemap;
		
		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		/// <param name="file">
		/// A <see cref="System.String"/> Full path (including filename) to sitemap file.
		/// </param>
		public SitemapDialog(string file)
		{
			this.sitemap = new Sitemap(file);
			this.Build();
			this.lblfilename.Text = file;
			//Add button btnok 
			this.btnok = new Button();
			this.btnok.Image = new Image(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "ok.png"));
			this.btnok.ImagePosition = PositionType.Left;
			this.btnok.Label = Catalog.GetString("_OK");
			this.btnok.UseUnderline = true;
			this.AddActionWidget(this.btnok, -5);
			this.btnok.CanDefault = true;
			this.lblstatus.Text = Catalog.GetString("Checking propriety of sitemap file");
			this.Icon = new Gdk.Pixbuf(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "grubby.ico"));
			this.uploadthread = new Thread(new ThreadStart(this.UploadThread));
			this.uploadthread.Start();
		}
		
		/// <summary>
		/// Thread for operations on sitemap file.
		/// </summary>
		void UploadThread()
		{
			this.uploadtime = new System.Timers.Timer();
			this.uploadtime.Elapsed += new ElapsedEventHandler(OnUploadElapsed);
			this.uploadtime.Interval = 500;
			this.uploadtime.Start();
			bool valid = this.sitemap.CheckSitemap();
			if (!valid)
			{
				Application.Invoke(delegate {
					this.lblstatus.Text = Catalog.GetString("Invalid sitemap file");
					this.btnok.ShowAll();
				});
				this.uploadtime.Stop();
				this.uploadthread.Abort();
			}
			this.sitemap.Upload = true;
			Application.Invoke(delegate {
				this.lblstatus.Text = Catalog.GetString("Uploading");
			});
			int[] status = this.sitemap.UploadSitemap(false);
			if (status[0] == 417)
			{
				status = this.sitemap.UploadSitemap(true);
			}
			this.uploadtime.Stop();
			this.sitemap = null;
			string message;
			switch (status[0])
			{
			case 0:
				message = Catalog.GetString("Upload fail. Reason: Cannot connect to server.");
				break;
			case 200:
				message = Catalog.GetString("Sitemap uploaded.");
				break;
			case 401:
				message = Catalog.GetString("Upload fail. Reason: Invalid login.");
				break;
			case 408:
				message = Catalog.GetString("Upload fail. Reason: Connection Timeout.");
				break;
			case 500:
				message = Catalog.GetString("Upload fail. Reason: Internal Server Error.");
				break;
			case 503:
				message = Catalog.GetString("Upload fail. Reason: Cannot connect to server.");
				break;
			default:
				message = status[0].ToString();
				break;
			}
			Application.Invoke(delegate {
				this.lblstatus.Text = message;
				this.btnok.ShowAll();
				this.btnok.GrabFocus();
			});
			this.uploadthread.Abort();
		}
		
		/// <summary>
		/// Timer for update GUI while uploading. Raised every 0.5 second when program works.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="ElapsedEventArgs"/> unused.
		/// </param>
		void OnUploadElapsed(object sender, ElapsedEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			double progress = (double)this.sitemap.BytesSend / (double)this.sitemap.FileLength;
			if (progress < 0)
			{
				progress = 0;
			}
			if (progress > 1)
			{
				progress = 1;
			}
			if (progress == 1 && this.sitemap.Upload)
			{
				Application.Invoke(delegate {
					this.lblstatus.Text = Catalog.GetString("Waiting for server answer.");
				});
			}
			Application.Invoke(delegate {
				this.prgupload.Fraction = progress;
				this.prgupload.Text = this.sitemap.BytesSend.ToString() + "/" + this.sitemap.FileLength.ToString();
			});
		}
		
		/// <summary>
		/// Function override Gtk.Object.Dispose function.
		/// </summary>
		public override void Dispose()
		{
			this.vbox2.Dispose();
			this.table1.Dispose();
			this.label2.Dispose();
			this.label3.Dispose();
			this.lblfilename.Dispose();
			this.lblstatus.Dispose();
			this.prgupload.Dispose();
			this.btnok.Dispose();
			this.uploadtime.Dispose();
			base.Dispose();
		}
	}
}
