// startGTK.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file StartGTK.cs
 * Start GTK GUI of program.*/

using System;
using System.Diagnostics;
using System.IO;
using Gtk;
using Mono.Unix;

namespace Grubng
{
	
	/// <summary>
	/// Start GTK GUI of program.
	/// </summary>
	internal static class StartGTK
	{
	
		/// <summary>
		/// Create GTK GUI for program.
		/// </summary>
		/// <param name="basedir">
		/// A <see cref="System.String"/> Full path to program directory.
		/// </param>
		public static void StartGUI(string basedir)
		{
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(GTKUnhandledException);
			GLib.ExceptionManager.UnhandledException += new GLib.UnhandledExceptionHandler(GLibUnhandledException);
			Application.Init();
			int result = 0;
			//Check mono version for compatibility
			if ((Environment.Version.Major == 1) && (Environment.Version.Minor == 2) && (Environment.Version.Build < 5))
			{
				MessageDialog mesversion = new MessageDialog (null, 
				                                              DialogFlags.DestroyWithParent,
				                                              MessageType.Error, 
					                                          ButtonsType.Close, 
					                                          Catalog.GetString("Unfortunately, you have too old version of Mono to run program.") + Catalog.GetString("You need at least Mono 1.2.6"));
				result = mesversion.Run();
				mesversion.Destroy();
				mesversion.Dispose();
				return;
			}
			//Check for running instance of client (if any, terminate this and show info to user)
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
				Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar + "grub.pid";
			if (File.Exists(filepath))
			{
				MessageDialog mesother = new MessageDialog (null, 
				                                            DialogFlags.DestroyWithParent,
				                                            MessageType.Error, 
				                                            ButtonsType.Close, 
				                                            Catalog.GetString("You cannot run more than one client simultaneously"));
				result = mesother.Run();
				mesother.Destroy();
				mesother.Dispose();
				return;
			}
			bool rootneed = false;
			Config conf = new Config();
			string[] path = basedir.Split('/');
			if (path[1] != "home")
			{
				rootneed = true;
			}
			path = null;
			bool checkfile = File.Exists(conf.FilePath);
			result = 0;
			if (!checkfile)
			{
				LogonDialog login = new LogonDialog();
				result = login.Run();
				if (result == -5)
				{
					login.Destroy();
					login.Dispose();
				}
				if ((result == -6) || (result == -4))
				{
					login.Destroy();
					login.Dispose();
					return;
				}
			}
			//Create .pid file
			MainClass.CreatePIDFile();
			if ((checkfile) || (result == -5))
			{
				string runhidden = Config.ReadConfig("/configuration/other/runhidden", conf.FilePath);
				SplashScreen splash = new SplashScreen();
				if (runhidden == "Y")
				{
					splash.Hide();
					while (Application.EventsPending())
					{
						Application.RunIteration();
					}
				}
				else
				{
					System.Threading.Thread.Sleep(500);
					splash.LabelStatus = Catalog.GetString("Checking correctness of configuration file...");
					splash.PrgStatus.Adjustment.Value = 20;
					while (Application.EventsPending())
					{
						Application.RunIteration();
					}
				}
				bool updateconf = conf.CheckConfig();
				if (runhidden == "N")
				{
					if (updateconf)
					{
						splash.LabelStatus += Catalog.GetString("Updated");
					}
					else
					{
						splash.LabelStatus += Catalog.GetString("Done");
					}
					while (Application.EventsPending())
					{
						Application.RunIteration();
					}
					splash.LabelStatus = Catalog.GetString("Checking for available disk space...");
					splash.PrgStatus.Adjustment.Value = 40;
					while (Application.EventsPending())
					{
						Application.RunIteration();
					}
				}
				long needmb = Convert.ToInt64(Config.ReadConfig("/configuration/work/maxmbdownload", conf.FilePath)) * 
					Convert.ToInt32(Config.ReadConfig("/configuration/work/crawlers", conf.FilePath));
				UnixDriveInfo drive = new UnixDriveInfo(Config.ReadConfig("/configuration/work/workdir", conf.FilePath));
				if ((needmb * 1048576) > drive.AvailableFreeSpace)
				{
					MessageDialog mesmb = new MessageDialog (null, 
					                                         DialogFlags.DestroyWithParent,
					                                         MessageType.Warning, 
					                                         ButtonsType.Close, 
					                                         Catalog.GetString("You have less than") + " " + needmb.ToString() + " " + 
					                                         Catalog.GetString("MB free disk space. Please increase it before start."));
					result = mesmb.Run();
					mesmb.Destroy();
					mesmb.Dispose();
				}
				drive = null;
				string checkupdates = Config.ReadConfig("/configuration/other/checkupdates", conf.FilePath);
				if (checkupdates == "Y")
				{
					string rootpass = String.Empty;
					string unstable = Config.ReadConfig("/configuration/other/unstable", conf.FilePath);
					if (runhidden == "N")
					{
						splash.LabelStatus = Catalog.GetString("Checking for updates...");
						splash.PrgStatus.Adjustment.Value = 60;
						while (Application.EventsPending())
						{
							Application.RunIteration();
						}
					}
					System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
					string version = asm.GetName().Version.Major.ToString() + "." + asm.GetName().Version.Minor.ToString() + "." + asm.GetName().Version.Build.ToString();
					Updater upd = new Updater(version);
					string update = upd.CheckUpdate(unstable);
					if ((update == "current") || (update == "older"))
					{
						if (runhidden == "N")
						{
							splash.LabelStatus = Catalog.GetString("Updating...");
							splash.PrgStatus.Adjustment.Value = 80;
							while (Application.EventsPending())
							{
								Application.RunIteration();
							}
						}
						if (rootneed)
						{
							PassDialog pass = new PassDialog();
							result = pass.Run();
							rootpass = pass.RootPass;
							while (String.IsNullOrEmpty(rootpass))
							{
								result = pass.Run();
								rootpass = pass.RootPass;
							}
							pass.Destroy();
							pass.Dispose();
						}
						splash.Destroy();
						splash.Dispose();
						string[] updated = Updater.Update(update, unstable, rootpass);
						MessageDialog mesupd = new MessageDialog (null, 
					                                          DialogFlags.DestroyWithParent,
					                                          MessageType.Info, 
					                                          ButtonsType.Ok, 
					                                          Catalog.GetString("Client successfully updated. ") + Catalog.GetString("You must restart it again. ") + Catalog.GetString("Click Ok button for exit."));
						mesupd.Text += Environment.NewLine + Environment.NewLine + Catalog.GetString("Updated files:") + Environment.NewLine;
						for (int i = 0; i < updated.Length; i++)
						{
							mesupd.Text += (i + 1).ToString() + " " + updated[i] + Environment.NewLine;
						}
						result = mesupd.Run();
						mesupd.Destroy();
						mesupd.Dispose();
						return;
					}
					if (runhidden == "N")
					{
						if (update == "none")
						{
							splash.LabelStatus += Catalog.GetString("none");
						}
						else
						{
							splash.LabelStatus += Catalog.GetString("failed");
						}
					}
				}
				else
				{
					if (runhidden == "N")
					{
						splash.LabelStatus += Catalog.GetString("Done");
					}
				}
				if (runhidden == "N")
				{
					splash.PrgStatus.Adjustment.Value = 100;
					while (Application.EventsPending())
					{
						Application.RunIteration();
					}
				}
				System.Threading.Thread.Sleep(500);
				splash.Destroy();
				splash.Dispose();
				MainWindow main = new MainWindow();
				if (runhidden == "Y")
				{
					main.Hide();
				}
				Application.Run();
				main.Dispose();
				conf = null;
			}
		}
		
		/// <summary>
		/// Function catch unhandled exceptions for Glib. Write informations about exception to error.log file and show
		/// information dialog about error.
		/// </summary>
		/// <param name="args">
		/// A <see cref="GLib.UnhandledExceptionArgs"/> Provide informations about error (source, stacktrace,
		/// general info about error)
		/// </param>
		static void GLibUnhandledException(GLib.UnhandledExceptionArgs args)
		{
			MainClass.MainUnhandledException(null, args);
			Exception e = (Exception)args.ExceptionObject;
			string tdate = DateTime.Now.ToString("yyyy/MM/dd/HH:mm:ss");
			string errorinfo = tdate + " Caught: " + e.GetType().ToString() + Environment.NewLine
				+ e.Message + Environment.NewLine + 
				"  Source: " + e.Source + Environment.NewLine + 
				"  StackTrace: " + e.StackTrace + Environment.NewLine +
				"  TargetSite: " + e.TargetSite + Environment.NewLine;
            ErrorDialog error = new ErrorDialog(errorinfo);
            int result = error.Run();
			if (result < 0)
			{
				error.Dispose();
			}
		}
		
		/// <summary>
		/// Function catch unhandled exceptions for GTK GUI. Write informations about exception to error.log and show
		/// information dialog about error.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="UnhandledExceptionEventArgs"/> Provide informations about error (source, stacktrace, 
		/// general info about error)
		/// </param>
		static void GTKUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			MainClass.MainUnhandledException(sender, args);
			Exception e = (Exception)args.ExceptionObject;
			string tdate = DateTime.Now.ToString("yyyy/MM/dd/HH:mm:ss");
			string errorinfo = tdate + " Caught: " + e.GetType().ToString() + Environment.NewLine
				+ e.Message + Environment.NewLine + 
				"  Source: " + e.Source + Environment.NewLine + 
				"  StackTrace: " + e.StackTrace + Environment.NewLine +
				"  TargetSite: " + e.TargetSite + Environment.NewLine;
            ErrorDialog error = new ErrorDialog(errorinfo);
            int result = error.Run();
			if (result < 0)
			{
				error.Dispose();
			}
		}
	}
}
