// Main.cs
// 
// Copyright (C) 2008, 2010 Bartek Jasicki
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**
 * @namespace Grubng
 * @brief Main program namespace.*/
/** 
 * @file Main.cs
 * Provide Main program class.
 */

using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using Mono.Unix;

namespace Grubng
{
	/**Provide Main program class.*/
	class MainClass
	{
		/**Main program function
		 * @param args if array is empty, start GTK or Winforms GUI of program. If 
		 * args[0] is "console", start console version of client. If args[0] is 
		 * "daemon", then args[1] "start" start program in background, if args[1] 
		 * "stop" then stop daemon.*/
		static void Main(string[] args)
		{	
			if (Environment.OSVersion.Platform.ToString() == "Unix")
			{
				Mono.Unix.Native.Syscall.nice(19);
			}
			else
			{
				System.Diagnostics.Process proc = new System.Diagnostics.Process();
				proc = System.Diagnostics.Process.GetCurrentProcess();
				proc.PriorityClass = System.Diagnostics.ProcessPriorityClass.BelowNormal;
			}
			string basedir = System.AppDomain.CurrentDomain.BaseDirectory;
			Catalog.Init("grubng", Path.Combine(basedir, "locale"));
			int argslength = args.Length;
			if (argslength == 0)
			{
				if (Environment.OSVersion.Platform.ToString() == "Unix")
				{
					StartGTK.StartGUI(basedir);
				}
				else
				{
					StartWinForm.StartGUI();
				}
			}
			else
			{
				//Check mono version for compatibility
				if ((Environment.Version.Major == 1) && (Environment.Version.Minor == 2) && (Environment.Version.Build < 5))
				{
					Console.WriteLine(Catalog.GetString("Unfortunately, you have too old version of Mono to run program.") + Catalog.GetString("You need at least Mono 1.2.6"));
					return;
				}
				//Run program in console mode.
				if (args[0] == "console")
				{
					AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(ConsoleUnhandledException);
					StartConsole.StartGUI(basedir);
				}
				//Start/stop program in background
				if (args[0] == "daemon")
				{
					if (Environment.OSVersion.Platform.ToString() != "Unix")
					{
						Console.WriteLine(Catalog.GetString("Unfortunately, this option is available only on Unix systems."));
						return;
					}
					AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(MainUnhandledException);
					if (args[1] == "start")
					{
						System.Diagnostics.ProcessStartInfo procinfo = new System.Diagnostics.ProcessStartInfo();
						procinfo.Arguments = "--server " + Path.Combine(basedir, "Grubng.exe") + " daemon run &";
						procinfo.FileName = "mono";
						procinfo.UseShellExecute = false;
						System.Diagnostics.Process proc = System.Diagnostics.Process.Start(procinfo);
						proc.Close();
						proc = null;
					}
					if (args[1] == "run")
					{
						Daemon grubd = new Daemon();
						grubd.Start();
						grubd.Dispose();
					}
					if (args[1] == "stop")
					{
						IPAddress ipAddress = Dns.GetHostEntry(Dns.GetHostName()).AddressList[0];
						IPEndPoint ipEndpoint = new IPEndPoint(ipAddress, 6688);
						Socket socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
						socket.Connect(ipEndpoint);
						socket.Shutdown(SocketShutdown.Both);
						socket.Close();
						socket = null;
					}
				}
			}
			return;
		}
		
		/**Function catch unhandled exceptions for console.
		 * Write informations about exception to error.log.
		 * @param sender Unused
		 * @param args Provide informations about error (source, stacktrace,
		 * general info about error)*/
		static void ConsoleUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			MainUnhandledException(sender, args);
			Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
			                  Catalog.GetString("Critical error appear and client been closed.") + " " + 
			                  Catalog.GetString("Please report bug on http://grub.org.") + " " + 
			                  Catalog.GetString("To report add file error.log from your work directory.") + "]");
			Console.Read();
		}
		
		/**Main function for catch unhandled exceptions.
		 * Write informations about exception to error.log.
		 * @param sender Unused
		 * @param args Provide informations about error (source, stacktrace,
		 * general info about error)*/
		public static void MainUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			Config conf = new Config();
			string filepath;
			if (conf.FilePath.Length > 0)
			{
				filepath = Config.ReadConfig("/configuration/work/workdir", conf.FilePath);
			}
			else
			{
				filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) +
					Path.DirectorySeparatorChar;
			}
			Exception e = (Exception)args.ExceptionObject;
			string tdate = DateTime.Now.ToString("yyyy/MM/dd/HH:mm:ss");
			FileStream errorlog = File.Open(filepath + "error.log", FileMode.Append);
			StreamWriter logstream = new StreamWriter(errorlog); 
			logstream.WriteLine(tdate + " Caught: " + e.GetType().ToString() + " " + e.Message);
			logstream.WriteLine("  Source: " + e.Source);
			logstream.WriteLine("  StackTrace: " + e.StackTrace);
			logstream.WriteLine("  TargetSite: " + e.TargetSite);
			logstream.Close();
		}
	}
}