// MainConsole.cs
// 
// Copyright (C) 2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
/** 
 * @file MainConsole.cs
 * Provide Main program class for console version for Unix systems.
 */

using System;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide Main program class.
	/// </summary>
	static class MainClass
	{
		/// <summary>
		/// Main program function.
		/// </summary>
		/// <param name="args">
		/// A <see cref="System.String[]"/> If args[0] is "start" start program in 
		/// background (as daemon).
		/// </param>
		static void Main(string[] args)
		{	
			SetProcessName("grubng");
			int result = Mono.Unix.Native.Syscall.nice(19);
			if (result != 19)
			{
				return;
			}
			MainClass.DeletePIDFile();
			string basedir = System.AppDomain.CurrentDomain.BaseDirectory;
			Catalog.Init("grubng", Path.Combine(basedir, "locale"));
			//Check mono version for compatibility
			if ((Environment.Version.Major == 1) && (Environment.Version.Minor == 2) && (Environment.Version.Build < 5))
			{
				Console.WriteLine(Catalog.GetString("Unfortunately, you have too old version of Mono to run program.") + 
				                  Catalog.GetString("You need at least Mono 1.2.6"));
			}
			if (args.Length > 0)
			{
				//Start/stop program in background
				if (Environment.OSVersion.Platform.ToString() != "Unix")
				{
					Console.WriteLine(Catalog.GetString("Unfortunately, this option is available only on Unix systems."));
					return;
				}
				AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(MainUnhandledException);
				if (args[0] == "start")
				{
					System.Diagnostics.ProcessStartInfo procinfo = new System.Diagnostics.ProcessStartInfo();
					procinfo.Arguments = Path.Combine(basedir, "Grubng.exe") + " run &";
					if (args.Length == 2)
					{
						procinfo.Arguments = "--debug " + procinfo.Arguments;
					}
					procinfo.FileName = "mono";
					procinfo.UseShellExecute = false;
					System.Diagnostics.Process proc = System.Diagnostics.Process.Start(procinfo);
					proc.Close();
					proc = null;
				}
				if (args[0] == "run")
				{
					Daemon grubd = new Daemon();
					grubd.Start();
					grubd.Dispose();
				}
			}
			else
			{
				//Run program in console mode.
				AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(ConsoleUnhandledException);
				StartConsole.StartGUI(basedir);
			}
			MainClass.DeletePIDFile();
			return;
		}
		
		[DllImport ("libc")] // GNU/Linux
		private static extern int prctl (int option, byte [] arg2, IntPtr arg3,
		                                 IntPtr arg4, IntPtr arg5);

		[DllImport ("libc")] // BSD
		private static extern void setproctitle (byte [] fmt, byte [] str_arg);

		/// <summary>
		/// Function set process name on Unix system. Code borrowed from 
		/// http://abock.org/2006/02/09/changing-process-name-in-mono/
		/// </summary>
		/// <param name="name">
		/// A <see cref="System.String"/> new name of process.
		/// </param>
		public static void SetProcessName (string name)
		{
			try 
			{
				if (prctl (15 /* PR_SET_NAME */, Encoding.ASCII.GetBytes (name + "\0"),
				           IntPtr.Zero, IntPtr.Zero, IntPtr.Zero) != 0) {
					throw new ApplicationException ("Error setting process name: " +
					                                Mono.Unix.Native.Stdlib.GetLastError ());
				}
			} 
			catch (EntryPointNotFoundException) 
			{
				setproctitle (Encoding.ASCII.GetBytes ("%s\0"),
				              Encoding.ASCII.GetBytes (name + "\0"));
			}
		}
		
		/// <summary>
		/// Function catch unhandled exceptions for console. Write informations about exception to error.log.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="args">
		/// A <see cref="UnhandledExceptionEventArgs"/> provide informations about error (source, stacktrace,
		/// general info about error)
		/// </param>
		static void ConsoleUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			MainUnhandledException(sender, args);
			Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
			                  Catalog.GetString("Critical error appear and client been closed.") + " " + 
			                  Catalog.GetString("Please report bug on http://grub.org.") + " " + 
			                  Catalog.GetString("To report add file error.log from your work directory.") + "]");
			int result = Console.Read();
			result ++;
		}
		
		/// <summary>
		/// Main function for catch unhandled exceptions. Write informations about exception to error.log.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="UnhandledExceptionEventArgs"/> provide informations about error (source, stacktrace,
		/// general info about error)
		/// </param>
		static void MainUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			Config conf = new Config();
			string filepath;
			if (File.Exists(conf.FilePath))
			{
				filepath = Config.ReadConfig("/configuration/work/workdir", conf.FilePath);
			}
			else
			{
				filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) +
					Path.DirectorySeparatorChar;
			}
			Exception e = (Exception)args.ExceptionObject;
			string tdate = DateTime.Now.ToString("yyyy/MM/dd/HH:mm:ss");
			FileStream errorlog = File.Open(filepath + "error.log", FileMode.Append);
			StreamWriter logstream = new StreamWriter(errorlog); 
			logstream.WriteLine(tdate + " Caught: " + e.GetType().ToString() + " " + e.Message);
			logstream.WriteLine("  Source: " + e.Source);
			logstream.WriteLine("  StackTrace: " + e.StackTrace);
			logstream.WriteLine("  TargetSite: " + e.TargetSite);
			logstream.Close();
			MainClass.DeletePIDFile();
		}
		
		/// <summary>
		/// Create grub.pid file in configuration directory with process ID.
		/// </summary>
		public static void CreatePIDFile()
		{
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
					Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar + "grub.pid";
			StreamWriter pidfile = new StreamWriter(filepath);
			pidfile.Write(Process.GetCurrentProcess().Id);
			pidfile.Flush();
			pidfile.Close();
			pidfile.Dispose();
		}
		
		/// <summary>
		/// Function delete grub.pid file.
		/// </summary>
		public static void DeletePIDFile()
		{
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
					Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar + "grub.pid";
			if (File.Exists(filepath))
			{
				StreamReader pidfile = new StreamReader(filepath);
				int pid = Convert.ToInt32(pidfile.ReadToEnd());
				pidfile.Close();
				pidfile.Dispose();
				try
				{
					Process proc = Process.GetProcessById(pid);
					proc.Dispose();
				}
				catch (ArgumentException)
				{
					File.Delete(filepath);
				}
				if (Process.GetCurrentProcess().Id == pid)
				{
					File.Delete(filepath);
				}
			}
		}
	}
}