// MainWindows.cs
// 
// Copyright (C) 2008,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the teCatalogs of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/** 
 * @file MainWindows.cs
 * Provide Main program class for Windows system.
 */

using System;
using System.Diagnostics;
using System.IO;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide Main program class.
	/// </summary>
	static class MainClass
	{
		/// <summary>
		/// Main program function.
		/// </summary>
		/// <param name="args">
		/// A <see cref="System.String[]"/> if array is empty, start WinForm GUI of program. 
		/// If args[0] is "console", start console version of client.
		/// </param>
		[STAThread]
		static void Main(string[] args)
		{	
			Process proc = Process.GetCurrentProcess();
			proc.PriorityClass = ProcessPriorityClass.BelowNormal;
			proc.Dispose();
			MainClass.DeletePIDFile();
			string basedir = System.AppDomain.CurrentDomain.BaseDirectory;
			Catalog.Init("grubng", Path.Combine(basedir, "locale"));
			int argslength = args.Length;
			if (argslength == 0)
			{
				StartWinForm.StartGUI();
			}
			else
			{
				//Check mono version for compatibility
				if ((Environment.Version.Major == 1) && (Environment.Version.Minor == 2) && (Environment.Version.Build < 5))
				{
					Console.WriteLine(Catalog.GetString("Unfortunately, you have too old version of Mono to run program.") + 
					                  Catalog.GetString("You need at least Mono 1.2.6"));
				}
				//Run program in console mode.
				if (args[0] == "console")
				{
					AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(ConsoleUnhandledException);
					StartConsole.StartGUI(basedir);
				}
			}
			MainClass.DeletePIDFile();
			return;
		}
		
		/// <summary>
		/// Function catch unhandled exceptions for console. Write informations about exception 
		/// to error.log.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="UnhandledExceptionEventArgs"/> provide informations about error (source, 
		/// stacktrace, general info about error)
		/// </param>
		static void ConsoleUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			MainUnhandledException(sender, args);
			Console.WriteLine(Catalog.GetString("[INFO:") + " " + 
			                  Catalog.GetString("Critical error appear and client been closed.") + " " + 
			                  Catalog.GetString("Please report bug on http://grub.org.") + " " + 
			                  Catalog.GetString("To report add file error.log from your work directory.") + "]");
			int result = Console.Read();
			result++;
		}
		
		/// <summary>
		/// Main function for catch unhandled exceptions. Write informations about 
		/// exception to error.log.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="UnhandledExceptionEventArgs"/> provide informations about error (source, 
		/// stacktrace, general info about error)
		/// </param>
		public static void MainUnhandledException(object sender, UnhandledExceptionEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			Config conf = new Config();
			string filepath;
			if (File.Exists(conf.FilePath))
			{
				filepath = Config.ReadConfig("/configuration/work/workdir", conf.FilePath);
			}
			else
			{
				filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) +
					Path.DirectorySeparatorChar;
			}
			Exception e = (Exception)args.ExceptionObject;
			string tdate = DateTime.Now.ToString("yyyy/MM/dd/HH:mm:ss");
			FileStream errorlog = File.Open(filepath + "error.log", FileMode.Append);
			StreamWriter logstream = new StreamWriter(errorlog); 
			logstream.WriteLine(tdate + " Caught: " + e.GetType().ToString() + " " + e.Message);
			logstream.WriteLine("  Source: " + e.Source);
			logstream.WriteLine("  StackTrace: " + e.StackTrace);
			logstream.WriteLine("  TargetSite: " + e.TargetSite);
			logstream.Close();
			MainClass.DeletePIDFile();
			return;
		}
		
		/// <summary>
		/// Create grub.pid file in configuration directory with process ID.
		/// </summary>
		public static void CreatePIDFile()
		{
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
					Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar + "grub.pid";
			StreamWriter pidfile = new StreamWriter(filepath);
			pidfile.Write(Process.GetCurrentProcess().Id);
			pidfile.Flush();
			pidfile.Close();
			pidfile.Dispose();
		}
		
		/// <summary>
		/// Function delete grub.pid file.
		/// </summary>
		public static void DeletePIDFile()
		{
			string filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
					Path.DirectorySeparatorChar + ".grub" + Path.DirectorySeparatorChar + "grub.pid";
			if (File.Exists(filepath))
			{
				StreamReader pidfile = new StreamReader(filepath);
				int pid = Convert.ToInt32(pidfile.ReadToEnd());
				pidfile.Close();
				pidfile.Dispose();
				try
				{
					Process proc = Process.GetProcessById(pid);
					proc.Dispose();
				}
				catch (ArgumentException)
				{
					File.Delete(filepath);
				}
				if (Process.GetCurrentProcess().Id == pid)
				{
					File.Delete(filepath);
				}
			}
		}
	}
}
