// WinMainWindow.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinMainWindow.cs
 * Provide WinFrom Window for program.
 * @fn void Grubng::WinMainWindow::InitializeComponent()
 * Create part of GUI.*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm Window for program.
	/// </summary>
	internal sealed partial class WinMainWindow : Form
	{	
		/// <summary>
		/// Object from threads class. Provide functions to upload, crawl and upload workunits.
		/// </summary>
		Threads workthread;
		/// <summary>
		/// Object from Config class. Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Object from RemoteControl class. Provide functions for web interface.
		/// </summary>
		RemoteControl rControl;
		/// <summary>
		/// Workunit time timer.
		/// </summary>
		System.Timers.Timer worktime;
		/// <summary>
		/// Program uptime
		/// </summary>
		TimeSpan uptimespan;
		/// <summary>
		/// Amount of working crawlers.
		/// </summary>
		int workingcrawlers;
		/// <summary>
		/// Amount of crawlers.
		/// </summary>
		int crawlers;
		/// <summary>
		/// Work status image.
		/// </summary>
		Bitmap workimg;
		/// <summary>
		/// List of current numbers of url in crawler.
		/// </summary>
		List<int> curiter;
		/// <summary>
		/// Stop status image.
		/// </summary>
		Bitmap stopimg;
		/// <summary>
		/// If true program start exit procedure.
		/// </summary>
		bool quitprogram;
		/// <summary>
		/// Array of translated crawler statuses
		/// </summary>
		string[] crawlerstatus;
		/// <summary>
		/// Amount of connection errors which appear in this session.
		/// </summary>
		int connectionerrors;
		/// <summary>
		/// Amount of seconds which program must wait before connect again to servers.
		/// </summary>
		int connectiondelay;
		/// <summary>
		/// Number of selected row.
		/// </summary>
		int rowselected;
		/// <summary>
		/// Amount of working crawlers.
		/// </summary>
		int maxcrawlers;
		/// <summary>
		/// Max download rate.
		/// </summary>
		int maxdownload;
		/// <summary>
		/// Max upload rate
		/// </summary>
		int maxupload;
		/// <summary>
		/// Amount of second of wait on quit from program.
		/// </summary>
		int quitdelay;
		
		/// <summary>
		/// Standard class constructor. Create GUI and show it to user.
		/// </summary>
		public WinMainWindow()
		{
			InitializeComponent();
			this.conf = new Config();
			this.ReadOptions();
			this.stopimg = new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "icon-grub-err.jpg"));			
			this.workimg = new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "icon-grub-run.jpg"));
			this.btnstats.Enabled = false;
			this.quitdelay = 25;
			this.worktime = new System.Timers.Timer();
			this.worktime.Elapsed += new System.Timers.ElapsedEventHandler(OnWorkElapsed);
			this.worktime.Interval = 1000;
			this.worktime.Start();
			this.crawlerstatus = new string[11]{Catalog.GetString("Stopped"),
				Catalog.GetString("Paused"),
				Catalog.GetString("In Queue"),
				Catalog.GetString("Downloading"),
				Catalog.GetString("Download Error"),
				Catalog.GetString("Crawling"),
				Catalog.GetString("Uploading"),
				Catalog.GetString("Upload Error"),
				Catalog.GetString("Waiting"),
				Catalog.GetString("Invalid arc file"),
				Catalog.GetString("Program error")
			};
			this.workthread = new Threads();
			this.lblworkunits2.Text = this.workthread.GetField(0, "overallworkunit").ToString();
			this.lblinvalidarc2.Text = this.workthread.GetField(0, "overallinvalidworkunit").ToString();
			this.lblcrawled2.Text = this.workthread.GetField(0, "overallcrawled").ToString();
			this.lblvalid2.Text = this.workthread.GetField(0, "overallvalid").ToString();
			this.lblinvalid2.Text = this.workthread.GetField(0, "overallinvalid").ToString();
			this.lblerrors2.Text = this.workthread.GetField(0, "overallerror").ToString();
			if (Config.ReadConfig("/configuration/other/enableremote", this.conf.FilePath) == "Y")
			{
				this.rControl = new RemoteControl(this.workthread);
				this.rControl.Start();
				this.rControl.WebEventHandler += new EventHandler<WebEventArgs>(OnWebMessage);
			}
			this.lblcrawlers.Text = "0/" + this.workthread.Crawlers.ToString();
			this.curiter = new List<int>();
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
			    this.tabledet.Rows.Add((i + 1).ToString(), Catalog.GetString("Stopped"), 
				                    "00%", String.Empty, String.Empty, String.Empty);
				this.curiter.Add(0);
			}
			this.tabledet.AutoResizeRows();
			this.tabledet.AllowUserToAddRows = false;
			this.crawlers = this.workthread.Crawlers;
			this.connectiondelay = 0;
			this.connectionerrors = 0;
			this.trayicon.Text = Catalog.GetString("Working crawlers:") + " " + 
				"0/" + this.workthread.Crawlers.ToString();
			//Check for automatic mode
			if (Config.ReadConfig("/configuration/work/automatic", this.conf.FilePath) == "Y")
			{
				this.cbtnauto.Checked = true;
				this.workthread.Running = true;
				this.OnRunClick(null, null);
			}
			this.Activated += new EventHandler(OnActivate);
			this.Activate();
		}
		
		/// <summary>
		/// Function hide main window if option run program hidden is set on true. Raised on activate window.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnActivate(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			string runhidden = Config.ReadConfig("/configuration/other/runhidden", this.conf.FilePath);
			if (runhidden == "Y")
			{
				this.Hide();
			}
		}
		
		/// <summary>
		/// Function quit from program.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnQuit(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.workthread.WriteStats();
			this.quitprogram = true;
			this.UseWaitCursor = true;
		}
		
		/// <summary>
		/// Function show list of crawlers.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void ShowDetails(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tabcontrol.SelectedIndex = 0;
			this.btndetails.Enabled = false;
			this.btnstats.Enabled = true;
			this.btnoptions.Enabled = true;
		}
		
		/// <summary>
		/// Function show program stats.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void ShowStats(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tabcontrol.SelectedIndex = 1;
			this.btnstats.Enabled = false;
			this.btndetails.Enabled = true;
			this.btnoptions.Enabled = true;
		}
		
		/// <summary>
		/// Function show crawling preferences
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void ShowPreferences(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tabcontrol.SelectedIndex = 2;
			this.btnoptions.Enabled = false;
			this.btndetails.Enabled = true;
			this.btnstats.Enabled = true;
		}
		
		/// <summary>
		/// Function show README.txt file.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		static void OnHelp(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			WinHelpDialog helpdlg = new WinHelpDialog("README.txt");
			DialogResult result = helpdlg.ShowDialog();
			if (result != DialogResult.None)
			{
				helpdlg.Dispose();
			}
		}
		
		/// <summary>
		/// Function show dialog with informations about program.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		static void OnAbout(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			WinAboutDialog aboutdlg = new WinAboutDialog();
			DialogResult result = aboutdlg.ShowDialog();
			if (result != DialogResult.None)
			{
				aboutdlg.Dispose();
			}
		}
		
		/// <summary>
		/// Function show preferences dialog and set program preferences.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnPref(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			bool available = this.workthread.CrawlersStopped();
			WinPrefDialog prefdlg = new WinPrefDialog(available);
			DialogResult result = DialogResult.Yes;
			string enableremote;
			while (result == DialogResult.Yes)
			{
				result = prefdlg.ShowDialog();
				if (result == DialogResult.Ignore)
				{
					this.workthread.SetField(0, "crawlerstats", 0);
					this.lblworkunits.Text = "0";
					this.lblcrawled.Text = "0";
					this.lblvalid.Text = "0";
					this.lblinvalid.Text = "0";
					this.lblerrors.Text = "0";
					this.lblreceived.Text = "0";
					this.lblsend.Text = "0";
				}
				if ((result == DialogResult.OK) || (result == DialogResult.Yes))
				{
					this.workthread.SetPrefs(false);
					enableremote = Config.ReadConfig("/configuration/other/enableremote", this.conf.FilePath);
					if ((this.rControl == null) && (enableremote == "Y"))
					{
						this.rControl = new RemoteControl(this.workthread);
						this.rControl.Start();
						this.rControl.WebEventHandler += new EventHandler<WebEventArgs>(OnWebMessage);
					}
					if ((this.rControl != null) && (enableremote == "N"))
					{
						this.rControl.Stop();
						this.rControl.WebEventHandler -= OnWebMessage;
						this.rControl = null;
					}
				}
			}
			prefdlg.Dispose();
		}
			
		delegate void DelegateUpdateText(Label label, string text);
		delegate void DelegateUpdateImage(Bitmap imgstatus);
		delegate void DelegateUpdateButton(Bitmap btnimage, string btnlabel);
		delegate string DelegateReadTable(int row, int item);
		delegate void DelegateUpdateTable(int row, int item, string val);
		delegate void DelegateQuit();
		delegate void DelegateUpdateTooltip(int working);
		delegate void DelegateDeleteRows();
		delegate void DelegateUpdateSettings();
		
		/// <summary>
		/// Timer for update gui, check net connection, close program.
		/// Raised every second when program works.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="ElapsedEventArgs"/> Unused
		/// </param>
		void OnWorkElapsed(object sender, System.Timers.ElapsedEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			TimeSpan tmpspan = new TimeSpan(0, 0, 1);
			this.uptimespan += tmpspan;
			string[] time = this.uptimespan.ToString().Split('.');
			if (this.uptimespan.Days == 1)
			{
				time[0] = time[0] + " " + Catalog.GetString("day") + " " + time[1];
			}
			if (this.uptimespan.Days > 1)
			{
				time[0] = time[0] + " " + Catalog.GetString("days") + " " + time[1];
			}
			this.Invoke(new DelegateUpdateText(UpdateText), this.lbluptime, time[0]);
			int stoped = 0;
			int working = 0;
			int paused = 0;
			double mbused = 0;
			double[] diskusage = new double[this.workthread.Crawlers];
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{		
				switch (this.workthread.GetField(i, "working"))
				{
				case 0:
					stoped++;
					break;
				case 1:
					working++;
					break;
				case 2:
					paused++;
					break;
				}
				diskusage[i] = this.workthread.CountDiskUsage(i);
				mbused += diskusage[i];
			}
			if (paused != this.workthread.Crawlers)
			{
				this.CheckNet();
			}
			if ((working > 0) && (paused == 0) && (this.pcstatus.Image != this.workimg))
			{
				this.Invoke(new DelegateUpdateImage(this.UpdateImage), this.workimg);
			}
			if (Convert.ToInt32(this.lblworkunits.Text) != this.workthread.GetField(0, "workunitsend"))
			{
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblworkunits, this.workthread.GetField(0, "workunitsend").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblworkunits2, this.workthread.GetField(0, "overallworkunit").ToString());
			}
			if (Convert.ToInt32(this.lblinvalidarc.Text) != this.workthread.GetField(0, "invalidarc"))
			{
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblinvalidarc, this.workthread.GetField(0, "invalidarc").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblinvalidarc2, 
				            this.workthread.GetField(0, "overallinvalidworkunit").ToString());
			}
			if (Convert.ToDouble(this.lbldiskusage.Text) != mbused)
			{
				this.Invoke(new DelegateUpdateText(UpdateText), this.lbldiskusage, mbused.ToString());
			}
			if (Convert.ToInt32(this.lblcrawled.Text) != this.workthread.GetField(0, "crawledurls"))
			{
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblcrawled, this.workthread.GetField(0, "crawledurls").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblvalid, this.workthread.GetField(0, "validurls").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblinvalid, this.workthread.GetField(0, "invalidurls").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblerrors, this.workthread.GetField(0, "errorurls").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblcrawled2, this.workthread.GetField(0, "overallcrawled").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblvalid2, this.workthread.GetField(0, "overallvalid").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblinvalid2, this.workthread.GetField(0, "overallinvalid").ToString());
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblerrors2, this.workthread.GetField(0, "overallerror").ToString());
			}
			double tmp = Math.Round(this.workthread.GetDoubleField("uptimereceived"), 1);
			this.Invoke(new DelegateUpdateText(UpdateText), this.lblreceived, tmp.ToString());
			tmp = Math.Round(this.workthread.GetDoubleField("uptimesend"), 1);
			this.Invoke(new DelegateUpdateText(UpdateText), this.lblsend, tmp.ToString());
			tmp = Math.Round(this.workthread.GetDoubleField("overallreceived"), 1);
			this.Invoke(new DelegateUpdateText(UpdateText), this.lblreceived2, tmp.ToString());
			tmp = Math.Round(this.workthread.GetDoubleField("overallsent"), 1);
			this.Invoke(new DelegateUpdateText(UpdateText), this.lblsend2, tmp.ToString());
			if ((this.crawlers != this.workthread.MaxCrawlers) || (this.workingcrawlers != working))
			{
				this.Invoke(new DelegateUpdateText(UpdateText), this.lblcrawlers, working.ToString() + "/" + 
				            this.workthread.MaxCrawlers.ToString());
				this.crawlers = this.workthread.MaxCrawlers;
				this.workingcrawlers = working;
			}
			this.Invoke(new DelegateUpdateTooltip(UpdateTooltip), working);
			string temp = null;
			tmp = 0;
			int row;
			if ((this.curiter.Count > this.workthread.Crawlers) && (this.workthread.Crawlers == this.workthread.MaxCrawlers))
			{
				this.Invoke(new DelegateDeleteRows(DeleteRows));
			}
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				temp = (string)this.Invoke(new DelegateReadTable(ReadTable), i, 0);
				row = Convert.ToInt32(temp) - 1;
				temp = (string)this.Invoke(new DelegateReadTable(ReadTable), row, 1);
				if (temp != this.crawlerstatus[this.workthread.GetField(i, "status")])
				{
					this.Invoke(new DelegateUpdateTable(UpdateTable), row, 1, 
					            this.crawlerstatus[this.workthread.GetField(i, "status")]);
				}
				if (this.workthread.GetField(i, "status") == 6)
				{
					tmp = this.workthread.UploadProgress();
					if (tmp < 0)
					{
						tmp = 0;
					}
					if (tmp > 100)
					{
						tmp = 100;
					}
					if ((tmp < int.MinValue) || (tmp > int.MaxValue))
					{
						tmp = 0;
					}
					if (tmp >= 10)
					{
						this.Invoke(new DelegateUpdateTable(UpdateTable), row, 2, 
						            Convert.ToInt32(tmp).ToString() + " %");
					}
					else
					{
						this.Invoke(new DelegateUpdateTable(UpdateTable), row, 2, "0" + 
						            Convert.ToInt32(tmp).ToString() + " %");
					}
				}
				if (this.curiter[i] != this.workthread.GetField(i, "curiter"))
				{
					if (this.workthread.GetField(i, "status") == 5)
					{
						tmp = (double)this.workthread.GetField(i, "curiter") / (double)(this.workthread.GetField(i, "iter") - 1) * 100;
					}
					if (tmp < 0)
					{
						tmp = 0;
					}
					if (tmp > 100)
					{
						tmp = 100;
					}
					if (tmp >= 10)
					{
						this.Invoke(new DelegateUpdateTable(UpdateTable), row, 2, 
						            Convert.ToInt32(tmp).ToString() + " %");
					}
					else
					{
						this.Invoke(new DelegateUpdateTable(UpdateTable), row, 2, "0" + 
						            Convert.ToInt32(tmp).ToString() + " %");
					}
					this.curiter[i] = this.workthread.GetField(i, "curiter");
				}
				if (this.workthread.GetField(i, "status") == 0)
				{
					this.Invoke(new DelegateUpdateTable(UpdateTable), row, 2, "00%");
				}
				temp = (string)this.Invoke(new DelegateReadTable(ReadTable), row, 3);
				if (temp != this.workthread.GetStringField(i, "workfilename"))
				{
					this.Invoke(new DelegateUpdateTable(UpdateTable), row, 3, this.workthread.GetStringField(i, "workfilename"));
				}
				temp = (string)this.Invoke(new DelegateReadTable(ReadTable), row, 4);
				if (temp != this.workthread.GetStringField(i, "downloaddate"))
				{
					this.Invoke(new DelegateUpdateTable(UpdateTable), row, 4, this.workthread.GetStringField(i, "downloaddate"));
				}
				temp = (string)this.Invoke(new DelegateReadTable(ReadTable), row, 5);
				if (temp != diskusage[i].ToString())
				{
					this.Invoke(new DelegateUpdateTable(UpdateTable), row, 5, diskusage[i].ToString());
				}
			}
			this.Invoke(new DelegateUpdateSettings(UpdateSettings));
			if (this.connectiondelay > 0)
			{
				this.connectiondelay--;
			}
			if (stoped > 0)
			{
				if (!this.cbtnauto.Checked)
				{
					if (working == 0)
					{
						this.Invoke(new DelegateUpdateImage(UpdateImage), this.stopimg);
						this.Invoke(new DelegateUpdateButton(UpdateButton), 
						            new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "run.png")), 
						            this.btnrun.Text = Catalog.GetString("Start"));
					}
				}
				else
				{
					if (this.workthread.ConnectionError)
					{
						this.connectionerrors ++;
						this.connectiondelay = 30 * this.connectionerrors;
						this.workthread.ConnectionError = false;
						this.workthread.Running = false;
					}
					if ((this.connectiondelay <= 0) && (!this.workthread.Running))
					{
						this.workthread.Running = true;
						this.workthread.Start();
					}
				}
			}
			if (this.quitprogram)
			{
				if (!this.workthread.CrawlersStopped())
				{
					if (this.cbtnauto.Checked)
					{
						this.cbtnauto.Checked = false;
					}
					if (this.btnrun.Text != Catalog.GetString("Start"))
					{
						this.Invoke(new DelegateUpdateImage(UpdateImage), 
						            new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
						                                              System.IO.Path.DirectorySeparatorChar + "icon-grub-pause.jpg")));
						this.Invoke(new DelegateUpdateButton(UpdateButton), 
						            new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + 
						                                              System.IO.Path.DirectorySeparatorChar + "run.png")), 
						            this.btnrun.Text = Catalog.GetString("Start"));
					}
					for (int i = 0; i < this.workthread.Crawlers; i++)
					{
						this.workthread.SetField(i, "working", 0);
					}
					this.quitdelay --;
					if (this.quitdelay == 0)
					{
						this.workthread.WriteStats();
						MainClass.DeletePIDFile();
						System.Diagnostics.Process proc = System.Diagnostics.Process.GetCurrentProcess();
						proc.Kill();
					}
				}
				else
				{
					this.worktime.Stop();
					this.workthread.WriteStats();
					Application.Exit();
				}
			}
		}
		
		/// <summary>
		/// Function update client settings from other thread
		/// </summary>
		void UpdateSettings()
		{
			bool changedsettings = false;
			if (this.tbmaxcrawlers.Value != this.maxcrawlers)
			{
				this.maxcrawlers = this.tbmaxcrawlers.Value;
				changedsettings = true;
			}
			if (this.tbmaxdownload.Value != this.maxdownload)
			{
				this.maxdownload = this.tbmaxdownload.Value;
				changedsettings = true;
			}
			if (this.tbmaxupload.Value != this.maxupload)
			{
				this.maxupload = this.tbmaxupload.Value;
				changedsettings = true;
			}
			if (changedsettings)
			{
				string[] xmlnodes = new string[] {
					"/configuration/work/crawlers",
					"/configuration/work/downloadrate",
					"/configuration/work/uploadrate"
				};
				string[] xmlvalues = new string[] {
					this.tbmaxcrawlers.Value.ToString(),
					this.tbmaxdownload.Value.ToString(),
					this.tbmaxupload.Value.ToString()
				};
				Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
				changedsettings = false;
				this.workthread.SetPrefs(false);
				this.RecountCrawlers();
			}
		}
		
		/// <summary>
		/// Function delete unused rows from crawlers table from other thread.
		/// </summary>
		void DeleteRows()
		{
			int tempindex;
			for (int i = 0; i < this.curiter.Count; i++)
			{
				tempindex = Convert.ToInt32(this.tabledet[0, i].Value);
				if (tempindex > this.workthread.Crawlers)
				{
					this.tabledet.Rows.RemoveAt(i);
					this.curiter.RemoveAt((this.curiter.Count - 1));
					i--;
				}
			}
			this.tabledet.Refresh();
		}
		
		/// <summary>
		/// Function update labels from other thread.
		/// </summary>
		/// <param name="label">
		/// A <see cref="Label"/> to update.
		/// </param>
		/// <param name="text">
		/// A <see cref="System.String"/> text to add to label.
		/// </param>
		static void UpdateText(Label label, string text)
		{
			label.Text = text;
		}
		
		/// <summary>
		/// Function update status image from other thread.
		/// </summary>
		/// <param name="imgstatus">
		/// A <see cref="Bitmap"/> to add to pcstatus.
		/// </param>
		void UpdateImage(Bitmap imgstatus)
		{
			this.pcstatus.Image = imgstatus;
		}
		
		/// <summary>
		/// Function update btnrun from other thread.
		/// </summary>
		/// <param name="btnimage">
		/// A <see cref="Bitmap"/> to add to btnrun
		/// </param>
		/// <param name="text">
		/// A <see cref="System.String"/> text to add to btnrun
		/// </param>
		void UpdateButton(Bitmap btnimage, string text)
		{
			this.btnrun.Image = btnimage;
			this.btnrun.Text = text;
		}
		
		/// <summary>
		/// Function read value from crawlers table from other thread.
		/// </summary>
		/// <param name="row">
		/// A <see cref="System.Int32"/> number of row to read
		/// </param>
		/// <param name="item">
		/// A <see cref="System.Int32"/> number of column to read
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> value of cell in table
		/// </returns>
		string ReadTable(int row, int item)
		{
			return this.tabledet[item, row].Value.ToString();
		}
		
		/// <summary>
		/// Function set value to crawlers table from other thread.
		/// </summary>
		/// <param name="row">
		/// A <see cref="System.Int32"/> number of row to write
		/// </param>
		/// <param name="item">
		/// A <see cref="System.Int32"/> number of column to write
		/// </param>
		/// <param name="val">
		/// A <see cref="System.String"/> value to write
		/// </param>
		void UpdateTable(int row, int item, string val)
		{
			this.tabledet[item, row].Value = val;
		}
		
		/// <summary>
		/// Function update trayicon tooltip.
		/// </summary>
		/// <param name="working">
		/// A <see cref="System.Int32"/> amount of working crawlers
		/// </param>
		void UpdateTooltip(int working)
		{
			this.trayicon.Text = Catalog.GetString("Working crawlers:") + " " + 
				working.ToString() + "/" + this.workthread.MaxCrawlers.ToString();
		}
		
		/// <summary>
		/// Start or pause web crawling. Raised when button Run/Pause clicked. 
		/// Function start new thread for crawling
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnRunClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			bool stoped = false;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				if ((this.workthread.GetField(i, "working") == 0) || (this.workthread.GetField(i, "working") == 2))
				{
					stoped = true;
					break;
				}
			}
			if (stoped)
			{
				this.btnrun.Image = new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "pause.png"));
				this.btnrun.Text = Catalog.GetString("Pause");
				this.workthread.WriteStats();
				this.workthread.TimeOut = 0;
				this.workthread.Start();
			}
			else
			{
				this.pcstatus.Image = new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "icon-grub-pause.jpg"));
				this.btnrun.Image = new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "run.png"));
				this.btnrun.Text = Catalog.GetString("Start");
				for (int i = 0; i < this.workthread.Crawlers; i++)
				{
					this.workthread.SetField(i, "working", 2);
				}
			}
		}
		
		/// <summary>
		/// Function check internet connection. If there is more than 49 request timeout in row, function ping net
		/// for check internet connection. If fail, pause all crawlers and show information to user about problems with connection.
		/// </summary>
		void CheckNet()
		{
			bool check = true;
			if (this.workthread.TimeOut > 49)
			{
				System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.WebRequest.Create("http://grub.org/");
				request.Timeout = 5000;
				request.ProtocolVersion = System.Net.HttpVersion.Version11;
				request.UserAgent = "GrubNG";
				request.KeepAlive = false;
				request.Proxy = null;
				request.AllowAutoRedirect = false;
				try
				{
					System.Net.HttpWebResponse response = (System.Net.HttpWebResponse)request.GetResponse();
					response.Close();
				}
				catch (System.Net.WebException e)
				{
					int error = (int)e.Status;
					if (error == 1)
					{
						check = false;
						for (int i = 0; i < this.workthread.Crawlers; i++)
						{
							this.workthread.SetField(i, "working", 2);
						}
						this.Invoke(new DelegateUpdateImage(this.UpdateImage), new Bitmap(System.IO.Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, "Images" + System.IO.Path.DirectorySeparatorChar + "icon-grub-pause.jpg")));
						WinMessageBox.Show(Catalog.GetString("Couldn't connect to several sites. ") + 
						                   Catalog.GetString("Possibly your computer isn't connected with the Internet anymore. ") + 
						                   Catalog.GetString("Please check the Internet connection."));
					}
				}
				if (check)
				{
					this.workthread.TimeOut = 0;
				}
			}
		}
		
		/// <summary>
		/// Function show details about selected crawler. Raised when user double click on row.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="MouseEventArgs"/> button clicked and coordinates of click
		/// </param>
		void OnDoubleClick(object sender, MouseEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e.Button == MouseButtons.Left)
			{
				DataGridView.HitTestInfo hit = this.tabledet.HitTest(e.X, e.Y);
				if (hit.Type == DataGridViewHitTestType.Cell)
				{
					int crawler = (Convert.ToInt32(this.tabledet.Rows[hit.RowIndex].Cells[0].Value) - 1);
					WinCrawlerInfo crawlerdlg = new WinCrawlerInfo(crawler, this.workthread);
					DialogResult result = crawlerdlg.ShowDialog();
					if (result != DialogResult.None)
					{
						crawlerdlg.Dispose();
					}
				}
			}
		}
		
		/// <summary>
		/// Function resized tablepanel. Raised when window is resized.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnResize(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			int x = this.Size.Width - 648;
			int y = this.Size.Height - 477;
			this.tabcontrol.Size = new Size(528 + x, 365 + y);
			this.tabledet.Size = this.tabcontrol.Size;
		}
		
		delegate void DelegateHide();
		delegate void DelegateShow();
		
		/// <summary>
		/// Function show/hide main window. Raised when left button mouse clicked on tray icon.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="MouseEventArgs"/> mouse button
		/// </param>
		void OnTrayClick(object sender, MouseEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args.Button == MouseButtons.Left)
			{
				if (this.Visible)
				{
					this.Invoke(new DelegateHide(this.Hide));
				}
				else
				{
					this.Invoke(new DelegateShow(this.Show));
				}
			}
		}
		
		/// <summary>
		/// Function show main window. Raised when selected option show window from tray icon menu.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnShowClick(object sender, EventArgs e)
		{
			this.Invoke(new DelegateShow(this.Show));
		}
		
		/// <summary>
		/// Function get number of selected row. Raised when user right click on row.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="MouseEventArgs"/> Button and coordinates of click by mouse.
		/// </param>
		void OnMouseDown(object sender, MouseEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e.Button == MouseButtons.Right)
			{
				DataGridView.HitTestInfo hit = this.tabledet.HitTest(e.X, e.Y);
				if (hit.Type == DataGridViewHitTestType.Cell)
				{
					this.rowselected = (Convert.ToInt32(this.tabledet.Rows[hit.RowIndex].Cells[0].Value) - 1);
				}
			}
		}
		
		/// <summary>
		/// Function show details about selected crawler. Raised when details selected from context menu.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnDetailsClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			if (this.rowselected != -1)
			{
				WinCrawlerInfo crawlerdlg = new WinCrawlerInfo(this.rowselected, this.workthread);
				DialogResult result = crawlerdlg.ShowDialog();
				if (result != DialogResult.None)
				{
					crawlerdlg.Dispose();
				}
			}
		}
		
		/// <summary>
		/// Function run/pause selected crawler. Raised when user select option
		/// run from context menu
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/>
		/// </param>
		void OnRunPauseClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			switch (this.workthread.GetField(this.rowselected, "working"))
			{
			case 0:
				this.workthread.StartCrawler(this.rowselected);
				break;
			case 1:
				this.workthread.SetField(this.rowselected, "working", 2);
				break;
			case 2:
				this.workthread.SetField(this.rowselected, "working", 1);
				break;
			default:
				break;
			}
		}
		
		/// <summary>
		/// Start automatic mode. Raises when Automatic button toggled.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnAutoClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			int stoped = 0;
			for (int i = 0; i < this.workthread.Crawlers; i++)
			{
				if (this.workthread.GetField(i, "working") == 0)
				{
					stoped++;
				}
			}
			if (this.cbtnauto.Checked)
			{
				//Start crawling
				if (stoped == this.workthread.Crawlers)
				{
					this.workthread.Running = true;
					this.OnRunClick(null, null);
				}
			}
			else
			{
				this.workthread.Running = false;
			}
		}
		
		/// <summary>
		/// Function override standard OnFormClosing function. If raised by quit from program it close window, if raised by
		/// close window button it hide window.
		/// </summary>
		/// <param name="e">
		/// A <see cref="FormClosingEventArgs"/> used CloseReason fields for check reason of raised close window event.
		/// </param>
		protected override void OnFormClosing(FormClosingEventArgs e)
        {           
			if (e.CloseReason == CloseReason.UserClosing)
			{
				e.Cancel = true;
				this.Hide();
			}
        }
		
		/// <summary>
		/// Start create sitemap. Raise when menu entry CreateSitemap clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnCreateSitemap(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			WinSitemapURLDialog mapdlg = new WinSitemapURLDialog();
			mapdlg.Show();
		}
	
		/// <summary>
		/// Function show changed amount of working crawlers. Raised when option working crawlers changed
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnMaxCrawlersScroll(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.nudmaxcrawlers.Value = this.tbmaxcrawlers.Value;
		}
		
		/// <summary>
		/// Function show changed amount of max download speed. Raised when option max download speed changed.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnMaxDownloadScroll(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.nudmaxdownload.Value = this.tbmaxdownload.Value;
		}
		
		/// <summary>
		/// Function show changed amount of max upload speed. Raised when option max upload speed changed
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnMaxUploadScroll(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.nudmaxupload.Value = this.tbmaxupload.Value;
		}
		
		/// <summary>
		/// Function update selected value on tbmaxcrawlers. Raised when value changed in nudmaxcrawlers.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnCrawlersChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tbmaxcrawlers.Value = (int)this.nudmaxcrawlers.Value;
		}
		
		/// <summary>
		/// Function update selected value on tbmaxdownload. Raised when value changed on nudmaxdownload.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/>
		/// </param>
		void OnDownloadChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tbmaxdownload.Value = (int)this.nudmaxdownload.Value;
		}
		
		/// <summary>
		/// Function update selected value on tbmaxupload. Raised when value changed in nudmaxupload.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/>
		/// </param>
		void OnUploadChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tbmaxupload.Value = (int)this.nudmaxupload.Value;
		}
		
		/// <summary>
		/// Function read options from config file and set it on GUI
		/// </summary>
		void ReadOptions()
		{
			string[] nodes = new string[] {
				"/configuration/work/crawlers",
				"/configuration/work/downloadrate",
				"/configuration/work/uploadrate"};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			nodes = null;
			this.tbmaxcrawlers.Value = Convert.ToInt32(values[0]);
			this.nudmaxcrawlers.Value = this.tbmaxcrawlers.Value;
			this.maxcrawlers = this.tbmaxcrawlers.Value;
			this.tbmaxdownload.Value = Convert.ToInt32(values[1]);
			this.nudmaxdownload.Value = this.tbmaxdownload.Value;
			this.maxdownload = this.tbmaxdownload.Value;
			this.tbmaxupload.Value = Convert.ToInt32(values[2]);
			this.nudmaxupload.Value = this.tbmaxupload.Value;
			this.maxupload = this.tbmaxupload.Value;
		}
		
		/// <summary>
		/// Function update configuration file or quit from program. Raised when program received message from
		/// web interface.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="WebEventArgs"/> type of even, "quit" or "config"
		/// </param>
		void OnWebMessage(object sender, WebEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args.Type == "quit")
			{
				this.OnQuit(null, null);
			}
			if (args.Type == "config")
			{
				this.ReadOptions();
				this.workthread.SetPrefs(false);
				this.RecountCrawlers();
			}
		}
		
		/// <summary>
		/// Function synchronize amount of crawlers
		/// </summary>
		void RecountCrawlers()
		{
			if (this.curiter.Count < this.workthread.Crawlers)
			{
				//Check for amount of crawlers
				int tempcrawlers = this.workthread.Crawlers - this.curiter.Count;
				for (int i = 0; i < tempcrawlers; i++)
				{
					this.tabledet.Rows.Add((this.curiter.Count + 1).ToString(), Catalog.GetString("Stopped"), "00%", 
					                       String.Empty, String.Empty, String.Empty);
					this.curiter.Add(0);
				}
				this.tabledet.Refresh();
				if (this.workthread.Running)
				{
					this.workthread.Start();
				}
			}
			if (this.curiter.Count > this.workthread.Crawlers)
			{
				int tempindex;
				for (int i = 0; i < this.curiter.Count; i++)
				{
					tempindex = Convert.ToInt32(this.tabledet[0, i].Value);
					if (tempindex > this.workthread.Crawlers)
					{
						this.tabledet.Rows.RemoveAt(i);
						this.curiter.RemoveAt((this.curiter.Count - 1));
						i--;
					}
				}
				this.tabledet.Refresh();
			}
		}

		/// <summary>
		/// Function override standard WinForm function Dispose. Dispose trayicon and main window.
		/// </summary>
		/// <param name="disposing">
		/// A <see cref="System.Boolean"/> If true, dispose all components.
		/// </param>
		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				if (this.container != null)
				{
					this.container.Dispose();
				}
			}
			this.btnrun.Dispose();
			this.btnquit.Dispose();
			this.cbtnauto.Dispose();
			this.btndetails.Dispose();
			this.btnstats.Dispose();
			this.tabcontrol.Dispose();
			this.menu.Dispose();
			this.trayicon.Dispose();
			this.pcstatus.Dispose();
			this.lbluptime.Dispose();
			this.lblcrawlers.Dispose();
			this.lbldiskusage.Dispose();
			this.lblworkunits.Dispose();
			this.lblinvalidarc.Dispose();
			this.lblcrawled.Dispose();
			this.lblvalid.Dispose();
			this.lblinvalid.Dispose();
			this.lblerrors.Dispose();
			this.lblreceived.Dispose();
			this.lblsend.Dispose();
			this.lblworkunits2.Dispose();
			this.lblinvalidarc2.Dispose();
			this.lblcrawled2.Dispose();
			this.lblvalid2.Dispose();
			this.lblinvalid2.Dispose();
			this.lblerrors2.Dispose();
			this.lblreceived2.Dispose();
			this.lblsend2.Dispose();
			this.tabledet.Dispose();
			this.datatab.Dispose();
			this.iconmenu.Dispose();
			this.workthread.Dispose();
			this.workimg.Dispose();
			this.stopimg.Dispose();
			this.tbmaxcrawlers.Dispose();
			this.nudmaxcrawlers.Dispose();
			this.tbmaxdownload.Dispose();
			this.nudmaxdownload.Dispose();
			this.tbmaxupload.Dispose();
			this.nudmaxupload.Dispose();
			this.btnmoreoptions.Dispose();
			base.Dispose(disposing);
		}
	}
}
