// WinPrefDialog.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinPrefDialog.cs
 * Provide WinFrom Window for set program preferences.
 * @fn void Grubng::WinPrefDialog::InitializeComponent()
 * Create GUI.*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm Window for set program preferences.
	/// </summary>
	internal sealed partial class WinPrefDialog : Form
	{
		/// <summary>
		/// Object from Config class. Provide function to save, read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Used to check availability of workdir preference. Can be true only when program not working.
		/// </summary>
		bool available;
		
		/// <summary>
		/// Standard class constructor. Create GUI and show it to user.
		/// </summary>
		/// <param name="avail">
		/// A <see cref="System.Boolean"/> Avaiability of working directory preference
		/// </param>
		public WinPrefDialog(bool avail)
		{
			InitializeComponent();
			this.available = avail;
			this.conf = new Config();
			string[] nodes = new string[] 
			{
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/workserver",
				"/configuration/work/workdir",
				"/configuration/work/compression",
				"/configuration/work/automatic",
				"/configuration/other/runhidden",
				"/configuration/work/proxystatus",
				"/configuration/other/updateserver",
				"/configuration/other/checkupdates",
				"/configuration/other/unstable",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"/configuration/work/createsitemaps",
				"/configuration/other/enableremote",
				"/configuration/other/remoteport"
			};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			this.edtuser.Text = values[0];
			this.edtpass.Text = values[1];
			this.edtworkserv.Text = values[2];
			this.edtdir.Text = values[3];
			if (values[4] == ".gz")
			{
				this.rbtngzip.Checked = true;
			}
			else
			{
				this.rbtnlzma.Checked = true;
			}
			if (values[5] == "Y")
			{
				this.cbtnauto.Checked = true;
			}
			else
			{
				this.cbtnauto.Checked = false;
			}
			if (values[6] == "Y")
			{
				this.cbtnrunhidden.Checked = true;
			}
			else
			{
				this.cbtnrunhidden.Checked = false;
			}
			switch (values[7])
			{
			case "none":
				this.rbtnnone.Checked = true;
				break;
			case "default":
				this.rbtndefault.Checked = true;
				break;
			case "custom":
				this.rbtncustom.Checked = true;
				this.tblpanel4.Enabled = true;
				nodes = new string[] {
					"/configuration/work/proxyaddress",
					"/configuration/work/proxyport",
					"/configuration/work/proxylogin",
					"/configuration/work/proxypass"};
				string[] proxyvals = Config.ReadFew(nodes, this.conf.FilePath);
				this.edtproxyserv.Text = proxyvals[0];
				this.edtproxyport.Text = proxyvals[1];
				this.edtproxylogin.Text = proxyvals[2];
				this.edtproxypass.Text = proxyvals[3];
				proxyvals = null;
				break;
			}
			this.edtupdateserv.Text = values[8];
			if (values[9] == "Y")
			{
				this.cbtnupdate.Checked = true;
				this.grpbox2.Enabled = true;
				this.btncheck.Enabled = true;
			}
			else
			{
				this.cbtnupdate.Checked = false;
				this.grpbox2.Enabled = false;
				this.btncheck.Enabled = false;
			}
			if (values[10] == "Y")
			{
				this.rbtnunstable.Checked = true;
			}
			else
			{
				this.rbtnstable.Checked = true;
			}
			this.edtmapserv.Text = values[11];
			if (values[12] == "Y")
			{
				this.cbtndeletemap.Checked = true;
			}
			else
			{
				this.cbtndeletemap.Checked = false;
			}
			if (values[13] == "Y")
			{
				this.cbtncreatesitemaps.Checked = true;
			}
			else
			{
				this.cbtncreatesitemaps.Checked = false;
			}
			if (values[14] == "Y")
			{
				this.tblpanel7.Enabled = true;
				this.cbtnenableremote.Checked = true;
			}
			else
			{
				this.tblpanel7.Enabled = false;
				this.cbtnenableremote.Checked = false;
			}
			this.edtremoteport.Text = values[15];
			values = null;
			this.btnapply.Enabled = false;
			this.btnok.Enabled = false;
		}
		
		/// <summary>
		/// Function close window, without saving changes. Raised when btncancel clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> Unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> Unused
		/// </param>
		void OnCancel(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Close();
		}
		
		/// <summary>
		/// Function show Folder Browser dialog.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnDirSelect(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			FolderBrowserDialog browsedlg = new FolderBrowserDialog();
			browsedlg.Description = Catalog.GetString("Select work directory for program");
			DialogResult result = browsedlg.ShowDialog();
			if (result == DialogResult.OK)
			{
				this.edtdir.Text = browsedlg.SelectedPath + System.IO.Path.DirectorySeparatorChar;
				this.OnChange(null, null);
			}
			browsedlg.Dispose();
		}
		
		/// <summary>
		/// Function unblock buttons btnok and btnapply. Raised when any option are changed.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnChange(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.btnok.Enabled = true;
			this.btnapply.Enabled = true;
		}
			
		/// <summary>
		/// Function enable or disable ability to set custom proxy settings. Raised when proxy setting selected.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnCustomProxy(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tblpanel4.Enabled = this.rbtncustom.Checked;
			this.OnChange(null, null);
		}
		
		/// <summary>
		/// Function enable or disable ability to set remote control port. Raised when cbtnenableremote selected.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		void OnRemoteControl(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.tblpanel7.Enabled = this.cbtnenableremote.Checked;
			this.OnChange(null, null);
		}
		
		/// <summary>
		/// Function enable or disable ability to set autoupdater. Raised when cbtnupdate checked or unchecked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnUpdater(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.grpbox2.Enabled = this.cbtnupdate.Checked;
			if (this.cbtnupdate.Checked && this.available)
			{
				this.btncheck.Enabled = true;
			}
			else
			{
				this.btncheck.Enabled = false;
			}
			this.OnChange(null, null);
		}
		
		/// <summary>
		/// Clear statistics in grub.xml file.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnClearClick(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			string[] xmlnodes = new string[] {
				"/configuration/statistics/worksend",
				"/configuration/statistics/urlcrawled",
				"/configuration/statistics/success",
				"/configuration/statistics/failure",
				"/configuration/statistics/errors",
				"/configuration/statistics/mbreceived",
				"/configuration/statistics/mbsend",
				"/configuration/statistics/invalidarc"
			};
			string[] xmlvalues = new string[] {"0", "0", "0", "0", "0", "0", "0", "0"};
			Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
		}
			
		/// <summary>
		/// Apply changes in program settings. Raised when btnok or btnapply clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> WinForm Button which emit event
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnOkClick(object sender, EventArgs e)
		{
			if (e != null)
			{
				e = null;
			}
			bool answer = SavePrefs();
			if (answer)
			{
				Button clicked = (Button)sender;
				if (clicked.Name == "btnok")
				{
					this.Close();
				}
				else
				{
					this.btnapply.Enabled = false;
				}
			}
		}
		
		/// <summary>
		/// Function check corectness of values and save settings to grub.xml file.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> True if configuration was saved, otherwise false..
		/// </returns>
		bool SavePrefs()
		{
			//Check for empty fields
			bool checkuser = String.IsNullOrEmpty(this.edtuser.Text);
			bool checkpass = String.IsNullOrEmpty(this.edtpass.Text);
			bool checkwork = String.IsNullOrEmpty(this.edtworkserv.Text);
			if (checkuser || checkpass || checkwork)
			{
				WinMessageBox.Show(Catalog.GetString("Fill empty fields"));
				return false;
			}
			//Check for valid url's
			if (!this.ValidURL(this.edtworkserv.Text))
			{
				return false;
			}
			//Check sitemap server option only when creating sitemap is enabled
			if (this.cbtncreatesitemaps.Checked)
			{
				if (this.edtmapserv.Text.Length == 0)
				{
					WinMessageBox.Show(Catalog.GetString("Fill sitemap server settings"));
					return false;
				}
				if (!this.ValidURL(this.edtmapserv.Text))
				{
					return false;
				}
			}
			//Check updater server option only when auto-updater is enabled
			if (this.cbtnupdate.Checked)
			{
				if (this.edtupdateserv.Text.Length == 0)
				{
					WinMessageBox.Show(Catalog.GetString("Fill updater settings"));
					return false;
				}
				if (!this.ValidURL(this.edtupdateserv.Text))
				{
					return false;
				}
			}
			string[] xmlnodes = new string[] 
			{
				"/configuration/work/proxystatus",
				"/configuration/work/proxyaddress",
				"/configuration/work/proxyport",
				"/configuration/work/proxylogin",
				"configuration/work/proxypass",
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/workdir",
				"/configuration/work/automatic",
				"/configuration/other/runhidden",
				"/configuration/work/workserver",
				"/configuration/work/compression",
				"/configuration/other/updateserver",
				"/configuration/other/checkupdates",
				"/configuration/other/unstable",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"/configuration/work/createsitemaps",
				"/configuration/other/enableremote",
				"/configuration/other/remoteport"
			};
			string[] xmlvalues = new string[xmlnodes.Length];
			//If set custom proxy setting, check for empty fields
			if (this.rbtncustom.Checked)
			{
				bool checkproxy = String.IsNullOrEmpty(this.edtproxyserv.Text);
				bool checkport = String.IsNullOrEmpty(this.edtproxyport.Text);
				if (checkproxy || checkport)
				{
					WinMessageBox.Show(Catalog.GetString("Fill proxy settings"));
					return false;
				}
				//Check valid proxy url
				checkproxy = this.ValidURL(this.edtproxyserv.Text);
				if (!checkproxy)
				{
					return false;
				}
				//Check valid proxy port
				System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^[1-9][0-9]*$");
				if (!rx.IsMatch(this.edtproxyport.Text))
				{
					WinMessageBox.Show(Catalog.GetString("Value must be a number."));
					return false;
				}
				xmlvalues[0] = "custom";
				xmlvalues[1] = this.edtproxyserv.Text;
				xmlvalues[2] = this.edtproxyport.Text;
				xmlvalues[3] = this.edtproxylogin.Text;
				xmlvalues[4] = this.edtproxypass.Text;
			}
			else
			{
				xmlvalues[1] = String.Empty;
				xmlvalues[2] = String.Empty;
				xmlvalues[3] = String.Empty;
				xmlvalues[4] = String.Empty;
				if (this.rbtndefault.Checked)
				{
					xmlvalues[0] = "default";
				}
				else
				{
					xmlvalues[0] = "none";
				}
			}
			xmlvalues[5] = this.edtuser.Text;
			xmlvalues[6] = this.edtpass.Text;
			xmlvalues[7] = this.edtdir.Text;
			if (this.cbtnauto.Checked)
			{
				xmlvalues[8] = "Y";
			}
			else
			{
				xmlvalues[8] = "N";
			}
			if (this.cbtnrunhidden.Checked)
			{
				xmlvalues[9] = "Y";
			}
			else
			{
				xmlvalues[9] = "N";
			}
			xmlvalues[10] = this.edtworkserv.Text;
			if (this.rbtngzip.Checked)
			{
				xmlvalues[11] = ".gz";
			}
			else
			{
				xmlvalues[11] = ".lzma";
			}
			xmlvalues[12] = this.edtupdateserv.Text;
			if (this.cbtnupdate.Checked)
			{
				xmlvalues[13] = "Y";
			}
			else
			{
				xmlvalues[13] = "N";
			}
			if (this.rbtnunstable.Checked)
			{
				xmlvalues[14] = "Y";
			}
			else
			{
				xmlvalues[14] = "N";
			}
			xmlvalues[15] = this.edtmapserv.Text;
			if (this.cbtndeletemap.Checked)
			{
				xmlvalues[16] = "Y";
			}
			else
			{
				xmlvalues[16] = "N";
			}
			if (this.cbtncreatesitemaps.Checked)
			{
				xmlvalues[17] = "Y";
			}
			else
			{
				xmlvalues[17] = "N";
			}
			if (this.cbtnenableremote.Checked)
			{
				//Check valid proxy port
				System.Text.RegularExpressions.Regex rx2 = new System.Text.RegularExpressions.Regex(@"^[1-9][0-9]*$");
				if (!rx2.IsMatch(this.edtremoteport.Text))
				{
					WinMessageBox.Show(Catalog.GetString("Value must be a number."));
					return false;
				}
				xmlvalues[18] = "Y";
			}
			else
			{
				xmlvalues[18] = "N";
			}
			xmlvalues[19] = this.edtremoteport.Text;
			Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
			return true;
		}
		
		/// <summary>
		/// Function check valid URL.
		/// </summary>
		/// <param name="url">
		/// A <see cref="System.String"/> URL to check.
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if URL is valid, otherwise return false.
		/// </returns>
		static bool ValidURL(string url)
		{
			System.Text.RegularExpressions.Regex rx = new System.Text.RegularExpressions.Regex(@"^(ht|f)tp(s?)\:\/\/[0-9a-zA-Z]([-.\w]*[0-9a-zA-Z])*(:(0-9)*)*(\/?)([a-zA-Z0-9\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$");
			if (rx.IsMatch(url))
			{
				return true;
			}
			else
			{
				WinMessageBox.Show(Catalog.GetString("Invalid url"));
				return false;
			}
		}
		
		/// <summary>
		/// Function set availability of options to set workdir and check for updates. Raised when tab changed.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="e">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnTabChanged(object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.btndir.Enabled = this.available;
			this.btncheck.Enabled = this.available;
		}
		
		/// <summary>
		/// Function override System.ComponentModel.Component.Dispose function.
		/// </summary>
		public new void Dispose()
		{
			this.tabcontrol.Dispose();
			this.btnok.Dispose();
			this.btnapply.Dispose();
			this.btncancel.Dispose();
			this.edtuser.Dispose();
			this.edtpass.Dispose();
			this.edtdir.Dispose();
			this.btndir.Dispose();
			this.edtworkserv.Dispose();
			this.rbtnnone.Dispose();
			this.rbtndefault.Dispose();
			this.rbtncustom.Dispose();
			this.tblpanel4.Dispose();
			this.edtproxyserv.Dispose();
			this.edtproxyport.Dispose();
			this.edtproxylogin.Dispose();
			this.edtproxypass.Dispose();
			this.cbtnauto.Dispose();
			this.cbtnrunhidden.Dispose();
			this.edtupdateserv.Dispose();
			this.cbtnupdate.Dispose();
			this.grpbox2.Dispose();
			this.rbtnstable.Dispose();
			this.rbtnunstable.Dispose();
			this.edtmapserv.Dispose();
			this.cbtndeletemap.Dispose();
			this.rbtngzip.Dispose();
			this.rbtnlzma.Dispose();
			this.btncheck.Dispose();
			this.cbtncreatesitemaps.Dispose();
			this.cbtnenableremote.Dispose();
			this.edtremoteport.Dispose();
			this.tblpanel7.Dispose();
			base.Dispose();
		}
		
		/// <summary>
		/// Function resize tabcontrol. Raised when window is resized.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused
		/// </param>
		void OnResize(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			int x = this.Size.Width - 578;
			int y = this.Size.Height - 407;
			this.tabcontrol.Size = new Size(568 + x, 320 + y);
			this.btnok.Location = new Point(480 + x, 340 + y);
			this.btnapply.Location = new Point(392 + x, 340 + y);
			this.btncancel.Location = new Point(304 + x, 340 + y);
		}
		
		/// <summary>
		/// Function check for updates of client. Raised when btncheck clicked.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/> unused.
		/// </param>
		void OnCheckClick (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.btncheck.Enabled = false;
			string unstable = Config.ReadConfig("/configuration/other/unstable", conf.FilePath);
			System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
			string version = asm.GetName().Version.Major.ToString() + "." + asm.GetName().Version.Minor.ToString() + "." + asm.GetName().Version.Build.ToString();
			Updater upd = new Updater(version);
			string update = upd.CheckUpdate(unstable);
			if ((update == "current") || (update == "older"))
			{
				string[] updated = Updater.Update(update, unstable, String.Empty);
				string message = Catalog.GetString("Client successfully updated. ") + 
					Catalog.GetString("You must restart it again. ") + 
						Environment.NewLine + 
						Environment.NewLine + 
						Catalog.GetString("Updated files:") + 
						Environment.NewLine;
				for (int i = 0; i < updated.Length; i++)
				{
					message += (i + 1).ToString() + " " + updated[i] + Environment.NewLine;
				}
				WinMessageBox.Show(message);
			}
			else
			{
				string message;
				if (update == "none")
				{
					message = Catalog.GetString("No available updates.");
				}
				else
				{
					message = Catalog.GetString("Update failed.");
				}
				WinMessageBox.Show(message);
			}
			this.btncheck.Enabled = true;
		}
	}
}
