// WinShowSitemapDialog.cs
// 
// Copyright (C) 2008,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinShowSitemapDialog.cs
 * Provide WinForm window for edit and send sitemap.
 * @fn virtual void Grubng::WinShowSitemapDialog::InitializeComponent()
 * Create GUI*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Mono.Unix;
using System.IO;
using System.Xml;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm window for edit and send sitemap.
	/// </summary>
	public partial class WinShowSitemapDialog : Form
	{
		/// <summary>
		/// Full path to sitemap with filename.
		/// </summary>
		string mapfile;

		/// <summary>
		/// Standard class constructor. Create dialog and show it to user.
		/// </summary>
		/// <param name="sitemap">
		/// A <see cref="System.String"/> full path to sitemap with filename.
		/// </param>
		public WinShowSitemapDialog(string sitemap)
		{
			this.mapfile = sitemap;
			InitializeComponent();
			//Read sitemap file (if exist)
			if (System.IO.File.Exists(this.mapfile))
			{
				this.btnsend.Enabled = this.ReadSitemap();
			}
		}
		
		/// <summary>
		/// Function read existing sitemap and show it in tablemap.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if sitemap was not empty, otherwise false.
		/// </returns>
		bool ReadSitemap()
		{
			string[] extension = this.mapfile.Split('.');
			int index = extension.Length - 1;
			if (extension[index] == "gz")
			{
				FileStream file = File.OpenRead(this.mapfile);
				this.mapfile = this.mapfile.Substring(0, (this.mapfile.Length - 3));
				byte[] buffer = new byte[1024];
				FileStream unfile = File.Create(this.mapfile);
				System.IO.Compression.GZipStream gzstream = new System.IO.Compression.GZipStream(file,
				                                                                                 System.IO.Compression.CompressionMode.Decompress);
				int bytesRead = 0;
				while (true)
				{
					bytesRead = gzstream.Read(buffer, 0, buffer.Length);
					if (bytesRead == 0)
					{
						break;
					}
					unfile.Write(buffer, 0, bytesRead);
				}
				gzstream.Close();
				gzstream.Dispose();;
				buffer = null;
				unfile.Close();
				unfile.Dispose();
				file.Close();
				file.Dispose();
				File.Delete(this.mapfile + ".gz");
			}
			XmlDocument doc = new XmlDocument();
			doc.Load(this.mapfile);
			XmlNode node = doc.SelectSingleNode("/");
			XmlElement url = (XmlElement)node.ChildNodes[1];
			if (url.GetElementsByTagName("url").Count == 0)
			{
				return false;
			}
			for (int i = 1; i < url.GetElementsByTagName("url").Count; i++)
			{
				this.datatab.Rows.Add(url.GetElementsByTagName("url")[i].FirstChild.InnerText);
			}
			this.tablemap.DataSource = this.datatab;
			this.tablemap.ReadOnly = false;
			this.datatab.RowChanged += new System.Data.DataRowChangeEventHandler(OnCellChange);
			doc = null;
			return true;
		}
		
		/**Send created sitemap on server.
		 * Raised when buttonsend clicked.
		 * @param sender Unused
		 * @param e Unused*/
		private void OnBtnSendClick (object sender, System.EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			WinSitemapDialog sitemapdlg = new WinSitemapDialog(this.mapfile);
			DialogResult result = sitemapdlg.ShowDialog();
			if (result == DialogResult.OK)
			{
				sitemapdlg.Dispose();
				this.OnBtnCancelClick(null, null);
			}
		}
		
		/**Function close window.
		 * Raised when btncancel clicked.
		 * @param sender Unused
		 * @param args Unused*/
		private void OnBtnCancelClick(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			this.Close();
			this.Dispose();
		}
		
		/**Save list of urls from DataGrid to sitemap.
		 * Raised when btnsave clicked.
		 * @param sender Unused
		 * @param args Unused*/
		private void OnBtnSaveClick(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			string tmpmap = this.mapfile.Substring(0, (this.mapfile.Length - 3)) + "tmp";
			FileStream file = File.Create(tmpmap);
			StreamWriter mapwriter = new StreamWriter(file);
			for (int i = 0; i < this.datatab.Rows.Count; i++)
			{
				mapwriter.WriteLine(this.datatab.Rows[i].ItemArray[0].ToString());
			}
			mapwriter.Dispose();
			file.Dispose();
			File.Delete(this.mapfile);
			Sitemap sitemap = new Sitemap(this.mapfile);
			sitemap.CreateSitemap(false);
			sitemap = null;
			this.btnsave.Enabled = false;
			this.btnsend.Enabled = true;
		}
		
		/**Function add new empty link to sitemap.
		 * Raised when btnadd clicked.
		 * @param sender Unused
		 * @param args Unused*/
		private void OnBtnAddClick(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			this.tablemap.ReadOnly = true;
			this.datatab.Rows.Add("http://");
			this.tablemap.ReadOnly = false;
			if (!this.btnsave.Enabled)
			{
				this.btnsave.Enabled = true;
				this.btnsend.Enabled = false;
			}
		}
		
		/**Function delete selected link from sitemap.
		 * Raised when btndelete clicked.
		 * @param sender Unused
		 * @param args Unused*/
		private void OnBtnDeleteClick(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			this.tablemap.ReadOnly = true;
			this.datatab.Rows.RemoveAt(this.tablemap.CurrentRowIndex);
			this.tablemap.ReadOnly = false;
			if (!this.btnsave.Enabled)
			{
				this.btnsave.Enabled = true;
				this.btnsend.Enabled = false;
			}
		}
		
		/// <summary>
		/// Function update edited link. Raised when user end editing link.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="DataRowChangeEventArgs"/> changed row.
		/// </param>
		void OnCellChange(object sender, DataRowChangeEventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (!System.Text.RegularExpressions.Regex.IsMatch(args.Row.ItemArray[0].ToString(), 
			                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
			                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
			{
				WinMessageBox.Show(Catalog.GetString("Invalid url"));
				return;
			}
			if (!this.btnsave.Enabled)
			{
				this.btnsave.Enabled = true;
				this.btnsend.Enabled = false;
			}
		}
		
		/// <summary>
		/// Function enable btndelete if disabled. Raised when row selected in tablemap.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		void OnCursorChanged(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			if (!this.btndelete.Enabled)
			{
				this.btndelete.Enabled = true;
			}
		}
		
		/// <summary>
		/// Function resize tablemap and replace buttons. Raised when window is resized.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="args">
		/// A <see cref="EventArgs"/> unused.
		/// </param>
		void OnResize(object sender, EventArgs args)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (args != null)
			{
				args = null;
			}
			int x = this.Size.Width - 827;
			int y = this.Size.Height - 600;
			this.tablemap.Size = new Size(670 + x, 525 + y);
			this.btnadd.Location = new Point(688 + x, 200 + y);
			this.btndelete.Location = new Point(688 + x, 264 + y);
			this.btncancel.Location = new Point(488 + x, 536 + y);
			this.btnsave.Location = new Point(584 + x, 536 + y);
			this.btnsend.Location = new Point(680 + x, 536 + y);
		}
	}
}
