// WinSitemapDialog.cs
// 
// Copyright (C) 2008, 2009 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file WinSitemapDialog.cs
 * Provide WinForm Window for show informations about uploading sitemap.
 * @fn virtual void Grubng::WinSitemapDialog::InitializeComponent()
 * Create GUI*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Timers;
using System.Windows.Forms;
using Mono.Unix;

namespace Grubng
{
	/// <summary>
	/// Provide WinForm Window for show informations about uploading sitemap.
	/// </summary>
	internal sealed partial class WinSitemapDialog : Form
	{
		/// <summary>
		/// Upload thread.
		/// </summary>
		Thread uploadthread;
		/**Upload time timer*/
		/// <summary>
		/// Upload time timer.
		/// </summary>
		System.Timers.Timer uploadtime;
		/// <summary>
		/// Object from Sitemap class. Provide functions for manipulate sitemap file.
		/// </summary>
		Sitemap sitemap;
		
		/**Standard class constructor.
		 * Create dialog.
		 * @param file Full path to sitemap file*/
		public WinSitemapDialog(string file)
		{
			this.sitemap = new Sitemap(file);
			InitializeComponent();
			this.lblfilename.Text = file;
			this.lblstatus.Text = Catalog.GetString("Checking propriety of sitemap file");
			this.uploadthread = new Thread(new ThreadStart(this.UploadThread));
			this.uploadthread.Start();
		}
		
		/**Function close window after upload.
		 * Raised when btnok clicked.
		 * @param sender Unused
		 * @param e Unused*/
		private void OnOkClick (object sender, EventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			this.Close();
		}
		
		delegate void DelegateUpdateText(string text, bool button);
		delegate void DelegateUpdateProgress(string text, double uploadprg);
		
		/**Thread for upload.*/
		private void UploadThread()
		{
			this.uploadtime = new System.Timers.Timer();
			this.uploadtime.Elapsed += new ElapsedEventHandler(OnUploadElapsed);
			this.uploadtime.Interval = 500;
			this.uploadtime.Start();
			bool valid = this.sitemap.CheckSitemap();
			if (!valid)
			{
				this.Invoke(new DelegateUpdateText(this.UpdateText), Catalog.GetString("Invalid sitemap file"), true);
				this.uploadtime.Stop();
				this.uploadthread.Abort();
			}
			this.sitemap.Upload = true;
			this.Invoke(new DelegateUpdateText(this.UpdateText), Catalog.GetString("Uploading"), false);
			int[] status = sitemap.UploadSitemap(false);
			if (status[0] == 417)
			{
				status = sitemap.UploadSitemap(true);
			}
			this.uploadtime.Stop();
			sitemap = null;
			string message;
			switch (status[0])
			{
			case 0:
				message = Catalog.GetString("Upload fail. Reason: Cannot connect to server.");
				break;
			case 200:
				message = Catalog.GetString("Sitemap uploaded.");
				break;
			case 401:
				message = Catalog.GetString("Upload fail. Reason: Invalid login.");
				break;
			case 408:
				message = Catalog.GetString("Upload fail. Reason: Connection Timeout.");
				break;
			case 500:
				message = Catalog.GetString("Upload fail. Reason: Internal Server Error.");
				break;
			case 503:
				message = Catalog.GetString("Upload fail. Reason: Cannot connect to server.");
				break;
			default:
				message = status.ToString();
				break;
			}
			this.Invoke(new DelegateUpdateText(this.UpdateText), message, true);
			this.uploadthread.Abort();
		}
		
		/// <summary>
		/// Timer for update GUI while uploading. Raised every 0.5 second when program works.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/> unused.
		/// </param>
		/// <param name="e">
		/// A <see cref="ElapsedEventArgs"/> unused.
		/// </param>
		void OnUploadElapsed(object sender, ElapsedEventArgs e)
		{
			if (sender != null)
			{
				sender = null;
			}
			if (e != null)
			{
				e = null;
			}
			double progress = (double)this.sitemap.BytesSend / (double)this.sitemap.FileLength;
			if (progress < 0)
			{
				progress = 0;
			}
			if (progress > 1)
			{
				progress = 1;
			}
			if (progress == 1)
			{
				this.Invoke(new DelegateUpdateText(this.UpdateText), Catalog.GetString("Waiting for server answer."), false);
			}
			this.Invoke(new DelegateUpdateProgress(this.UpdateProgress), this.sitemap.BytesSend.ToString() + "/" + this.sitemap.FileLength.ToString(), progress);
		}
		
		/// <summary>
		/// Function update lblstatus from other thread and show btnok.
		/// </summary>
		/// <param name="text">
		/// A <see cref="System.String"/>  Text to show on lblstatus
		/// </param>
		/// <param name="button">
		/// A <see cref="System.Boolean"/> If true, show btnok and focus it
		/// </param>
		void UpdateText(string text, bool button)
		{
			this.lblstatus.Text = text;
			if (button)
			{
				this.btnok.Show();
				this.btnok.Focus();
			}
		}
			
		/// <summary>
		/// Function upldate prgupload from other thread.
		/// </summary>
		/// <param name="text">
		/// A <see cref="System.String"/> text for prgupload.
		/// </param>
		/// <param name="uploadprg">
		/// A <see cref="System.Double"/> value for prgupload.
		/// </param>
		void UpdateProgress(string text, double uploadprg)
		{
			this.prgupload.Value = Convert.ToInt32(uploadprg * 100);
			this.prgupload.Text = text;
			this.prgupload.Update();
		}
		
		/// <summary>
		/// Function override System.ComponentModel.Component.Dispose function.
		/// </summary>
		public new void Dispose()
		{
			this.lblfilename.Dispose();
			this.lblstatus.Dispose();
			this.prgupload.Dispose();
			this.btnok.Dispose();
			this.uploadtime.Dispose();
			base.Dispose();
		}
	}
}
