// Compress.cs
// 
// Copyright (C) 2008,2009 Bartek Jasicki
//
// This file is part of Grubng.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Compress.cs
 * Provide functions for compress files.*/

using System;
using System.IO;
using System.IO.Compression;

namespace Grubng
{
	/// <summary>
	/// Provide function for compress files.
	/// </summary>
	internal static class Compress
	{
		/// <summary>
		/// Function to compress file.
		/// </summary>
		/// <param name="filename">
		/// A <see cref="System.String"/> Full path (include filename) to file which been compressed.
		/// </param>
		/// <param name="extension">
		/// A <see cref="System.String"/> Compression file extension (and selection of compression method).
		/// </param>
		public static void Do(string filename, string extension)
		{
			using (FileStream file = File.OpenRead(filename))
			{
				using (FileStream workunit = File.Create(filename + extension))
				{
					switch (extension)
					{
					case ".gz":
						byte[] buffer = new byte[32768];
						int bytesRead;
						using (GZipStream gzstream = new GZipStream(workunit, CompressionMode.Compress))
						{
							while ((bytesRead = file.Read(buffer, 0, buffer.Length)) != 0)
							{
								gzstream.Write(buffer, 0, bytesRead);
								gzstream.Flush();
							}
						}
						buffer = null;
						break;
					case ".lzma":
						int length = (int)file.Length;
						if (length > 4194304)
						{
							length = 4194304;
						}
						object[] properties = {	(int)(length),
							(int)(2),
							(int)(3),
							(int)(0),
							(int)(2),
							(int)(128),
							"bt4",
							false
						};
						using (Encoder encoder = new Encoder())
						{
							CoderPropID[] propIDs = {CoderPropID.DictionarySize,
								CoderPropID.PosStateBits,
								CoderPropID.LitContextBits,
								CoderPropID.LitPosBits,
								CoderPropID.Algorithm,
								CoderPropID.NumFastBytes,
								CoderPropID.MatchFinder,
								CoderPropID.EndMarker
							};
							encoder.SetCoderProperties(propIDs, properties);
							encoder.WriteCoderProperties(workunit);
							long fileSize = file.Length;
							for (int i = 0; i < 8; i++)
							{
								workunit.WriteByte((Byte)(fileSize >> (8 * i)));
							}
							encoder.Code(file, workunit);
							properties = null;
						}
						break;
					default:
						break;
					}
				}
			}
		}
	}
}
