// Config.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Config.cs
 * Provide functions for config files.*/

using System;
using System.Xml;

namespace Grubng
{
	
	/// <summary>
	/// Provide functions for config files.
	/// </summary>
	public sealed class Config
	{
		/// <summary>
		/// Path to grub.xml file
		/// </summary>
		string filepath;
		
		/// <summary>
		/// Standard class constructor. Set path to grub.xml file
		/// </summary>
		public Config()
		{
			this.filepath = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
				System.IO.Path.DirectorySeparatorChar + ".grub" + System.IO.Path.DirectorySeparatorChar 
					+ ".grub.xml";
		}
			
		/// <summary>
		/// Function create grub.xml file and write default configuration
		/// </summary>
		/// <param name="user">
		/// A <see cref="System.String"/> Username needed to connect with dispatch server
		/// </param>
		/// <param name="pass">
		/// A <see cref="System.String"/> Password needed to connect with dispatch server
		/// </param>
		public void CreateConfig(string user, string pass)
		{
			string path = Config.GetDir();
			using (XmlTextWriter xmlw = new XmlTextWriter(this.filepath, null))
			{
				xmlw.Formatting = Formatting.Indented;
				xmlw.WriteStartDocument(false);
				xmlw.WriteStartElement("configuration");
				xmlw.WriteStartElement("login", null);
				xmlw.WriteElementString("username", user);
				xmlw.WriteElementString("password", pass);
				xmlw.WriteEndElement();
				xmlw.WriteStartElement("work");
				xmlw.WriteElementString("crawlers", "1");
				xmlw.WriteElementString("workdir", path);
				xmlw.WriteElementString("automatic", "N");
				xmlw.WriteElementString("workserver", "http://grub.silc.org.ua/");
				xmlw.WriteElementString("maxmbdownload", "10");
				xmlw.WriteElementString("proxystatus", "none");
				xmlw.WriteElementString("proxyaddress", null);
				xmlw.WriteElementString("proxyport", null);
				xmlw.WriteElementString("proxylogin", null);
				xmlw.WriteElementString("proxypass", null);
				xmlw.WriteElementString("downloadrate", "10");
				xmlw.WriteElementString("uploadrate", "10");
				xmlw.WriteElementString("compression", ".gz");
				xmlw.WriteElementString("createsitemaps", "N");
				xmlw.WriteEndElement();
				xmlw.WriteStartElement("statistics");
				xmlw.WriteElementString("worksend", "0");
				xmlw.WriteElementString("invalidarc", "0");
				xmlw.WriteElementString("urlcrawled", "0");
				xmlw.WriteElementString("success", "0");
				xmlw.WriteElementString("failure", "0");
				xmlw.WriteElementString("errors", "0");
				xmlw.WriteElementString("mbreceived", "0");
				xmlw.WriteElementString("mbsend", "0");
				xmlw.WriteEndElement();
				xmlw.WriteStartElement("other");
				xmlw.WriteElementString("checkupdates", "N");
				xmlw.WriteElementString("updateserver", "http://grub.org/updater/");
				xmlw.WriteElementString("unstable", "Y");
				xmlw.WriteElementString("sitemapserver", "http://grub.silc.org.ua/");
				xmlw.WriteElementString("deletesitemap", "N");
				xmlw.WriteElementString("runhidden", "N");
				xmlw.WriteElementString("enableremote", "N");
				xmlw.WriteElementString("remoteport", "6688");
				xmlw.WriteEndElement();
				xmlw.WriteEndElement();
				xmlw.Flush();
			}			
		}
		
		/// <summary>
		/// Function read one data from .xml file.
		/// </summary>
		/// <param name="node">
		/// A <see cref="System.String"/> XML node which been read
		/// </param>
		/// <param name="filename">
		/// A <see cref="System.String"/> full path including filename to file which been read
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> value of node
		/// </returns>
		public static string ReadConfig(string node, string filename)
		{
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.Load(filename);
			}
			catch (XmlException)
			{
				return "Invalid XML";
			}
			string answer = null;
			try
			{
				XmlNode noder = doc.SelectSingleNode(node);
				answer = noder.InnerText;
				doc = null;
			}
			catch (System.NullReferenceException)
			{
				answer = String.Empty;
			}
			return answer;
		}
		
		/// <summary>
		/// Function read data from selected nodes from grub.xml file
		/// </summary>
		/// <param name="nodes">
		/// A <see cref="System.String"/> array with XML nodes which been read
		/// </param>
		/// <param name="filename">
		/// A <see cref="System.String"/> full path including filename to file which been read.
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> array with values of nodes
		/// </returns>
		public static string[] ReadFew(string[] nodes, string filename)
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(filename);
			string[] answer = new string[nodes.Length];
			XmlNode noder;
			for (int i = 0; i < nodes.Length; i++)
			{
				noder = doc.SelectSingleNode(nodes[i]);
				answer[i] = noder.InnerText;
			}
			doc = null;
			return answer;
		}
		
		/// <summary>
		/// Function write data to .xml file
		/// </summary>
		/// <param name="data">
		/// A <see cref="System.String"/> value which been write to file
		/// </param>
		/// <param name="node">
		/// A <see cref="System.String"/> XML node which been overwrite
		/// </param>
		/// <param name="filename">
		/// A <see cref="System.String"/> full path including filename to file which been modified
		/// </param>
		public static void WriteConfig(string data, string node, string filename)
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(filename);
			XmlNode nodew = doc.SelectSingleNode(node);
			nodew.InnerText = data;
			try
			{
				doc.Save(filename);
			}
			catch (System.IO.IOException)
			{
				System.Threading.Thread.Sleep(500);
				doc.Save(filename);
			}
			doc = null;
		}
		
		/// <summary>
		/// Function write few values simultaneously to .xml file
		/// </summary>
		/// <param name="data">
		/// A <see cref="System.String"/> array of values which been write to file
		/// </param>
		/// <param name="nodes">
		/// A <see cref="System.String"/> array of XML nodes which been overwrite
		/// </param>
		/// <param name="filename">
		/// A <see cref="System.String"/> full path including filename to file which been modified
		/// </param>
		public static void WriteFew(string[] data, string[] nodes, string filename)
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(filename);
			XmlNode nodew;
			for (int i = 0; i < nodes.Length; i ++)
			{
				nodew = doc.SelectSingleNode(nodes[i]);
				nodew.InnerText = data[i];
			}
			try
			{
				doc.Save(filename);
			}
			catch (System.IO.IOException)
			{
				System.Threading.Thread.Sleep(500);
				doc.Save(filename);
			}
			doc = null;
		}
		
		/// <summary>
		/// Function insert new data to grub.xml file.
		/// </summary>
		/// <param name="data">
		/// A <see cref="System.String"/> value which been inserted to file
		/// </param>
		/// <param name="name">
		/// A <see cref="System.String"/> name of XML Element which been inserted to file
		/// </param>
		/// <param name="node">
		/// A <see cref="System.String"/> XML Node to which been inserted new XML Element
		/// </param>
		public void InsertConfig(string data, string name, string node)
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(this.filepath);
			XmlNode nodew = doc.SelectSingleNode(node);
			XmlNode childNode = doc.CreateNode(XmlNodeType.Element, name, String.Empty);
			childNode.InnerText = data;
			childNode = nodew.AppendChild(childNode);
			doc.Save(this.filepath);
			doc = null;
		}
		
		/// <summary>
		/// Function create new config file for crawler.
		/// </summary>
		/// <param name="filename">
		/// A <see cref="System.String"/> full path with filename of new config file
		/// </param>
		public static void CreateSecondary(string filename)
		{
			using (XmlTextWriter xmlw = new XmlTextWriter(filename, null))
			{
				xmlw.Formatting = Formatting.Indented;
				xmlw.WriteStartDocument(false);
				xmlw.WriteStartElement("work");
				xmlw.WriteElementString("workunit", null);
				xmlw.WriteElementString("workiter", "0");
				xmlw.WriteEndElement();
				xmlw.Flush();
			}
		}
		
		/// <summary>
		/// Function check correctness of main config file. If node not exist - insert
		/// node to config file with default value
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> True if config file was updated, otherwise false.
		/// </returns>
		public bool CheckConfig()
		{
			bool result = false;
			XmlDocument doc = new XmlDocument();
			doc.Load(this.filepath);
			string[] nodes = new string[32] {"login/username", "login/password", 
				"work/crawlers", "work/workdir", "work/automatic", "work/workserver",
				"work/maxmbdownload", "work/proxystatus", "work/proxyaddress",
				"work/proxyport", "work/proxylogin", "work/proxypass",
				"work/downloadrate", "work/uploadrate", "work/compression", "work/createsitemaps",
				"statistics/worksend", "statistics/invalidarc", "statistics/urlcrawled", "statistics/success", 
				"statistics/failure", "statistics/errors", "statistics/mbreceived", "statistics/mbsend",
				"other/checkupdates", "other/updateserver", "other/unstable",
				"other/sitemapserver", "other/deletesitemap", "other/runhidden", "other/enableremote",
				"other/remoteport"};
			string[] values = new string[32] {String.Empty, String.Empty, "1", Config.GetDir(),
				"N", "http://grub.silc.org.ua/", "10", "none",
				String.Empty, String.Empty, String.Empty, String.Empty, "10", "10", ".gz" ,"N", "0",
				"0", "0", "0", "0", "0", "0", "0", "N", "http://grub.org/updater/",	"Y", "http://grub.silc.org.ua/", 
				"N", "N", "N", "6688"};
			XmlNode noder;
			foreach (string node in nodes)
			{
				noder = doc.SelectSingleNode("/configuration/" + node);
				if (noder == null)
				{
					int index = Array.IndexOf(nodes, node);
					string[] arrnodes = node.Split('/');
					this.InsertConfig(values[index], arrnodes[1], "/configuration/" + arrnodes[0]);
					result = true;
				}
			}
			doc = null;
			return result;
		}
		
		/// <summary>
		/// Function get all paths of XML nodes from grub.xml file
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/> with all XPath of nodes, separated by semicolon
		/// </returns>
		public string GetNodes()
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(this.filepath);
			XmlNode root = doc.LastChild;
			System.Text.StringBuilder result = new System.Text.StringBuilder();
			foreach (XmlNode groupnode in root.ChildNodes)
			{
				foreach (XmlNode node in groupnode.ChildNodes)
				{
					result.Append("/");
					result.Append(root.Name);
					result.Append("/");
					result.Append(groupnode.Name);
					result.Append("/");
					result.Append(node.Name);
					result.Append(";");
				}
			}
			return result.ToString();
		}
		
		/// <summary>
		/// Function return default working directory (and create it, if not exists).
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/> full path to work directory.
		/// </returns>
		static string GetDir()
		{
			string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + 
					System.IO.Path.DirectorySeparatorChar + ".grub" + System.IO.Path.DirectorySeparatorChar;
			if (!System.IO.Directory.Exists(path))
			{
				System.IO.Directory.CreateDirectory(path);
			}
			return path;
		}
		
		/// <value>
		/// Property get/set private field filepath
		/// </value>
		public string FilePath
		{
			get {return this.filepath;}
			set {this.filepath = value;}
		}
	}
}
