// Crawler.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Crawler.cs
 * Provide functions for download, crawl and upload workunits.*/

using System;
using System.Diagnostics;
using System.Net;
using System.IO;
using System.IO.Compression;
using System.Text.RegularExpressions;
using System.Threading;
using Grubng;

namespace Grubng
{
	
	/// <summary>
	/// Provide functions for download, crawl and upload workunits.
	/// </summary>
	public sealed class Crawler
	{
		/// <summary>
		/// Path to workunit file.
		/// </summary>
		string filepath;
		/// <summary>
		/// Address of server with workunits.
		/// </summary>
		string workserver;
		/// <summary>
		/// Max downloaded MB allowed by one request
		/// </summary>
		long maxmbamount;
		/// <summary>
		/// Proxy status (none, default, custom)
		/// </summary>
		string proxystat;
		/// <summary>
		/// Proxy address (set only when status set on custom)
		/// </summary>
		string proxyaddress;
		/// <summary>
		/// Proxy port (set only when status set on custom)
		/// </summary>
		string proxyport;
		/// <summary>
		/// Proxy login (set only when status set on custom)
		/// </summary>
		string proxylogin;
		/// <summary>
		/// Proxy password (set only when status set on custom)
		/// </summary>
		string proxypass;
		/// <summary>
		/// Max allowed upload speed
		/// </summary>
		int uploadspeed;
		/// <summary>
		/// Amount of crawlers which download data (need for calculate max speed)
		/// </summary>
		int downloading;
		/// <summary>
		/// Max allowed download speed
		/// </summary>
		int downloadspeed;
		/// <summary>
		/// Amount of uploaded bytes to server
		/// </summary>
		int bytessend;
		/// <summary>
		/// Amount of bytes to upload on server
		/// </summary>
		long filelength;
		/// <summary>
		/// Message send by dispatch or upload server on error
		/// </summary>
		string servermessage;
		/// <summary>
		/// Default User-Agent Header if not set
		/// </summary>
		string useragent;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public Crawler()
		{
			System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
			this.useragent = asm.GetName().Name + "/" + asm.GetName().Version.ToString();
			asm = null;
		}
		
		/// <summary>
		/// Function connect to server with workunits, download new workunit and write it to work directory.
		/// </summary>
		/// <param name="user">
		/// A <see cref="System.String"/> username needed for login to server
		/// </param>
		/// <param name="pass">
		/// A <see cref="System.String"/> password needed for login to server
		/// </param>
		/// <param name="number">
		/// A <see cref="System.Int32"/> number of crawler (start from 0)
		/// </param>
		/// <returns>
		/// A <see cref="System.Int32"/> array, element 0 is HTTP status code received from server, element 1 is amount
		/// of bytes sent by server, element 2 - amount of bytes sent by client.
		/// </returns>
		public int[] Download(ref string user, ref string pass, ref int number)
		{
			int[] returnvalue = new int[3];
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(this.workserver);
			if (this.proxystat == "custom")
			{
				Uri proxyuri = new Uri(this.proxyaddress + ":" + this.proxyport);
				request.Proxy = new WebProxy(proxyuri);
				if (!String.IsNullOrEmpty(this.proxylogin))
				{
					request.Proxy.Credentials = new NetworkCredential(this.proxylogin, this.proxypass);
				}
			}
			if (this.proxystat == "none")
			{
				request.Proxy = null;
			}
			else
			{
				request.Headers.Add(HttpRequestHeader.CacheControl, "no-cache");
				request.Headers.Add(HttpRequestHeader.Pragma, "no-cache");
			}
			request.KeepAlive = false;
			request.UserAgent = this.useragent;
			request.Credentials = new NetworkCredential(user, pass);
			request.PreAuthenticate = true;
			returnvalue[2] = request.Headers.ToString().Length;
			HttpWebResponse response;
			returnvalue[1] = 0;
			byte[] buffer = new byte[1024];
			int bytesRead = 0;
			int tmpbytes = 0;
			int pause = 0;
			Stopwatch speedwatch = new Stopwatch();
			try
			{
				FileStream file;
				response = (HttpWebResponse)request.GetResponse();
				try
				{
					file = File.Create(this.filepath + "workunit" + number.ToString());
				}
				catch (IOException)
				{
					System.Threading.Thread.Sleep(1000);
					file = File.Create(this.filepath + "workunit" + number.ToString());
				}
				using (Stream streamr = response.GetResponseStream())
				{
					speedwatch.Start();
					while ((bytesRead = streamr.Read(buffer, 0, buffer.Length)) != 0)
					{
						file.Write(buffer, 0, bytesRead);
						file.Flush();
						tmpbytes += bytesRead;
						speedwatch.Stop();
						if (this.downloading < 1)
						{
							this.downloading = 1;
						}
						if (tmpbytes >= (this.downloadspeed / this.downloading))
						{
							tmpbytes = 0;
							if (speedwatch.ElapsedMilliseconds < 1000)
							{
								pause = (int)speedwatch.ElapsedMilliseconds;
								System.Threading.Thread.Sleep(1000 - pause);
							}
							speedwatch.Reset();
						}
						speedwatch.Start();
					}
					streamr.Close();
				}
				file.Close();
				file.Dispose();
				file = null;
				buffer = null;
				speedwatch = null;
				returnvalue[0] = (int)response.StatusCode;
				returnvalue[1] = (int)new FileInfo(this.filepath + "workunit" + number.ToString()).Length;
				response.Close();
				response = null;
			}
			catch (WebException e)
			{
				if (e.Status.ToString() == "ProtocolError")
				{
					response = (HttpWebResponse)e.Response;
					returnvalue[0] = this.CatchErrorResponse(ref response);
				}
				else
				{
					int status = (int)e.Status;
					switch (status)
					{
					case 1:
						returnvalue[0] = 501;
						break;
					case 2:
						returnvalue[0] = 501;
						break;
					case 14:
						returnvalue[0] = 408;
						break;
					default:
						Logs.WriteLog(this.filepath + "error.log", "Unknown download error: " + e.Status.ToString());
						returnvalue[0] = 450;
						break;
					}
				}
			}
			return returnvalue;
		}
		
		/// <summary>
		/// Function to crawl URL's from workunit. Crawl one url from workunit, get content of url, write all
		/// information to file (in .arc file standard) and retrieve urls from pages.
		/// </summary>
		/// <param name="url">
		/// A <see cref="System.String"/> Full URL to crawl (with http://).
		/// </param>
		/// <param name="host">
		/// A <see cref="System.String"/> Host name for URL.
		/// </param>
		/// <param name="workunit">
		/// A <see cref="FileStream"/> associated with .arc file.
		/// </param>
		/// <param name="protocolver">
		/// A <see cref="System.String"/> HTTP protocol version of crawler request.
		/// </param>
		/// <param name="requestHeaders">
		/// A <see cref="System.Collections.Generic.List"/> List of headers to add to client.
		/// </param>
		/// <param name="tmpsitemap">
		/// A <see cref="FileStream"/> associated with temporary sitemap.
		/// </param>
		/// <param name="createsitemaps">
		/// A <see cref="System.Boolean"/> enable/disable creating sitemaps.
		/// </param>
		/// <returns>
		/// A <see cref="System.Int32"/> array, element 0 - is HTTP Status Code reveived from
		/// remote server, element 1 is amount of bytes send by server, element 2 - 1 mean
		/// timeout request, element 3 - amount of bytes send by client.
		/// </returns>
		public int[] Crawling2(string url, string host, ref FileStream workunit, string protocolver, 
		                       System.Collections.Generic.List<string> requestHeaders, ref FileStream tmpsitemap,
		                       bool createsitemaps)
		{
			HttpWebRequest request;
			HttpWebResponse response;
			string page = null;
			string ip;
			string contype = String.Empty;
			int[] returnvalue = new int[4];
			returnvalue[1] = 0;
			byte[] filebuffer;
			//test for valid host
			int uname = host.IndexOf(':');
			if (uname > -1)
			{
				string[] searchhost = host.Split(':');
				host = searchhost[0];
			}
			string date = DateTime.UtcNow.ToString("yyyyMMddHHmmss");
			//send request to server
			try
			{
				request = (HttpWebRequest)WebRequest.Create(url);
				request.Timeout = 20000;
				if (protocolver == "HTTP/1.0")
				{
					request.ProtocolVersion = HttpVersion.Version10;
				}
				if (protocolver == "HTTP/1.1")
				{
					request.ProtocolVersion = HttpVersion.Version11;
				}
				request.Headers.Add(HttpRequestHeader.AcceptEncoding, "gzip,deflate");
				//Add other headers
				foreach (string requestHeader in requestHeaders)
				{
					if (requestHeader.StartsWith("User-Agent:"))
					{
						request.UserAgent = requestHeader.Replace("User-Agent: ", String.Empty);
						continue;
					}
					if (requestHeader.StartsWith("Accept:"))
					{
						request.Accept = requestHeader.Replace("Accept: ", String.Empty);
						continue;
					}
					if (requestHeader.StartsWith("If-Modified-Since:"))
					{
						request.IfModifiedSince = Convert.ToDateTime(requestHeader.Replace("If-Modified-Since: ", 
						                                                                   String.Empty));
						continue;
					}
					request.Headers.Add(requestHeader);
				}
				if (String.IsNullOrEmpty(request.Accept))
				{
					request.Accept = "*/*";
				}
				if (this.proxystat == "custom")
				{
					Uri proxyuri = new Uri(this.proxyaddress + ":" + this.proxyport);
					request.Proxy = new WebProxy(proxyuri);
					if (!String.IsNullOrEmpty(this.proxylogin))
					{
						request.Proxy.Credentials = new NetworkCredential(this.proxylogin, this.proxypass);
					}
				}
				if (this.proxystat == "none")
				{
					request.Proxy = null;
				}
				else
				{
					if (protocolver == "HTTP/1.1")
					{
						request.Headers.Add(HttpRequestHeader.CacheControl, "no-cache");
					}
					request.Headers.Add(HttpRequestHeader.Pragma, "no-cache");
				}
				request.AllowAutoRedirect = false;
				returnvalue[3] = request.Headers.ToString().Length;
			}
			catch (UriFormatException)
			{
				filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " + date + 
				                                                " application/x-grub-error 39\n" +
				                                                "HTTP/1.0 500 Invalid URL\r\n\r\nInvalid URL");
				workunit.Write(filebuffer, 0, filebuffer.Length);
				workunit.Flush();
				url = url.Substring(7);
				Logs.WriteLog(this.filepath + "invalidurl.log", "URL: " + url + " Host: " + host);
				filebuffer = null;
				returnvalue[0] = 500;
				return returnvalue;
			}
			string statusdesc = null;
			string headers = String.Empty;
			string protver = null;
			Stream crawler2;
			MemoryStream memstream = new MemoryStream();
			// Get response
			try
			{
				response = (HttpWebResponse)request.GetResponse();
				if (!request.HaveResponse)
				{
					filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
					                                                date + " application/x-grub-error 52\n" +
					                                                "HTTP/1.0 503 No Response\r\n\r\nServer not sent any data");
					workunit.Write(filebuffer, 0, filebuffer.Length);
					workunit.Flush();
					filebuffer = null;
					memstream.Dispose();
					memstream = null;
					returnvalue[0] = 503;
					return returnvalue;
				}
				returnvalue[0] = (int)response.StatusCode;
				statusdesc = response.StatusDescription;
				headers = response.Headers.ToString();
				if ((returnvalue[0] != 304) && (response.ContentLength != 0))
				{
					contype = Crawler.MimeType(response.ContentType);
					if (!request.Accept.ToLower().Contains("*/*"))
					{
						if (Array.IndexOf(request.Accept.ToLower().Split(','), contype) < 0)
						{
							filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
							                                                date + " application/x-grub-error 59\n" +
							                                                "HTTP/1.0 406 Not Acceptable\r\n\r\nInvalid data sent by server.");
							workunit.Write(filebuffer, 0, filebuffer.Length);
							workunit.Flush();
							filebuffer = null;
							memstream.Dispose();
							memstream = null;
							returnvalue[0] = 406;
							response.Close();
							return returnvalue;
						}
					}
					crawler2 = response.GetResponseStream();
					if (response.ContentEncoding.ToLower().Contains("gzip"))
					{
						crawler2 = new GZipStream(crawler2, CompressionMode.Decompress);
					}
					if (response.ContentEncoding.ToLower().Contains("deflate"))
					{
						crawler2 = new DeflateStream(crawler2, CompressionMode.Decompress);
					}
					try
					{
						byte[] buffer = new byte[1024];
						int bytesRead = 0;
						int tmpbytes = 0;
						int pause = 0;
						Stopwatch speedwatch = new Stopwatch();
						speedwatch.Start();
						while ((bytesRead = crawler2.Read(buffer, 0, buffer.Length)) != 0)
						{
							memstream.Write(buffer, 0, bytesRead);
							memstream.Flush();
							if (memstream.Length > this.maxmbamount)
							{
								break;
							}
							tmpbytes += bytesRead;
							speedwatch.Stop();
							if (this.downloading < 1)
							{
								this.downloading = 1;
							}
							if (tmpbytes >= (this.downloadspeed / this.downloading))
							{
								tmpbytes = 0;
								if (speedwatch.ElapsedMilliseconds < 1000)
								{
									pause = (int)speedwatch.ElapsedMilliseconds;
									System.Threading.Thread.Sleep(1000 - pause);
								}
								speedwatch.Reset();
							}
							speedwatch.Start();
						}
						speedwatch = null;
					}
					catch (WebException)
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 51\n" +
						                                                "HTTP/1.0 408 Request Timeout\r\n\r\nOperation timed out");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						crawler2.Dispose();
						crawler2 = null;
						memstream.Dispose();
						memstream = null;
						returnvalue[0] = 408;
						return returnvalue;
					}
					catch (InvalidDataException)
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 46\n" +
						                                                "HTTP/1.0 500 Client error\r\n\r\nInvalid data sent");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						crawler2.Dispose();
						crawler2 = null;
						memstream.Dispose();
						memstream = null;
						response.Close();
						response = null;
						returnvalue[0] = 500;
						return returnvalue;
					}
					catch (IOException)
					{
					}
					catch (ObjectDisposedException)
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 46\n" +
						                                                "HTTP/1.0 500 Client error\r\n\r\nInvalid data sent");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						crawler2.Dispose();
						crawler2 = null;
						memstream.Dispose();
						memstream = null;
						response.Close();
						response = null;
						returnvalue[0] = 500;
						return returnvalue;
					}
					//Check for empty content
					if ((memstream.Length == 0) && (response.StatusCode == HttpStatusCode.OK))
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 57\n" +
						                                                "HTTP/1.0 204 No Content\r\n\r\nEmpty page send (only headers)");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						crawler2.Dispose();
						crawler2 = null;
						response.Close();
						response = null;
						returnvalue[0] = 204;
						memstream.Dispose();
						memstream = null;
						return returnvalue;
					}
					int index;
					if (memstream.Length < 2000)
					{
						index = (int)memstream.Length;
					}
					else
					{
						index = 2000;
					}
					string urlpage = System.Text.Encoding.ASCII.GetString(memstream.GetBuffer(), 0, index);
					index = urlpage.IndexOf("<meta name=\"robots\" content=\"", 0, StringComparison.OrdinalIgnoreCase);
					int lastindex = -1;
					if (index > -1)
					{
						lastindex = urlpage.IndexOf("</head>", 0, StringComparison.OrdinalIgnoreCase);
						if (index < lastindex)
						{
							lastindex = urlpage.IndexOf('"', index + 29);
							string meta = urlpage.Substring(index + 29, (lastindex - index - 29));
							if (meta.IndexOf("NOINDEX", StringComparison.OrdinalIgnoreCase) != -1)
							{
								filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
								                                                date + " application/x-grub-error 62\n" +
								                                                "HTTP/1.0 403 Client error\r\n\r\nPage cannot be indexed (META tag)");
								workunit.Write(filebuffer, 0, filebuffer.Length);
								workunit.Flush();
								filebuffer = null;
								crawler2.Dispose();
								crawler2 = null;
								response.Close();
								response = null;
								returnvalue[0] = 403;
								memstream.Dispose();
								memstream = null;
								urlpage = null;
								return returnvalue;
							}
							lastindex = meta.IndexOf("NOFOLLOW", StringComparison.OrdinalIgnoreCase);
						}
					}
					//Create or update temporary sitemap file
					if ((createsitemaps) && (returnvalue[0] == 200) && (lastindex == -1) && ((contype == "text/html") || 
					                                                                         (contype == "application/xhtml+xml")))
					{
						urlpage = System.Text.Encoding.ASCII.GetString(memstream.GetBuffer());
						Crawler.CreateTmpSitemap(ref urlpage, ref tmpsitemap, ref response, host, url);
					}
					urlpage = null;
					crawler2.Dispose();
					crawler2 = null;
				}
				if (contype.Length == 0)
				{
					if (returnvalue[0] != 200)
					{
						contype = Crawler.MimeType(response.ContentType);
					}
					else
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 57\n" +
						                                                "HTTP/1.0 204 No Content\r\n\r\nEmpty page send (only headers)");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						response.Close();
						response = null;
						returnvalue[0] = 204;
						memstream.Dispose();
						memstream = null;
						return returnvalue;
					}
				}
				protver = "HTTP/" + response.ProtocolVersion.ToString();
				response.Close();
				response = null;
			}
			catch (UriFormatException)
			{
				filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " + date + 
				                                                " application/x-grub-error 39\n" +
				                                                "HTTP/1.0 500 Invalid URL\r\n\r\nInvalid URL");
				workunit.Write(filebuffer, 0, filebuffer.Length);
				workunit.Flush();
				url = url.Substring(7);
				Logs.WriteLog(this.filepath + "invalidurl.log", "URL: " + url + " Host: " + host);
				filebuffer = null;
				returnvalue[0] = 500;
				return returnvalue;
			}
			catch (WebException e)
			{
				if (e.Status.ToString() == "ProtocolError")
				{
					response = (HttpWebResponse)e.Response;
					if (!request.HaveResponse)
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 52\n" +
						                                                "HTTP/1.0 503 No Response\r\n\r\nServer not sent any data");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						memstream.Dispose();
						memstream = null;
						returnvalue[0] = 503;
						return returnvalue;
					}
					try
					{
						returnvalue[0] = (int)response.StatusCode;
					}
					catch (Exception ex)
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 39\n" +
						                                                "HTTP/1.0 500 Invalid URL\r\n\r\nInvalid URL");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						memstream = null;
						returnvalue[0] = 500;
						string message = " Caught: " + ex.GetType().ToString() + " " + ex.Message +
							"\n  Source: " + ex.Source +
								"\n  StackTrace: " + ex.StackTrace +
								"\n  TargetSite: " + ex.TargetSite;
						Logs.WriteLog(this.filepath + "error.log", message);
						return returnvalue;
					}
					statusdesc = response.StatusDescription;
					headers = response.Headers.ToString();
					if (response.ContentLength != 0)
					{
						crawler2 = response.GetResponseStream();
						if (response.ContentEncoding.ToLower().Contains("gzip"))
						{
							crawler2 = new GZipStream(crawler2, CompressionMode.Decompress);
						}
						if (response.ContentEncoding.ToLower().Contains("deflate"))
						{
							crawler2 = new DeflateStream(crawler2, CompressionMode.Decompress);
						}
						byte[] buffer = new byte[1024];
						int bytesRead = 0;
						try
						{
							int tmpbytes = 0;
							int pause = 0;
							Stopwatch speedwatch = new Stopwatch();
							speedwatch.Start();
							while ((bytesRead = crawler2.Read(buffer, 0, buffer.Length)) != 0)
							{
								memstream.Write(buffer, 0, bytesRead);
								memstream.Flush();
								tmpbytes += bytesRead;
								speedwatch.Stop();
								if (this.downloading < 1)
								{
									this.downloading = 1;
								}
								if (tmpbytes >= (this.downloadspeed / this.downloading))
								{
									tmpbytes = 0;
									if (speedwatch.ElapsedMilliseconds < 1000)
									{
										pause = (int)speedwatch.ElapsedMilliseconds;
										System.Threading.Thread.Sleep(1000 - pause);
									}
									speedwatch.Reset();
								}
								speedwatch.Start();
							}
							speedwatch = null;
						}
						catch (InvalidDataException)
						{
							filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
							                                                date + " application/x-grub-error 46\n" +
							                                                "HTTP/1.0 500 Client error\r\n\r\nInvalid data sent");
							workunit.Write(filebuffer, 0, filebuffer.Length);
							workunit.Flush();
							filebuffer = null;
							crawler2.Dispose();
							crawler2 = null;
							memstream.Dispose();
							memstream = null;
							response.Close();
							response = null;
							returnvalue[0] = 500;
							return returnvalue;
						}
						crawler2.Dispose();
						crawler2 = null;
					}
					contype = Crawler.MimeType(response.ContentType);
					protver = "HTTP/" + response.ProtocolVersion.ToString();
					response.Close();
					response = null;
				}
				else
				{
					returnvalue[0] = 500;
					statusdesc = e.Status.ToString();
					contype = "application/x-grub-error";
					protver = protocolver;
					int error = (int)e.Status;
					switch (error)
					{
					case 1:
						page = "The name resolver service could not resolve the host name.";
						returnvalue[2] = 1;
						break;
					case 2:
						page = "The remote service point could not be contacted at the transport level. ";
						returnvalue[2] = 1;
						break;
					case 3:
						page = "A complete response was not received from the remote server.";
						break;
					case 4:
						page = "A complete request could not be sent to the remote server.";
						break;
					case 9:
						page = "A server certificate could not be validated.";
						break;
					case 11:
						page = "The server response was not a valid HTTP response.";
						break;
					case 14:
						page = "No response was received during the timeout period for a request.";
						returnvalue[0] = 408;
						returnvalue[2] = 1;
						break;
					case 16:
						page = "Unknow error";
						break;
					default:
						Logs.WriteLog(this.filepath + "error.log", "Unknown connection error: " + e.Status.ToString());
						break;
					}
				}
			}
			Regex rx = new Regex(@"^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+");
			if (rx.IsMatch(host))
			{
				ip = host;
			}
			else
			{
				if ((returnvalue[0] != 500) && (returnvalue[0] != 408))
				{
					try
					{
						ip = Dns.GetHostEntry(host).AddressList[0].ToString();
					}
					catch (System.Net.Sockets.SocketException)
					{
						filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " 0.0.0.0 " +
						                                                date + " application/x-grub-error 50\n" +
						                                                "HTTP/1.0 404 Unknown Host\r\n\r\nNo such host is known");
						workunit.Write(filebuffer, 0, filebuffer.Length);
						workunit.Flush();
						filebuffer = null;
						memstream.Close();
						memstream = null;
						returnvalue[0] = 404;
						return returnvalue;
					}
				}
				else
				{
					ip = "0.0.0.0";
				}
			}
			rx = null;
			string header = protver + " " + returnvalue[0] + " " + statusdesc + "\r\n" + headers + "\r\n";
			long length = System.Text.UTF8Encoding.UTF8.GetByteCount(header);
			long len = 0;
			long lenpage = 0;
			if (!String.IsNullOrEmpty(page))
			{
				lenpage = System.Text.UTF8Encoding.UTF8.GetByteCount(page);
				len = length + lenpage;
			}
			else
			{
				len = length + memstream.Length;
			}
			filebuffer = System.Text.Encoding.UTF8.GetBytes("\n" + url + " " + ip + " " + date + " " +
			                                                contype + " " + len + "\n");
			workunit.Write(filebuffer, 0, filebuffer.Length);
			workunit.Flush();
			filebuffer = System.Text.Encoding.UTF8.GetBytes(header);
			workunit.Write(filebuffer, 0, filebuffer.Length);
			workunit.Flush();
			if (lenpage > 0)
			{
				filebuffer = System.Text.Encoding.UTF8.GetBytes(page);
				workunit.Write(filebuffer, 0, filebuffer.Length);
				workunit.Flush();
			}
			else
			{
				memstream.WriteTo(workunit);
				memstream.Dispose();
				memstream = null;
			}
			if ((returnvalue[0] == 500) || (returnvalue[0] == 408))
			{
				len = length;
			}
			returnvalue[1] = Convert.ToInt32(len);
			return returnvalue;
		}
		
		/// <summary>
		/// Function sending compressed .arc file to server. Additional, show progress of uploading on progress bar.
		/// </summary>
		/// <param name="workurl">
		/// A <see cref="System.String"/> full address to upload server.
		/// </param>
		/// <param name="filenamegz">
		/// A <see cref="System.String"/> full path (with filename) to compressed .arc file
		/// </param>
		/// <returns>
		/// A <see cref="System.Int32"/> array, element 0 - is HTTP Status Code reveived from server, element 1 is 
		///  amount of bytes send by client, elemnt 2 is amount of bytes received by client
		/// </returns>
		public int[] Upload(string workurl, string filenamegz)
		{
			workurl = workurl.Replace(' ', '+');
			HttpWebResponse response;
			FileStream file;
			try
			{
				file = new FileStream(filenamegz, FileMode.Open, FileAccess.Read);
			}
			catch(IOException)
			{
				Thread.Sleep(1000);
				file = new FileStream(filenamegz, FileMode.Open, FileAccess.Read);
			}
			Stream streamw1;
			this.filelength = file.Length;
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(workurl);
			if (this.proxystat == "custom")
			{
				Uri proxyuri = new Uri(this.proxyaddress + ":" + this.proxyport);
				request.Proxy = new WebProxy(proxyuri);
				if (!String.IsNullOrEmpty(this.proxylogin))
				{
					request.Proxy.Credentials = new NetworkCredential(this.proxylogin, this.proxypass);
				}
			}
			if (this.proxystat == "none")
			{
				request.Proxy = null;
			}
			request.AllowWriteStreamBuffering = false;
			request.Method = "PUT";
			request.ContentLength = this.filelength;
			request.Timeout = 600000;
			request.UserAgent = this.useragent;
			int[] returnvalue = new int[3];
			try
			{
				streamw1 = request.GetRequestStream();
			}
			catch (WebException web)
			{
				if (web.Status == WebExceptionStatus.ConnectFailure)
				{
					returnvalue[0] = 503;
					returnvalue[1] = 0;
					returnvalue[2] = 0;
				}
				file.Close();
				file.Dispose();
				file = null;
				return returnvalue;
			}
			Stopwatch speedwatch = new Stopwatch();
			byte[] buffer = new byte[1024];
            int bytesRead = 0;
			this.bytessend = 0;
			int tmpbytes = 0;
			int pause = 0;
			try
			{
				if (request.HaveResponse)
				{
					response = (HttpWebResponse)request.GetResponse();
				}
			}
			catch (WebException ex)
			{
				file.Close();
				file.Dispose();
				file = null;
				streamw1.Close();
				streamw1.Dispose();
				streamw1 = null;
				response = (HttpWebResponse)ex.Response;
				returnvalue[0] = (int)response.StatusCode;
				returnvalue[1] = (int)request.Headers.ToByteArray().Length;
				returnvalue[2] = (int)response.ContentLength + response.Headers.ToString().Length;
				this.servermessage = String.Empty;
				System.Text.StringBuilder servresponse = new System.Text.StringBuilder();
				Stream streamr = response.GetResponseStream();
				speedwatch = new Stopwatch();
				speedwatch.Start();
				while ((bytesRead = streamr.Read(buffer, 0, buffer.Length)) != 0)
				{
					servresponse.Append(System.Text.Encoding.UTF8.GetString(buffer));
					tmpbytes += bytesRead;
					speedwatch.Stop();
					if (this.downloading < 1)
					{
						this.downloading = 1;
					}
					if (tmpbytes >= (this.downloadspeed / this.downloading))
					{
						tmpbytes = 0;
						if (speedwatch.ElapsedMilliseconds < 1000)
						{
							pause = (int)speedwatch.ElapsedMilliseconds;
							System.Threading.Thread.Sleep(1000 - pause);
						}
						speedwatch.Reset();
					}
					speedwatch.Start();
				}
				speedwatch = null;
				streamr.Close();
				streamr.Dispose();
				this.servermessage = servresponse.ToString();
				servresponse = null;
				response.Close();
				return returnvalue;
			}
			try
			{
				speedwatch.Start();
				while ((bytesRead = file.Read(buffer, 0, buffer.Length)) != 0)
				{
					streamw1.Write(buffer, 0, bytesRead);
					this.bytessend += bytesRead;
					tmpbytes += bytesRead;
					speedwatch.Stop();
					if (tmpbytes >= this.uploadspeed)
					{
						tmpbytes = 0;
						if (speedwatch.ElapsedMilliseconds < 1000)
						{
							pause = (int)speedwatch.ElapsedMilliseconds;
							System.Threading.Thread.Sleep(1000 - pause);
						}
						speedwatch.Reset();
					}
					speedwatch.Start();
				}
				speedwatch = null;
			}
			catch
			{
				returnvalue[0] = 408;
				returnvalue[1] = this.bytessend;
				streamw1.Close();
				streamw1.Dispose();
				streamw1 = null;
				buffer = null;
				file.Close();
				file.Dispose();
				file = null;
				returnvalue[2] = 0;
				return returnvalue;
			}
			streamw1.Close();
			streamw1.Dispose();
			streamw1 = null;
			file.Close();
			file.Dispose();
			file = null;
			try
			{
				response = (HttpWebResponse)request.GetResponse();
				returnvalue[0] = (int)response.StatusCode;
				returnvalue[2] = (int)response.ContentLength + response.Headers.ToString().Length;
				response.Close();
				response = null;
			}
			catch (WebException e)
			{
				try
				{
					response = (HttpWebResponse)e.Response;
					returnvalue[0] = this.CatchErrorResponse(ref response);
				}
				catch (Exception error)
				{
					string message = " Caught: " + error.GetType().ToString() + " " + error.Message +
							"\n  Source: " + error.Source +
								"\n  StackTrace: " + error.StackTrace +
								"\n  TargetSite: " + error.TargetSite;
					Logs.WriteLog(this.filepath + "error.log", message);
					returnvalue[0] = 0;
					returnvalue[2] = 0;
				}
			}
			buffer = null;
			returnvalue[1] = (int)this.filelength;
			return returnvalue;
		}
		
		/// <summary>
		/// Function read configuration from grub.xml file. Read informations about work directory path, 
		/// addres of server with workunits, maximu allowed MB per one url and proxy settings for
		/// downloading workunits and uploading compressed .arc files
		/// </summary>
		public void SetPath()
		{
			Config conf = new Config();
			string[] nodes = new string[] {
				"/configuration/work/workdir",
				"/configuration/work/workserver",
				"/configuration/work/maxmbdownload",
				"configuration/work/uploadrate",
				"configuration/work/downloadrate",
				"/configuration/work/proxystatus"};
			string[] values = Config.ReadFew(nodes, conf.FilePath);
			this.filepath = values[0];
			this.workserver = values[1];
			int upspeed = Convert.ToInt32(values[3]);
			if (upspeed > 512)
			{
				upspeed = 512;
			}
			if (upspeed < 4)
			{
				upspeed = 4;
			}
			int downspeed = Convert.ToInt32(values[4]);
			if (downspeed > 512)
			{
				downspeed = 512;
			}
			if (downspeed < 4)
			{
				downspeed = 4;
			}
			this.maxmbamount = Convert.ToInt64(values[2]) * 1048576;
			this.uploadspeed = upspeed * 1024;
			this.downloadspeed = downspeed * 1024;
			this.proxystat = values[5];
			if (this.proxystat == "custom")
			{
				nodes = new string[] {
					"/configuration/work/proxyaddress",
					"/configuration/work/proxyport",
					"/configuration/work/proxylogin",
					"/configuration/work/proxypass"};
				values = Config.ReadFew(nodes, conf.FilePath);
				this.proxyaddress = values[0];
				this.proxyport = values[1];
				this.proxylogin = values[2];
				this.proxypass = values[3];
			}
			nodes = null;
			values = null;
			conf = null;
		}
		
		/// <summary>
		/// Function check mime type of response.
		/// </summary>
		/// <param name="type">
		/// A <see cref="System.String"/> Response mime type send by server.
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> valid response type.
		/// </returns>
		static string MimeType(string type)
		{
			string[] contenttype = new string[3];
			if (String.IsNullOrEmpty(type))
			{
				return "text/html";
			}
			else
			{
				contenttype = type.Split(';', ' ');
			}
			if (String.IsNullOrEmpty(contenttype[0]))
			{
				int testindex = -1;
				for (int i = 0; i < contenttype.Length; i++)
				{
					testindex = contenttype[i].IndexOf("charset");
					if (testindex > -1)
					{
						return "text/html";
					}
				}
			}
			return contenttype[0];
		}
		
		/// <summary>
		/// Function extract URL's from downloaded page and write they to temporary sitemap file.
		/// </summary>
		/// <param name="urlpage">
		/// A <see cref="System.String"/> with content of page
		/// </param>
		/// <param name="tmpsitemap">
		/// A <see cref="FileStream"/> to temporary sitemap file
		/// </param>
		/// <param name="response">
		/// A <see cref="HttpWebResponse"/> response sent by server
		/// </param>
		/// <param name="host">
		/// A <see cref="System.String"/> server host
		/// </param>
		/// <param name="url">
		/// A <see cref="System.String"/> server full URL
		/// </param>
		static void CreateTmpSitemap(ref string urlpage, ref FileStream tmpsitemap, ref HttpWebResponse response, string host,
		                      string url)
		{
			byte[] filebuffer;
			try
			{
				//Retrieve url's from crawled page
				int index = urlpage.IndexOf("<body", 0, StringComparison.OrdinalIgnoreCase);
				int markindex = 0;
				int markindex2 = 0;
				int lastindex = 0;
				string mark;
				string mark2;
				char[] lastchars = new char[2] {'>', ' '};
				while (index > -1)
				{
					index = urlpage.IndexOf("<a", index, StringComparison.OrdinalIgnoreCase);
					index = urlpage.IndexOf(" href=", index, StringComparison.OrdinalIgnoreCase);
					if (index > -1)
					{
						index += 6;
						mark = urlpage.Substring(index, 1);
						if (mark == "#")
						{
							continue;
						}
						if ((mark == "\"") || (mark == "'"))
						{
							index ++;
							if (mark == "\"")
							{
								lastindex = urlpage.IndexOf('"', index);
							}
							else
							{
								lastindex = urlpage.IndexOf('\'', index);
							}
							mark = urlpage.Substring(index, 1);
							if (mark == "#")
							{
								continue;
							}
							mark = urlpage.Substring(index, 4);
							if (mark == "http")
							{
								mark = urlpage.Substring(index, (lastindex - index));
								mark = mark.Replace(" ", "%20");
								filebuffer = System.Text.Encoding.UTF8.GetBytes(mark + Environment.NewLine);
								tmpsitemap.Write(filebuffer, 0, filebuffer.Length);
								tmpsitemap.Flush();
								index = lastindex;
								continue;
							}
							else
							{
								mark = urlpage.Substring(index, (lastindex - index));
								markindex = mark.IndexOf(':');
								markindex2 = mark.IndexOf('@');
								if ((markindex == -1) && (markindex2 == -1) && (mark.Length > 0))
								{
									markindex = mark.IndexOf('/');
									if (markindex == 0)
									{
										mark = "http://" + host + mark;
									}
									else
									{
										markindex = response.ResponseUri.AbsolutePath.LastIndexOf('/');
										if ((response.ResponseUri.AbsolutePath.Length - 1) == markindex)
										{
											mark = "http://" + host + response.ResponseUri.AbsolutePath + mark;
										}
										else
										{
											markindex = response.ResponseUri.AbsolutePath.LastIndexOf('/');
											mark2 = response.ResponseUri.AbsolutePath.Substring(0, (markindex + 1));
											mark = "http://" + host + mark2 + mark;
										}
									}
									mark = mark.Replace(" ", "%20");
									filebuffer = System.Text.Encoding.UTF8.GetBytes(mark + Environment.NewLine);
									tmpsitemap.Write(filebuffer, 0, filebuffer.Length);
									tmpsitemap.Flush();
									index = lastindex;
									continue;
								}
							}
						}
						if (mark == "h")
						{
							mark = urlpage.Substring(index, 4);
							lastindex = urlpage.IndexOfAny(lastchars, index);
							if (mark == "http")
							{
								mark = urlpage.Substring(index, (lastindex - index));
								mark = mark.Replace(" ", "%20");
								filebuffer = System.Text.Encoding.UTF8.GetBytes(mark + Environment.NewLine);
								tmpsitemap.Write(filebuffer, 0, filebuffer.Length);
								tmpsitemap.Flush();
								index = lastindex;
								continue;
							}
							else
							{
								mark = urlpage.Substring(index, (lastindex - index));
								markindex = mark.IndexOf(':');
								markindex2 = mark.IndexOf('@');
								if ((markindex == -1) && (markindex2 == -1) && (mark.Length > 0))
								{
									markindex = mark.IndexOf('/');
									if (markindex == 0)
									{
										mark = "http://" + host + mark;
									}
									else
									{
										markindex = response.ResponseUri.AbsolutePath.LastIndexOf('/');
										if ((response.ResponseUri.AbsolutePath.Length - 1) == markindex)
										{
											mark = "http://" + host + response.ResponseUri.AbsolutePath + mark;
										}
										else
										{
											markindex = response.ResponseUri.AbsolutePath.LastIndexOf('/');
											mark2 = response.ResponseUri.AbsolutePath.Substring(0, (markindex + 1));
											mark = "http://" + host + mark2 + mark;
										}
									}
									mark = mark.Replace(" ", "%20");
									filebuffer = System.Text.Encoding.UTF8.GetBytes(mark + Environment.NewLine);
									tmpsitemap.Write(filebuffer, 0, filebuffer.Length);
									tmpsitemap.Flush();
									index = lastindex;
									continue;
								}
							}
						}
						if ((mark != "\"") && (mark != "h") && (mark != "'") && (mark != "#"))
						{
							lastindex = urlpage.IndexOfAny(lastchars, index);
							mark = urlpage.Substring(index, (lastindex - index));
							markindex = mark.IndexOf(':');
							markindex2 = mark.IndexOf('@');
							if ((markindex == -1) && (markindex2 == -1) && (mark != url) && (mark.Length > 0))
							{
								markindex = mark.IndexOf('/');
								if (markindex == 0)
								{
									mark = "http://" + host + mark;
								}
								else
								{
									markindex = response.ResponseUri.AbsolutePath.LastIndexOf('/');
									if ((response.ResponseUri.AbsolutePath.Length - 1) == markindex)
									{
										mark = "http://" + host + response.ResponseUri.AbsolutePath + mark;
									}
									else
									{
										markindex = response.ResponseUri.AbsolutePath.LastIndexOf('/');
										mark2 = response.ResponseUri.AbsolutePath.Substring(0, (markindex + 1));
										mark = "http://" + host + mark2 + mark;
									}
								}
								mark = mark.Replace(" ", "%20");
								filebuffer = System.Text.Encoding.UTF8.GetBytes(mark + Environment.NewLine);
								tmpsitemap.Write(filebuffer, 0, filebuffer.Length);
								tmpsitemap.Flush();
								index = lastindex;
							}
						}
						index ++;
					}
				}
			}
			catch
			{
			}
			finally
			{
			}
		}
		
		/// <summary>
		/// Function catch response of Grub server when error occurs.
		/// </summary>
		/// <param name="response">
		/// A <see cref="HttpWebResponse"/> response from server.
		/// </param>
		/// <returns>
		/// A <see cref="System.Int32"/> HTTP status code of answer.
		/// </returns>
		int CatchErrorResponse(ref HttpWebResponse response)
		{
			int statuscode = (int)response.StatusCode;
			if ((statuscode == 401) || (statuscode == 500))
			{
				this.servermessage = String.Empty;
				System.Text.StringBuilder servresponse = new System.Text.StringBuilder();
				using (Stream streamr = response.GetResponseStream())
				{
					Stopwatch speedwatch = new Stopwatch();
					speedwatch.Start();
					byte[] buffer = new byte[1024];
					int bytesRead = 0;
					int tmpbytes = 0;
					int pause = 0;
					while ((bytesRead = streamr.Read(buffer, 0, buffer.Length)) != 0)
					{
						servresponse.Append(System.Text.Encoding.UTF8.GetString(buffer));
						tmpbytes += bytesRead;
						speedwatch.Stop();
						if (this.downloading < 1)
						{
							this.downloading = 1;
						}
						if (tmpbytes >= (this.downloadspeed / this.downloading))
						{
							tmpbytes = 0;
							if (speedwatch.ElapsedMilliseconds > 1000)
							{
								pause = 1000;
							}
							else
							{
								pause = (int)speedwatch.ElapsedMilliseconds;
							}
							System.Threading.Thread.Sleep(1000 - pause);
							speedwatch.Reset();
						}
						speedwatch.Start();
					}
					speedwatch = null;
					streamr.Close();
				}
				this.servermessage = servresponse.ToString();
				servresponse = null;
			}
			response.Close();
			response = null;
			return statuscode;
		}
			
		/// <value>
		/// Property read/write private field downloading.
		/// </value>
		public int Downloading
		{
			get {return this.downloading;}
			set {this.downloading = value;}
		}
		
		/// <value>
		/// Property read/write private field bytessend.
		/// </value>
		public int BytesSend
		{
			get {return this.bytessend;}
			set {this.bytessend = value;}
		}
		
		/// <value>
		/// Property read/write private field filelength.
		/// </value>
		public long FileLength
		{
			get {return this.filelength;}
			set {this.filelength = value;}
		}
		
		/// <value>
		/// Property read private field servermessage.
		/// </value>
		public string ServerMessage
		{
			get {return this.servermessage;}
		}
	}
}
