//  
//  Copyright (C) 2009,2010 Bartek thindil Jasicki
// 
//  This file is part of Grubng
// 
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

/**@file RemoteControl.cs
 * Provide functions for remote control of crawler.*/

using System;
using System.Net;
using System.Text;

namespace Grubng
{
	public delegate void ListenerCaller();
	
	/// <summary>
	/// Provide arguments for webEvent event
	/// </summary>
	public class WebEventArgs:EventArgs
	{
		/// <summary>
		/// Type of event = "config" or "quit"
		/// </summary>
		string type;
		
		/// <summary>
		/// Standard class constructor
		/// </summary>
		/// <param name="message">
		/// A <see cref="System.String"/> type of event, "config" or "quit"
		/// </param>
		public WebEventArgs(string message)
		{
			this.type = message;
		}
		
		/// <summary>
		/// Property return type of event.
		/// </summary>
		public string Type
		{
			get {return this.type;}
		}
	}
	
	/// <summary>
	/// Provide functions for remote control of crawler
	/// </summary>
	public class RemoteControl : IDisposable
	{
		/// <summary>
		/// HttpListener class object.
		/// </summary>
		HttpListener listener;
		/// <summary>
		/// User name for authentication
		/// </summary>
		string username;
		/// <summary>
		/// Password for authentication
		/// </summary>
		string password;
		/// <summary>
		/// Connection context of HttpListener.
		/// </summary>
		HttpListenerContext context;
		/// <summary>
		/// Work thread for get informations about program
		/// </summary>
		Threads crawlerthread;
		/// <summary>
		/// Event raised when configuration of program was updated or received command to quit.
		/// </summary>
		public event EventHandler<WebEventArgs> WebEventHandler;
		/// <summary>
		/// Handler for start HttpListener asynchronous
		/// </summary>
		ListenerCaller caller;
		
		/// <summary>
		/// Standard class constructor
		/// </summary>
		/// <param name="thread">
		/// A <see cref="Threads"/> work thread for get informations about program
		/// </param>
		public RemoteControl(Threads thread)
		{
			this.caller = new ListenerCaller(this.StartListener);
			this.crawlerthread = thread;
			this.listener = new HttpListener();
			this.listener.Realm = "GrubNG";
			this.listener.AuthenticationSchemes = AuthenticationSchemes.Basic;
			Config conf = new Config();
			string[] nodes = new string[] {
				"/configuration/other/remoteport",
				"/configuration/login/username",
				"/configuration/login/password"
			};
			string[] values = Config.ReadFew(nodes, conf.FilePath);
			this.username = values[1];
			System.Security.Cryptography.MD5CryptoServiceProvider hash = new System.Security.Cryptography.MD5CryptoServiceProvider();
			byte[] HashValue = hash.ComputeHash(Encoding.UTF8.GetBytes(values[2]));
			StringBuilder key = new StringBuilder();
			foreach(byte b in HashValue) 
			{
				key.Append(String.Format("{0:x2}", b));
			}
			this.password = key.ToString();
			key = null;
			HashValue = null;
			hash = null;
			conf = null;
			this.listener.Prefixes.Add("http://*:" + values[0] + "/");
		}
		
		/// <summary>
		/// Main remote control function, manage all HTTP request to crawler.
		/// </summary>
		public void Start()
		{
			this.caller.BeginInvoke(null, null);
		}
		
		void StartListener()
		{
			this.listener.Start();
			HttpListenerBasicIdentity identity;
			byte[] buffer;
			int bytesRead;
			string requestString;
			string[] serverResponse;
			WebEventArgs type = new WebEventArgs("quit");
			while (true)
			{
				this.context = this.listener.GetContext();
				if (this.context.User == null)
				{
					continue;
				}
				if (!this.context.Request.HasEntityBody)
				{
					this.Response("Invalid request", 400);
					continue;
				}
				//Allow only POST requests
				if (this.context.Request.HttpMethod != "POST")
				{
					this.Response("Method not allowed", 405);
					continue;
				}
				//Authenticate client
				identity = (HttpListenerBasicIdentity)this.context.User.Identity;
				if ((identity.Name != this.username) || (identity.Password != this.password))
				{
					this.Response("Invalid username or password", 401);
					continue;
				}
				//If client is authenticated, process request
				buffer = new byte[255];
				bytesRead = this.context.Request.InputStream.Read(buffer, 0, buffer.Length);
				requestString = Encoding.UTF8.GetString(buffer, 0, bytesRead);
				this.context.Request.InputStream.Close();
				this.context.Request.InputStream.Dispose();
				//Get crawler status
				if (requestString.StartsWith("status"))
				{
					this.Response(this.Status(), 200);
					continue;
				}
				//Set crawler option
				if (requestString.StartsWith("set"))
				{
					serverResponse = this.Set(requestString);
					this.Response(serverResponse[0], Convert.ToInt32(serverResponse[1]));
					continue;
				}
				//Get crawler option/stat
				if (requestString.StartsWith("get"))
				{
					serverResponse = this.Get(requestString);
					this.Response(serverResponse[0], Convert.ToInt32(serverResponse[1]));
					continue;
				}
				//Stop crawler
				if (requestString.StartsWith("stop"))
				{
					this.Response("OK", 200);
					this.Stop();
					this.OnWebEvent(type);
					type = null;
					return;
				}
				this.Response("Invalid data sent", 400);
			}
		}
		
		/// <summary>
		/// Function return program statistics
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/>
		/// </returns>
		string Status()
		{
			StringBuilder status = new StringBuilder();
			int working = 0;
			double mbused = 0;
			for (int i = 0; i < this.crawlerthread.Crawlers; i++)
			{		
				if (this.crawlerthread.GetField(i, "working") == 1)
				{
					working++;
				}
				mbused += this.crawlerthread.CountDiskUsage(i);
			}
			status.Append(working.ToString());
			status.Append(";");
			status.Append(this.crawlerthread.Crawlers.ToString());
			status.Append(";");
			status.Append(mbused.ToString());
			status.Append(";");
			string[] names = new string[6] {"workunitsend", "invalidarc", "crawledurls", "validurls", "invalidurls", "errorurls"};
			for (int i = 0; i < 6; i++)
			{
				status.Append(this.crawlerthread.GetField(0, names[i]).ToString());
				status.Append(";");
			}
			return status.ToString();
		}
		
		/// <summary>
		/// Function read crawler options or statistics selected crawler
		/// </summary>
		/// <param name="requestString">
		/// A <see cref="System.String"/> client request body
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> array, element 0 is server answer, element 1 is HTTP status code
		/// </returns>
		string[] Get(string requestString)
		{
			string[] returnvalue = new string[2] {"Invalid data sent", "400"};
			//Check correctness of request
			string[] parts = requestString.Split(';');
			if (parts.Length < 2)
			{
				return returnvalue;
			}
			if ((parts[1] != "config") && (parts[1] != "stats"))
			{
				return returnvalue;
			}
			//Get program configuration
			if (parts[1] == "config")
			{
				if (parts.Length < 3)
				{
					return returnvalue;
				}
				Config conf = new Config();
				string[] xpaths = conf.GetNodes().Split(';');
				string[] names = new string[(xpaths.Length - 1)];
				for (int i = 0; i < names.Length; i++)
				{
					names[i] = xpaths[i].Substring(xpaths[i].LastIndexOf('/') + 1);
				}
				int index;
				System.Collections.Generic.List<int> indexes = new System.Collections.Generic.List<int>();
				for (int i = 2; i < parts.Length; i++)
				{
					index  = Array.IndexOf(names, parts[i]);
					if (index == -1)
					{
						continue;
					}
					indexes.Add(index);
				}
				if (indexes.Count == 0)
				{
					return returnvalue;
				}
				string[] nodes = new string[indexes.Count];
				for (int i = 0; i < indexes.Count; i++)
				{
					nodes[i] = xpaths[indexes[i]];
				}
				string[] values = Config.ReadFew(nodes, conf.FilePath);
				conf = null;
				returnvalue[0] = String.Join(";", values);
				returnvalue[1] = "200";
				return returnvalue;
			}
			//Get selected crawler informations
			if (parts[1] == "stats")
			{
				if (parts.Length < 3)
				{
					return returnvalue;
				}
				int index;
				if (!int.TryParse(parts[2], out index))
				{
					return returnvalue;
				}
				if ((index < 0) || (index >= this.crawlerthread.Crawlers))
				{
					return returnvalue;
				}
				string[] names = new string[3] {"status", "curiter", "iter"};
				string[] values = new string[6];
				for (int i = 0; i < 3; i++)
				{
					values[i] = this.crawlerthread.GetField(index, names[i]).ToString();
				}
				values[3] = this.crawlerthread.GetStringField(index, "workfilename");
				values[4] = this.crawlerthread.GetStringField(index, "downloaddate");
				values[5] = this.crawlerthread.CountDiskUsage(index).ToString();
				returnvalue[0] = String.Join(";", values);
				returnvalue[1] = "200";
			}
			return returnvalue;
		}
		
		/// <summary>
		/// Function set configuration of program.
		/// </summary>
		/// <param name="requestString">
		/// A <see cref="System.String"/> with client request body
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> array, element 0 is server answer, element 1 is HTTP status code
		/// </returns>
		string[] Set(string requestString)
		{
			string[] returnvalue = new string[2] {"Invalid data sent", "400"};
			//Check correctness of request
			string[] parts = requestString.Split(';');
			if (parts.Length < 2)
			{
				return returnvalue;
			}
			Config conf = new Config();
			string[] xpaths = conf.GetNodes().Split(';');
			string[] names = new string[(xpaths.Length - 1)];
			for (int i = 0; i < names.Length; i++)
			{
				names[i] = xpaths[i].Substring(xpaths[i].LastIndexOf('/') + 1);
			}
			int index, partindex;
			System.Collections.Generic.List<int> indexes = new System.Collections.Generic.List<int>();
			for (int i = 2; i < parts.Length; i++)
			{
				partindex = parts[i].IndexOf('=');
				if (partindex == -1)
				{
					return returnvalue;
				}
				index  = Array.IndexOf(names, parts[i].Substring(0, partindex));
				if (index == -1)
				{
					continue;
				}
				indexes.Add(index);
			}
			if (indexes.Count == 0)
			{
				return returnvalue;
			}
			string[] nodes = new string[indexes.Count];
			string[] values = new string[indexes.Count];
			for (int i = 0; i < indexes.Count; i++)
			{
				nodes[i] = xpaths[indexes[i]];
				values[i] = parts[indexes[i]].Substring(parts[indexes[i]].IndexOf('=') + 1);
			}
			Config.WriteFew(values, nodes, conf.FilePath);
			conf = null;
			WebEventArgs type = new WebEventArgs("config");
			this.OnWebEvent(type);
			type = null;
			returnvalue[0] = "OK";
			returnvalue[1] = "200";
			return returnvalue;
		}
		
		/// <summary>
		/// Function invoke event. Raised when configuration file was updated or received quit from program command.
		/// </summary>
		/// <param name="args">
		/// A <see cref="WebEventArgs"/> type of event "config" or "quit"
		/// </param>
		public void OnWebEvent(WebEventArgs args)
		{
			EventHandler<WebEventArgs> handler = this.WebEventHandler;
			if (handler != null)
			{
				handler(this, args);
			}
		}
		
		/// <summary>
		/// Function send response to client.
		/// </summary>
		/// <param name="body">
		/// A <see cref="System.String"/> text to send to client.
		/// </param>
		/// <param name="statuscode">
		/// A <see cref="System.Int32"/> HTTP status code of response.
		/// </param>
		void Response (string body, int statuscode)
		{
			this.context.Response.StatusCode = statuscode;
			this.context.Response.ContentType = "text/plain; charset=utf-8";
			this.context.Response.Headers.Add(HttpResponseHeader.ContentLanguage, "en");
			this.context.Response.Headers.Add(HttpResponseHeader.Server, "GrubNG");
			if (statuscode == 405)
			{
				this.context.Response.Headers.Add(HttpResponseHeader.Allow, "POST");
			}
			byte[] buffer2 = Encoding.UTF8.GetBytes(body);
			this.context.Response.ContentLength64 = buffer2.Length;
			try
			{
				this.context.Response.OutputStream.Write(buffer2, 0, buffer2.Length);
				this.context.Response.OutputStream.Close();
				this.context.Response.Close();
			}
			catch (System.Net.Sockets.SocketException)
			{
			}
			catch (System.IO.IOException)
			{
			}
			buffer2 = null;
			this.context.Response.Close();
		}
		
		/// <summary>
		/// Function stop listener.
		/// </summary>
		public void Stop()
		{
			this.listener.Stop();
		}
		
		/// <summary>
		/// Function dispose unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			this.listener = null;
			this.crawlerthread.Dispose();
		}
	}
}
