// Sitemap.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Sitemap.cs
 * Provide functions for manipulate sitemap files.*/

using System;
using System.IO;
using System.IO.Compression;
using System.Net;
using System.Diagnostics;
using System.Xml;

namespace Grubng
{
	
	/// <summary>
	/// Provide function for manipulate sitemap files.
	/// </summary>
	public sealed class Sitemap
	{
		/// <summary>
		/// Amount of bytes uploaded on server.
		/// </summary>
		int bytessend;
		/// <summary>
		/// Amount of bytes to upload on server.
		/// </summary>
		long filelength;
		/// <summary>
		/// Full path with filename to sitemap.
		/// </summary>
		string filename;
		/// <summary>
		/// If true, upload ongoing.
		/// </summary>
		bool upload;
		
		/// <summary>
		/// Standar class constructor.
		/// </summary>
		/// <param name="file">
		/// A <see cref="System.String"/> full path (including filename to sitemap file)
		/// </param>
		public Sitemap(string file)
		{
			this.filelength = 1;
			this.filename = file;
		}
		
		/// <summary>
		/// Function check propriety of sitemap file.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if file is sitemap file, otherwise false.
		/// </returns>
		public bool CheckSitemap()
		{
			string[] extension = this.filename.Split('.');
			int index = extension.Length - 1;
			bool decompress = false;
			string xmlfile = this.filename;
			if (extension[index] == "gz")
			{
				decompress = true;
				xmlfile = this.filename.Substring(0, (this.filename.Length - 3));
				using (FileStream file = File.OpenRead(this.filename))
				{
					using (GZipStream gzstream = new GZipStream(file, CompressionMode.Decompress))
					{
						using (FileStream unfile = File.Create(xmlfile))
						{
							byte[] buffer = new byte[1024];
							int bytesRead = 0;
							while (true)
							{
								bytesRead = gzstream.Read(buffer, 0, buffer.Length);
								if (bytesRead == 0)
								{
									break;
								}
								unfile.Write(buffer, 0, bytesRead);
							}
						}
					}
				}
			}
			try
			{
				XmlDocument doc = new XmlDocument();
				doc.Load(xmlfile);
				XmlNode noder = doc.SelectSingleNode("/");
				string xmlvalue = noder.ChildNodes[1].Attributes["xmlns"].InnerText;
				if (xmlvalue != "http://www.sitemaps.org/schemas/sitemap/0.9")
				{
					if (decompress)
					{
						File.Delete(xmlfile);
					}
					return false;
				}
				XmlElement element = (XmlElement)noder.ChildNodes[1];
				string urlloc;
				this.filelength = element.GetElementsByTagName("url").Count;
				for (this.bytessend = 0; this.bytessend < this.filelength; this.bytessend ++)
				{
					urlloc = element.GetElementsByTagName("loc")[this.bytessend].InnerText;
					if (String.IsNullOrEmpty(urlloc))
					{
						if (decompress)
						{
							File.Delete(xmlfile);
						}
						return false;
					}
				}
			}
			catch
			{
				if (decompress)
				{
					File.Delete(xmlfile);
				}
				return false;
			}
			if (decompress)
			{
				File.Delete(xmlfile);
			}
			return true;
		}
		
		/// <summary>
		/// Function upload sitemap file on server.
		/// </summary>
		/// <param name="expectfix">
		/// If <see cref="System.Boolean"/> true, apply Expect HTTP header fix.
		/// </param>
		/// <returns>
		/// A <see cref="System.Int32"/> array of integer, element 0 - is HTTP Status Code reveived from
		/// server, element 1 is amount of bytes send by server, element 2 - amount of bytes
		/// send by client.
		/// </returns>
		public int[] UploadSitemap(bool expectfix)
		{
			int[] returnvalue = new int[3];
			string[] extension = this.filename.Split('.');
			int index = extension.Length - 1;
			if (extension[index] == "xml")
			{
				Compress.Do(this.filename, ".gz");
				File.Delete(this.filename);
				this.filename = this.filename + ".gz";
			}
			Config conf = new Config();
			string[] nodes = new string[] {
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/proxystatus",
				"/configuration/other/sitemapserver",
				"/configuration/other/deletesitemap",
				"configuration/work/uploadrate"};
			string[] values = Config.ReadFew(nodes, conf.FilePath);
			int uploadspeed = Convert.ToInt32(values[5]) * 1024;
			string proxyaddress, proxyport, proxylogin, proxypass;
			if (values[2] == "custom")
			{
				nodes = new string[] {
					"/configuration/work/proxyaddress",
					"/configuration/work/proxyport",
					"/configuration/work/proxylogin",
					"/configuration/work/proxypass"};
				string[] proxysets = Config.ReadFew(nodes, conf.FilePath);
				proxyaddress = proxysets[0];
				proxyport = proxysets[1];
				proxylogin = proxysets[2];
				proxypass = proxysets[3];
				proxysets = null;
			}
			else
			{
				proxyaddress = null;
				proxyport = null;
				proxylogin = null;
				proxypass = null;
			}
			nodes = null;
			HttpWebResponse response;
			FileStream file = new FileStream(this.filename, FileMode.Open, FileAccess.Read);
			string[] filename2 = this.filename.Split(Path.DirectorySeparatorChar);
			string mapurl = values[3] + values[0] + "/" + filename2[filename2.Length - 1];
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(mapurl);
			if (values[2] == "custom")
			{
				Uri proxyuri = new Uri(proxyaddress + ":" + proxyport);
				request.Proxy = new WebProxy(proxyuri);
				if (!String.IsNullOrEmpty(proxylogin))
				{
					request.Proxy.Credentials = new NetworkCredential(proxylogin, proxypass);
				}
			}
			if (values[2] == "none")
			{
				request.Proxy = null;
			}
			this.filelength = file.Length;
			request.Method = "PUT";
			request.ContentLength = file.Length;
			request.Timeout = 600000;
			request.Credentials = new NetworkCredential(values[0], values[1]);
			request.PreAuthenticate = true;
			request.AllowWriteStreamBuffering = false;
			System.Reflection.Assembly asm = System.Reflection.Assembly.GetExecutingAssembly();
			request.UserAgent = asm.GetName().Name + "/" + asm.GetName().Version.ToString();
			asm = null;
			if (expectfix)
			{
				request.ServicePoint.Expect100Continue = false;
			}
			Stream streamw1;
			try
			{
				streamw1 = request.GetRequestStream();
			}
			catch (WebException)
			{
                returnvalue[0] = 503;
				returnvalue[2] = 0;
				return returnvalue;
			}
			byte[] buffer = new byte[1024];
			int bytesRead = 0;
			this.bytessend = 0;
			int tmpbytes = 0;
			int pause = 0;
			returnvalue[1] = (int)this.filelength;
			returnvalue[2] = 0;
			Stopwatch speedwatch = new Stopwatch();
			try
			{
				speedwatch.Start();
				while ((bytesRead = file.Read(buffer, 0, buffer.Length)) != 0)
				{
					streamw1.Write(buffer, 0, bytesRead);
					this.bytessend += bytesRead;
					tmpbytes += bytesRead;
					speedwatch.Stop();
					if (tmpbytes >= uploadspeed)
					{
						tmpbytes = 0;
						if (speedwatch.ElapsedMilliseconds > 1000)
						{
							pause = 1000;
						}
						else
						{
							pause = (int)speedwatch.ElapsedMilliseconds;
						}
						System.Threading.Thread.Sleep(1000 - pause);
						speedwatch.Reset();
					}
					speedwatch.Start();
				}
				speedwatch = null;
			}
			catch
			{
				streamw1.Close();
				streamw1 = null;
				buffer = null;
				file.Close();
				file = null;
                returnvalue[0] = 408;
				returnvalue[2] = this.bytessend;
				return returnvalue;
			}
			streamw1.Close();
			streamw1.Dispose();
			buffer = null;
			file.Close();
			file.Dispose();
			try
			{
				response = (HttpWebResponse)request.GetResponse();
				returnvalue[0] = (int)response.StatusCode;
				returnvalue[1] = (int)response.ContentLength + response.Headers.ToString().Length;
				response.Close();
			}
			catch (WebException web)
			{
				try
				{
					response = (HttpWebResponse)web.Response;
					returnvalue[0] = (int)response.StatusCode;
					returnvalue[1] = (int)response.ContentLength + response.Headers.ToString().Length;
					response.Close();
				}
				catch
				{
					returnvalue[0] = 0;
				}
			}
			if ((returnvalue[0] == 200) && (values[4] == "Y"))
			{
				File.Delete(this.filename);
			}
			values = null;
			return returnvalue;
		}
		
		/// <summary>
		/// Function create sitemap file from temporary sitemap.
		/// </summary>
		/// <param name="compress">
		/// A <see cref="System.Boolean"/> if true compress created sitemap, otherwise false.
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if sitemaps was created, otherwise false.
		/// </returns>
		public bool CreateSitemap(bool compress)
		{
			string tmpsitemap = this.filename.Substring(0, (this.filename.Length - 3)) + "tmp";
			FileInfo fileinfo = new FileInfo(tmpsitemap);
			if (fileinfo.Length == 0)
			{
				File.Delete(tmpsitemap);
				return false;
			}
			fileinfo = null;
			StreamReader reader = new StreamReader(tmpsitemap);
			string url;
			System.Collections.Generic.List<string> urls = new System.Collections.Generic.List<string>();
			int index;
			int amount = 0;
			XmlTextWriter xmlw = new XmlTextWriter(this.filename, null);
			xmlw.Formatting = Formatting.Indented;
			xmlw.WriteStartDocument(true);
			xmlw.WriteStartElement("urlset");
			while (!reader.EndOfStream)
			{
				url = reader.ReadLine();
				index = urls.IndexOf(url);
				if (index > -1)
				{
					continue;
				}
				if (!System.Text.RegularExpressions.Regex.IsMatch(url, 
				                                                  @"^http(s?)\:\/\/[a-z0-9]*([a-z0-9\=\-\.\?\,\'\/\\\+&amp;%\$#_~]*)?$",
				                                                  System.Text.RegularExpressions.RegexOptions.IgnoreCase))
				{
					continue;
				}
				urls.Add(url);
				xmlw.WriteStartElement("url");
				xmlw.WriteElementString("loc", url);
				xmlw.WriteEndElement();
				xmlw.Flush();
				amount ++;
			}
			reader.Dispose();
			xmlw.WriteEndElement();
			xmlw.Flush();
			xmlw.Close();
			if (amount == 0)
			{
				File.Delete(tmpsitemap);
				File.Delete(this.filename);
				return false;
			}
			System.Threading.Thread.Sleep(100);
			this.InsertSchema();
			File.Delete(tmpsitemap);
			if (compress)
			{
				Compress.Do(this.filename, ".gz");
				File.Delete(this.filename);
			}
			return true;
		}
		
		/// <summary>
		/// Function write xml schema to sitemap file. Workaround bug in Mono.
		/// </summary>
		public void InsertSchema()
		{
			XmlDocument doc = new XmlDocument();
			doc.Load(this.filename);
			XmlNode node = doc.ChildNodes.Item(1);
			XmlAttribute attr = doc.CreateAttribute("xmlns");
			attr.InnerText = "http://www.sitemaps.org/schemas/sitemap/0.9";
			node.Attributes.Append(attr);
			doc.Save(this.filename);
			doc = null;
		}
		
		/// <summary>
		/// Property read/write private field bytessend.
		/// </summary>
		public int BytesSend
		{
			get {return this.bytessend;}
			set {this.bytessend = value;}
		}
		
		/// <summary>
		/// Property read/write private field filelength.
		/// </summary>
		public long FileLength
		{
			get {return this.filelength;}
			set {this.filelength = value;}
		}
		
		/// <summary>
		/// Property read/write private field upload.
		/// </summary>
		public bool Upload
		{
			get {return this.upload;}
			set {this.upload = value;}
		}
	}
}
