// Threads.cs
// 
// Copyright (C) 2008,2009,2010,2011 Bartek Jasicki
//
// This file is part of Grubng.
//
// Grubng is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Threads.cs
 * Provide functions for managing crawler threads.*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Timers;
using System.Threading;

namespace Grubng
{
	public delegate void CrawlerCaller(int num);
	
	/// <summary>
	/// Provide function for managing crawlers threads.
	/// </summary>
	public sealed class Threads : IDisposable
	{
		/// <summary>
		/// Object from Crawler class. Provide functions to upload, crawl and download workunits.
		/// </summary>
		Crawler grubng;
		/// <summary>
		/// Object from Config class. Provide function to read and write setting to grub.xml file.
		/// </summary>
		Config conf;
		/// <summary>
		/// Object from Sitemap class. Provide functions to create and upload sitemap file.
		/// </summary>
		Sitemap sitemap;
		/// <summary>
		/// Enable/disable auto creating sitemaps
		/// </summary>
		bool createsitemaps;
		/// <summary>
		/// List of servers addresses for uploading .arc files.
		/// </summary>
		System.Collections.Generic.List<StringBuilder> uploadserver;
		/// <summary>
		/// Path to files
		/// </summary>
		string filepath;
		/// <summary>
		/// List of threads for downloading/crawling/uploading workunit
		/// </summary>
		System.Collections.Generic.List<IAsyncResult> crawlerthread;
		/// <summary>
		/// Amount of received/sent bytes by client in this uptime (replace uptimesend, uptimereceived)
		/// </summary>
		double[] uptimestats;
		/// <summary>
		/// List of crawling informations (URL's, amount of URL's, etc)
		/// </summary>
		System.Collections.Generic.List<Workunit> workunits;
		/// <summary>
		/// Amount of crawlers
		/// </summary>
		int crawlers;
		/// <summary>
		/// If true, run crawlers in automatic mode
		/// </summary>
		bool running;
		/// <summary>
		/// List of addresses of current crawled url
		/// </summary>
		System.Collections.Generic.List<StringBuilder> cururl;
		/// <summary>
		/// Amount of request timeout's
		/// </summary>
		int timeout;
		/// <summary>
		/// True if any crawler downloading workunit.
		/// </summary>
		bool downloading;
		/// <summary>
		/// List of dates when workunit was downloaded
		/// </summary>
		System.Collections.Generic.List<StringBuilder> downloaddate;
		/// <summary>
		/// True if any crawler uploading workunit.
		/// </summary>
		bool uploading;
		/// <summary>
		/// Check if resources are disposed.
		/// </summary>
		bool disposedres;
		/// <summary>
		/// True if connection error appear.
		/// </summary>
		bool connectionerror;
		/// <summary>
		/// Current stage of uploading.
		/// </summary>
		short uploadstage;
		/// <summary>
		/// Event raised when crawler sent any message.
		/// </summary>
		public event EventHandler messageEmit;
		/// <summary>
		/// Amount of maximum working crawlers.
		/// </summary>
		int maxcrawlers;
		/// <summary>
		/// Compression method - as a file extension.
		/// </summary>
		string compression;
		/// <summary>
		/// Thread to start crawlers.
		/// </summary>
		Thread startthread;
		/// <summary>
		/// List with dispatch/upload servers messages.
		/// </summary>
		System.Collections.Generic.List<string> servermessage;
		/// <summary>
		/// Handler for start asynchronous crawling
		/// </summary>
		CrawlerCaller caller;
		/// <summary>
		/// List of integer array with informations about crawler
		/// </summary>
		List<int[]> crawlerinfo;
		/// <summary>
		/// Array of integer with crawling statistics
		/// </summary>
		int[] crawlerstats;
		/// <summary>
		/// Overall client statistic.
		/// </summary>
		int[] overallstats;
		/// <summary>
		/// Overall amount of sent/received MB.
		/// </summary>
		double[] overallconnectionstats;
		/// <summary>
		/// User informations: 0 - name, 1 - password.
		/// </summary>
		string[] user;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		public Threads()
		{
			ThreadPool.SetMaxThreads(75, 150);
			this.caller = new CrawlerCaller(this.WorkThread);
			this.crawlerthread = new System.Collections.Generic.List<IAsyncResult>();
			this.crawlerinfo = new List<int[]>();
			this.crawlerstats = new int[6] {0, 0, 0, 0, 0, 0};
			this.uptimestats = new double[2] {0, 0};
			this.overallstats = new int[6];
			this.user = new string[2];
			this.overallconnectionstats = new double[2];
			this.conf = new Config();
			this.ReadStats();
			this.workunits = new System.Collections.Generic.List<Workunit>();
			this.cururl = new System.Collections.Generic.List<StringBuilder>();
			this.downloaddate = new System.Collections.Generic.List<StringBuilder>();
			this.uploadserver = new System.Collections.Generic.List<StringBuilder>();
			this.servermessage = new System.Collections.Generic.List<string>();
			this.grubng = new Crawler();
			this.SetPrefs(true);
		}
		
		/// <summary>
		/// Function write stats to grub.xml file. Write informations about crawled url's, send and received bytes
		/// </summary>
		public void WriteStats()
		{
			string[] xmlnodes = new string[] {
				"/configuration/statistics/worksend",
				"/configuration/statistics/invalidarc",
				"/configuration/statistics/urlcrawled",
				"/configuration/statistics/success",
				"/configuration/statistics/failure",
				"/configuration/statistics/errors",
				"/configuration/statistics/mbreceived",
				"/configuration/statistics/mbsend"
			};
			string[] xmlvalues = new string[] {
				this.overallstats[4].ToString(),
				this.overallstats[5].ToString(),
				this.overallstats[0].ToString(),
				this.overallstats[1].ToString(),
				this.overallstats[2].ToString(),
				this.overallstats[3].ToString(),
				this.overallconnectionstats[1].ToString(),
				this.overallconnectionstats[0].ToString()
			};
			Config.WriteFew(xmlvalues, xmlnodes, this.conf.FilePath);
		}
		
		/// <summary>
		/// Function read stats from grub.xml file. Read informations about
		/// crawled URL's, send and received bytes.
		/// </summary>
		void ReadStats()
		{
			string[] nodes = new string[] {
				"/configuration/statistics/urlcrawled",
				"/configuration/statistics/success",
				"/configuration/statistics/failure",
				"/configuration/statistics/errors",
				"/configuration/statistics/worksend",
				"/configuration/statistics/invalidarc",
				"/configuration/statistics/mbreceived",
				"/configuration/statistics/mbsend"};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			for (int i = 0; i < 6; i++)
			{
				this.overallstats[i] = Convert.ToInt32(values[i]);
			}
			string decseparator = System.Globalization.CultureInfo.CurrentUICulture.NumberFormat.NumberDecimalSeparator;
			values[6] = System.Text.RegularExpressions.Regex.Replace(values[6], @"\D", decseparator);
			this.overallconnectionstats[1] = Convert.ToDouble(values[6]);
			values[7] = System.Text.RegularExpressions.Regex.Replace(values[7], @"\D", decseparator);
			this.overallconnectionstats[0] = Convert.ToDouble(values[7]);
			nodes = null;
			values = null;
		}
		
		/// <summary>
		/// Function set preferences. Read preferences from grub.xml file and write it to variables.
		/// </summary>
		/// <param name="create">
		/// A <see cref="System.Boolean"/> If true = fill all lists
		/// </param>
		public void SetPrefs(bool create)
		{
			this.DisposeCheck();
			string[] nodes = new string[] {
				"/configuration/login/username",
				"/configuration/login/password",
				"/configuration/work/crawlers",
				"/configuration/work/createsitemaps"};
			string[] values = Config.ReadFew(nodes, this.conf.FilePath);
			this.user[0] = values[0];
			this.user[1] = values[1];
			int mcrawlers = Convert.ToInt32(values[2]);
			if (mcrawlers > 50)
			{
				mcrawlers = 50;
			}
			if (mcrawlers < 1)
			{
				mcrawlers = 1;
			}
			this.maxcrawlers = mcrawlers;
			if (values[3] == "Y")
			{
				this.createsitemaps = true;
			}
			else
			{
				this.createsitemaps = false;
			}
			if (create)
			{
				nodes = new string[] {
					"/configuration/work/workdir",
					"/configuration/work/crawlers",
					"/configuration/work/compression"};
				values = Config.ReadFew(nodes, this.conf.FilePath);
				this.filepath = values[0];
				this.crawlers = Convert.ToInt32(values[1]);
				this.compression = values[2];
				for (int i = 0; i < this.crawlers; i++)
				{
					this.AddToLists();
				}
			}
			nodes = null;
			values = null;
			if (this.maxcrawlers > this.crawlers)
			{
				int newamount = this.maxcrawlers - this.crawlers;
				for (int i = 0; i < newamount; i++)
				{
					this.AddToLists();
					this.crawlers++;
				}
			}
			if (this.maxcrawlers < this.crawlers)
			{
				bool working = false;
				for (int i = 0; i < this.crawlers; i++)
				{
					if (this.crawlerthread[i] != null)
					{
						if (!this.crawlerthread[i].IsCompleted)
						{
							working = true;
						}
					}
				}
				if (!working)
				{
					for (int i = 0; i < this.crawlers; i++)
					{
						this.crawlers--;
						i = 0;
						this.RemoveFromLists(this.crawlers);
						if (this.crawlers == this.maxcrawlers)
						{
							break;
						}
					}
				}
				else
				{
					this.RemoveCrawlers();
				}
			}
			this.grubng.SetPath();
		}
		
		/// <summary>
		/// Function add one element to each list
		/// </summary>
		void AddToLists()
		{
			this.crawlerthread.Add(null);
			this.crawlerinfo.Add(new int[6]{0, 0, (int)CrawlerStatus.Stopped, 0, -1, 0});
			this.workunits.Add(new Workunit());
			this.cururl.Add(new StringBuilder());
			this.downloaddate.Add(new StringBuilder());
			this.uploadserver.Add(new StringBuilder());
			this.servermessage.Add(String.Empty);
		}
		
		/// <summary>
		/// Function remove one element from each list
		/// </summary>
		/// <param name="iter">
		/// A <see cref="System.Int32"/> Number of element to remove
		/// </param>
		void RemoveFromLists(int iter)
		{
			this.crawlerthread.RemoveAt(iter);
			this.workunits.RemoveAt(iter);
			this.crawlerinfo.RemoveAt(iter);
			this.cururl.RemoveAt(iter);
			this.downloaddate.RemoveAt(iter);
			this.uploadserver.RemoveAt(iter);
			this.servermessage.RemoveAt(iter);
		}
		
		/// <summary>
		/// Function remove unused crawlers
		/// </summary>
		void RemoveCrawlers()
		{
			for (int i = this.crawlerthread.Count - 1; i >= this.maxcrawlers; i--)
			{
				if (this.crawlerthread[i].IsCompleted)
				{
					this.RemoveFromLists(i);
					this.crawlers--;
				}
				else
				{
					break;
				}
			}
		}
		
		/// <summary>
		/// Function to start selected crawler.
		/// </summary>
		/// <param name="iter">
		/// A <see cref="System.Int32"/> Number of thread to start.
		/// </param>
		public void StartCrawler(int iter)
		{
			this.DisposeCheck();
			if (this.crawlerthread[iter] != null)
			{
				if (!this.crawlerthread[iter].IsCompleted)
				{
					return;
				}
			}
			if (this.crawlers > this.maxcrawlers)
			{
				this.RemoveCrawlers();
			}
			if ((iter + 1) > this.maxcrawlers)
			{
				return;
			}
			this.crawlerinfo[iter][1] = 1;
			this.crawlerthread[iter] = this.caller.BeginInvoke(iter, null, null);
		}
		
		/// <summary>
		/// Function to start crawling.
		/// </summary>
		public void Start()
		{
			this.DisposeCheck();
			this.startthread = new Thread(this.StartThread);
			this.startthread.Name = "start";
			this.startthread.Start();
		}
		
		/// <summary>
		/// Function start all stopped/paused crawlers
		/// </summary>
		void StartThread()
		{
			for (int i = 0; i < this.crawlers; i++)
			{
				if (this.connectionerror)
				{
					break;
				}
				switch(this.crawlerinfo[i][1])
				{
				case 0:
					this.StartCrawler(i);
					Thread.Sleep(1000);
					break;
				case 2:
					this.crawlerinfo[i][1] = 1;
					this.crawlerinfo[i][2] = (int)CrawlerStatus.Crawling;
					this.OnMessageEmit(EventArgs.Empty);
					Thread.Sleep(1000);
					break;
				default:
					break;
				}
			}
		}
		
		/// <summary>
		/// Function search for compressed .arc file.
		/// </summary>
		/// <param name="filename">
		/// A <see cref="System.String"/> full filename (including path) for arc file.
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> with full path to compressed arc file or empty string.
		/// </returns>
		string FindFile(string filename)
		{
			if (!File.Exists(filename + this.compression))
			{
				string[] extensions = new string[] {".gz", ".lzma"};
				foreach (string extension in extensions)
				{
					if (File.Exists(filename + extension))
					{
						return filename + extension;
					}
				}
				return String.Empty;
			}
			return filename + this.compression;
		}
		
		/// <summary>
		/// Function count disk usage by selected crawler
		/// </summary>
		/// <param name="num">
		/// A <see cref="System.Int32"/> number of crawler.
		/// </param>
		/// <returns>
		/// Amount of MB used on disk by selected crawler.
		/// </returns>
		public double CountDiskUsage(int num)
		{
			double diskusage = 0;
			FileInfo filein = new FileInfo(this.filepath + "workunit" + num.ToString());
			if (filein.Exists)
			{
				diskusage = Convert.ToDouble(filein.Length);
			}
			filein = new FileInfo(this.filepath + this.workunits[num].ArcName);
			if (!filein.Exists)
			{
				string filename = FindFile(this.filepath + this.workunits[num].ArcName);
				if (filename.Length > 0)
				{
					filein = new FileInfo(filename);
				}
			}
			if (filein.Exists)
			{
				diskusage += Convert.ToDouble(filein.Length);
			}
			if (this.createsitemaps)
			{
				filein = new FileInfo(this.filepath + this.workunits[num].ArcName + ".xml.gz");
				if (!filein.Exists)
				{
					filein = new FileInfo(this.filepath + this.workunits[num].ArcName + ".tmp");
				}
				if (filein.Exists)
				{
					diskusage += Convert.ToDouble(filein.Length);
				}
			}
			filein = null;
			if (diskusage == 0)
			{
				return 0;
			}
			else
			{
				return Math.Round((diskusage / 1048576), 1);
			}
		}
		
		/// <summary>
		/// Work thread function. Function download, crawl and upload .arc files.
		/// </summary>
		void WorkThread(int num)
		{
			string test;
			this.crawlerinfo[num][4] = -1;
			StringBuilder tmp = new StringBuilder();
			do
			{
				if (tmp.Length > 0)
				{
					tmp.Remove(0, tmp.Length);
				}
				if (this.crawlers > this.maxcrawlers)
				{
					this.RemoveCrawlers();
				}
				if ((num + 1) > this.maxcrawlers)
				{
					break;
				}
				//If no old workunits, download new
				if (!File.Exists(this.filepath + num.ToString() +".xml"))
				{
					if (this.downloading)
					{
						this.crawlerinfo[num][2] = (int)CrawlerStatus.Waiting;
						this.crawlerinfo[num][4] = (int)CrawlerMessage.WaitForDownload;
						this.OnMessageEmit(EventArgs.Empty);
						while (this.downloading)
						{
							Thread.Sleep(1000);
							if (this.crawlerinfo[num][1] == 0)
							{
								this.crawlerinfo[num][2] = (int)CrawlerStatus.Stopped;
								this.OnMessageEmit(EventArgs.Empty);
								return;
							}
						}
					}
					this.downloading = true;
					if (!this.DownloadThread(num))
					{
						return;
					}
				}
				else
				{
					tmp.Append(Config.ReadConfig("/work/workunit", this.filepath + num.ToString() + ".xml"));
					if (tmp.ToString() == "Invalid XML")
					{
						File.Delete(this.filepath + num.ToString() + ".xml");
						tmp.Remove(0, tmp.Length);
						Config.CreateSecondary(this.filepath + num.ToString() + ".xml");
					}
				}
				//If compressed .arc file not exist, start/continue crawling
				if (tmp.Length == 0)
				{
					if (!this.workunits[num].Read(this.filepath + "workunit" + num.ToString(), this.user[0]))
					{
						File.Delete(this.filepath + "workunit" + num.ToString());
						File.Delete(this.filepath + num.ToString() + ".xml");
						this.downloading = false;
						continue;
					}
					for (int it = 0; it < this.crawlers; it++)
					{
						if (it != num)
						{
							if (File.Exists(this.filepath + "workunit" + it.ToString()) && this.workunits[it].ArcName.Length == 0)
							{
								Thread.Sleep(1000);
							}
							if (this.workunits[num].ArcName == this.workunits[it].ArcName)
							{
								Thread.Sleep(200);
								File.Delete(this.filepath + "workunit" + num.ToString());
								File.Delete(this.filepath + num.ToString() + ".xml");
								this.downloading = false;
								continue;
							}
						}
					}
					if (this.downloaddate[num].Length == 0)
					{
						this.downloaddate[num] = new StringBuilder(File.GetCreationTime(this.filepath + "workunit" + num.ToString()).ToString("yyyy/MM/dd/HH:mm:ss"));
					}
					test = Config.ReadConfig("/work/workiter", this.filepath + num.ToString() + ".xml");
					this.crawlerinfo[num][5] = Convert.ToInt32(test);
					this.downloading = false;
					if (this.CrawlerThread(num))
					{
						if (this.uploading)
						{
							this.crawlerinfo[num][2] = (int)CrawlerStatus.InQueue;
							this.crawlerinfo[num][4] = (int)CrawlerMessage.WaitForUpload;
							this.OnMessageEmit(EventArgs.Empty);
						}
						while (this.uploading)
						{
							Thread.Sleep(1000);
							if (this.crawlerinfo[num][1] == 2)
							{
								this.crawlerinfo[num][2] = (int)CrawlerStatus.Paused;
								this.OnMessageEmit(EventArgs.Empty);
								while (this.crawlerinfo[num][1] == 2)
								{
									Thread.Sleep(1000);
								}
							}
							if (this.crawlerinfo[num][1] == 0)
							{
								this.crawlerinfo[num][2] = (int)CrawlerStatus.Stopped;
								this.OnMessageEmit(EventArgs.Empty);
								break;
							}
						}
						this.uploading = true;
						this.UploadThread(num);
						this.uploading = false;
					}
					else
					{
						string compfilename = FindFile(this.filepath + this.workunits[num].ArcName);
						if ((!File.Exists(this.filepath + this.workunits[num].ArcName)) && (compfilename.Length == 0))
						{
							File.Delete(this.filepath + "workunit" + num.ToString());
							File.Delete(this.filepath + num.ToString() + ".xml");
						}
					}
				}
				else
				{
					this.workunits[num].FastRead(this.filepath + "workunit" + num.ToString(), this.user[0]);
					if (this.downloaddate[num].Length == 0)
					{
						this.downloaddate[num] = new StringBuilder(File.GetCreationTime(this.filepath + "workunit" + num.ToString()).ToString("yyyy/MM/dd/HH:mm:ss"));
					}
					if (this.uploading)
					{
						this.crawlerinfo[num][2] = (int)CrawlerStatus.InQueue;
						this.crawlerinfo[num][4] = (int)CrawlerMessage.WaitForUpload;
						this.OnMessageEmit(EventArgs.Empty);
					}
					while (this.uploading)
					{
						Thread.Sleep(1000);
						if (this.crawlerinfo[num][1] == 2)
						{
							this.crawlerinfo[num][2] = (int)CrawlerStatus.Paused;
							this.OnMessageEmit(EventArgs.Empty);
							while (this.crawlerinfo[num][1] == 2)
							{
								Thread.Sleep(1000);
							}
							this.crawlerinfo[num][2] = (int)CrawlerStatus.InQueue;
							this.crawlerinfo[num][4] = (int)CrawlerMessage.WaitForUpload;
							this.OnMessageEmit(EventArgs.Empty);
						}
						if (this.crawlerinfo[num][1] == 0)
						{
							this.crawlerinfo[num][2] = (int)CrawlerStatus.Stopped;
							this.OnMessageEmit(EventArgs.Empty);
							break;
						}
					}
					this.uploading = true;
					this.UploadThread(num);
					this.uploading = false;
				}
				this.workunits[num].Clear();
				Thread.Sleep(1000);
			}
			while (this.running);
			this.crawlerinfo[num][1] = 0;
		}
		
		/// <summary>
		/// Download workunit function. Function download new workunit from server.
		/// </summary>
		/// <param name="num">
		/// A <see cref="System.Int32"/> Number of thread.
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if workunit was downloaded, otherwise false.
		/// </returns>
		bool DownloadThread(int num)
		{
			//Download new workunit
			this.crawlerinfo[num][4] = -1;
			this.servermessage[num] = String.Empty;
			this.crawlerinfo[num][2] = (int)CrawlerStatus.Downloading;
			this.OnMessageEmit(EventArgs.Empty);
			this.grubng.Downloading += 1;
			int[] status = this.grubng.Download(ref this.user[0], ref this.user[1], ref num);
			this.grubng.Downloading -= 1;
			if (status[0] == 200)
			{
				Config.CreateSecondary(this.filepath + num.ToString() + ".xml");
				double mbtmp = Convert.ToDouble(status[1]) / 1048576;
				this.uptimestats[1] += mbtmp;
				this.overallconnectionstats[1] += mbtmp;
				mbtmp = Convert.ToDouble(status[2]) / 1048576;
				this.uptimestats[0] += mbtmp;
				this.overallconnectionstats[0] += mbtmp;
				this.crawlerinfo[num][4] = (int)CrawlerMessage.Done;
				this.OnMessageEmit(EventArgs.Empty);
				this.downloaddate[num] = new StringBuilder(DateTime.Now.ToString("yyyy/MM/dd/HH:mm:ss"));
			}
			else
			{
				switch (status[0])
				{
				case 401:
					this.crawlerinfo[num][4] = (int)CrawlerMessage.InvalidLogin;
					break;
				case 408:
					this.crawlerinfo[num][4] = (int)CrawlerMessage.RequestTimeout;
					break;
				case 500:
					this.crawlerinfo[num][4] = (int)CrawlerMessage.InternalServerError;
					this.servermessage[num] = this.grubng.ServerMessage;
					break;
				case 501:
					this.crawlerinfo[num][4] = (int)CrawlerMessage.ConnectionError;
					break;
				case 503:
					this.crawlerinfo[num][4] = (int)CrawlerMessage.NoWorkunits;
					break;
				default:
					this.crawlerinfo[num][4] = (int)CrawlerMessage.UnknownError;
					break;
				}
				this.downloading = false;
				if (this.crawlerinfo[num][1] == 1)
				{
					this.crawlerinfo[num][1] = 0;
				}
				this.crawlerinfo[num][2] = (int)CrawlerStatus.DownloadError;
				this.OnMessageEmit(EventArgs.Empty);
				this.connectionerror = true;
				return false;
			}
			return true;
		}
		
		/// <summary>
		/// Crawling thread function. Function crawl all url's from workunit, checking propriety of .arc
		/// file and compressing it.
		/// </summary>
		/// <param name="num">
		/// A <see cref="System.Int32"/> Number of thread
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> True if all ok, false if .arc file is corrupted.
		/// </returns>
		bool CrawlerThread(int num)
		{
			//Start crawling
			this.crawlerinfo[num][4] = -1;
			this.crawlerinfo[num][2] = (int)CrawlerStatus.Crawling;
			this.crawlerinfo[num][0] = 0;
			this.crawlerinfo[num][3] = 0;
			this.cururl[num] = new StringBuilder();
			this.OnMessageEmit(EventArgs.Empty);
			int[] infostr2 = new int[4];
			double tmpreceived;
			bool returnvalue = false;
			FileStream workunit = null;
			FileStream tmpsitemap = null;
			try
			{
				if (this.crawlerinfo[num][5] == 0)
				{
					workunit = File.Create(this.filepath + this.workunits[num].ArcName);
					byte[] filebuffer = System.Text.Encoding.UTF8.GetBytes("filedesc://dummy.arc.gz 0.0.0.0 20071005122244 text/plain 69\n1 0 grub.org\nURL IP-address Archive-date Content-type Archive-length\n");
					workunit.Write(filebuffer, 0, filebuffer.Length);
					workunit.Flush();
					filebuffer = null;
					if (this.createsitemaps)
					{
						tmpsitemap = File.Create(this.filepath + this.workunits[num].ArcName + ".tmp");
					}
				}
				else
				{
					workunit = File.Open(this.filepath + this.workunits[num].ArcName, FileMode.Append);
					if (this.createsitemaps)
					{
						tmpsitemap = File.Open(this.filepath + this.workunits[num].ArcName + ".tmp", FileMode.Append);
					}
				}
				string[] xmlnodes = new string[2] {"/work/workunit", "/work/workiter"};
				string[] xmlvalues = new string[2];
				for (int i = this.crawlerinfo[num][5]; i < this.workunits[num].Amount; i++)
				{
					if (this.workunits[num].GetField("methods", i) == "GET")
					{
						this.crawlerinfo[num][0] = i + 1;
						this.cururl[num] = new StringBuilder(this.workunits[num].GetField("urls", i));
						this.crawlerinfo[num][3] = 0;
						this.OnMessageEmit(EventArgs.Empty);
						this.grubng.Downloading += 1;
						infostr2 = this.grubng.Crawling2(this.workunits[num].GetField("urls", i), this.workunits[num].GetField("hosts", i), 
						                                 ref workunit, this.workunits[num].GetField("versions", i), 
						                                 this.workunits[num].Headers(i), ref tmpsitemap, this.createsitemaps);
						this.grubng.Downloading -= 1;
						this.crawlerinfo[num][3] = infostr2[0];
						this.OnMessageEmit(EventArgs.Empty);
						tmpreceived = Convert.ToDouble(infostr2[1]) / 1048576;
						this.crawlerstats[0] ++;
						this.overallstats[0] ++;
						switch (infostr2[0])
						{
						case 200:
							this.crawlerstats[1] ++;
							this.overallstats[1] ++;
							break;
						case 304:
							this.crawlerstats[1] ++;
							this.overallstats[1] ++;
							break;
						case 500:
							this.crawlerstats[3] ++;
							this.overallstats[3] ++;
							break;
						default:
							this.crawlerstats[2] ++;
							this.overallstats[2] ++;
							break;
						}
						this.uptimestats[1] += tmpreceived;
						this.overallconnectionstats[1] += tmpreceived;
						tmpreceived = Convert.ToDouble(infostr2[3]) / 1048576;
						this.uptimestats[0] += tmpreceived;
						this.overallconnectionstats[0] += tmpreceived;
						if (infostr2[2] == 1)
						{
							this.timeout ++;
						}
						else
						{
							this.timeout = 0;
						}
						if (!File.Exists(this.filepath + this.workunits[num].ArcName))
						{
							this.crawlerinfo[num][4] = (int)CrawlerMessage.ProgramError;
							this.crawlerinfo[num][2] = (int)CrawlerStatus.ProgramError;
							this.OnMessageEmit(EventArgs.Empty);
							if (tmpsitemap != null)
							{
								tmpsitemap.Close();
								tmpsitemap = null;
								File.Delete(this.filepath + this.workunits[num].ArcName + ".tmp");
							}
							File.Delete(this.filepath + "workunit" + num.ToString());
							File.Delete(this.filepath + num.ToString() + ".xml");
							return false;
						}
					}
					if (this.workunits[num].GetField("methods", i) == "PUT")
					{
						//Check arc file
						workunit.Close();
						workunit = null;
						if (tmpsitemap != null)
						{
							tmpsitemap.Close();
							tmpsitemap = null;
						}
						if (ARCCheck.CheckFile(this.filepath + "workunit" + num.ToString(), this.filepath + this.workunits[num].ArcName, 
						                       this.workunits[num].Amount - 1))
						{
							//Compress .arc file
							Compress.Do(this.filepath + this.workunits[num].ArcName, this.compression);
							File.Delete(this.filepath + this.workunits[num].ArcName);
							xmlvalues[0] = this.workunits[num].ArcName;
							xmlvalues[1] = "0";
							Config.WriteFew(xmlvalues, xmlnodes, this.filepath + num.ToString() + ".xml");
							this.crawlerinfo[num][4] = (int)CrawlerMessage.FileCompressed;
							this.OnMessageEmit(EventArgs.Empty);
							//Create sitemap file from temporary file
							if (this.createsitemaps)
							{
								this.crawlerinfo[num][4] = (int)CrawlerMessage.GenerateSitemap;
								this.OnMessageEmit(EventArgs.Empty);
								this.sitemap = new Sitemap((this.filepath + this.workunits[num].ArcName) + ".xml");
								this.sitemap.CreateSitemap(true);
							}
							returnvalue = true;
						}
						else
						{
							Config.WriteConfig("0", "/work/workiter", this.filepath + num.ToString() + ".xml");
							this.crawlerinfo[num][2] = (int)CrawlerStatus.InvalidArcFile;
							this.OnMessageEmit(EventArgs.Empty);
							if (this.createsitemaps)
							{
								File.Delete(this.filepath + this.workunits[num].ArcName + ".tmp");
							}
							returnvalue = false;
						}
					}
					//Pause crawler
					if (this.crawlerinfo[num][1] == 2)
					{
						Config.WriteConfig(this.crawlerinfo[num][0].ToString(), "/work/workiter", this.filepath + num.ToString() + ".xml");
						this.crawlerinfo[num][2] = (int)CrawlerStatus.Paused;
						this.OnMessageEmit(EventArgs.Empty);
						while (this.crawlerinfo[num][1] == 2)
						{
							Thread.Sleep(1000);
						}
						this.crawlerinfo[num][2] = (int)CrawlerStatus.Crawling;
					}
					//Stop crawler (during quit from program)
					if (this.crawlerinfo[num][1] == 0)
					{
						Config.WriteConfig(this.crawlerinfo[num][0].ToString(), "/work/workiter", this.filepath + num.ToString() + ".xml");
						this.crawlerinfo[num][2] = (int)CrawlerStatus.Stopped;
						this.OnMessageEmit(EventArgs.Empty);
						return false;
					}
				}
			}
			finally
			{
				if (workunit != null)
				{
					workunit.Dispose();
				}
				if (tmpsitemap != null)
				{
					tmpsitemap.Dispose();
				}
			}
			return returnvalue;
		}
		
		/// <summary>
		/// Upload thread function. Function upload .arc files on server
		/// </summary>
		/// <param name="num">
		/// A <see cref="System.Int32"/> Number of thread
		/// </param>
		void UploadThread(int num)
		{
			this.uploadstage = 1;
			this.grubng.BytesSend = 0;
			this.grubng.FileLength = 0;
			//Upload arc file on serwer
			this.crawlerinfo[num][4] = -1;
			this.servermessage[num] = String.Empty;
			this.crawlerinfo[num][2] = (int)CrawlerStatus.Uploading;
			this.OnMessageEmit(EventArgs.Empty);
			string filename = FindFile(this.filepath + this.workunits[num].ArcName);
			int[] httpstatus = this.grubng.Upload(this.workunits[num].UploadURL + filename, filename);
			this.grubng.BytesSend = (int)this.grubng.FileLength;
			switch (httpstatus[0])
			{
			case 0:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.NoResponse;
				break;
			case 200:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.UploadedAndAdded;
				break;
			case 401:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.InvalidWorkunit;
				this.servermessage[num] = this.grubng.ServerMessage;
				break;
			case 402:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.OldResults;
				break;
			case 403:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.UploadDisabled;
				break;
			case 408:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.TimeoutReached;
				this.timeout += 50;
				break;
			case 409:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.UploadedEarlier;
				break;
			case 500:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.UploadServerError;
				this.servermessage[num] = this.grubng.ServerMessage;
				break;
			case 503:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.CannotConnect;
				break;
			default:
				this.crawlerinfo[num][4] = (int)CrawlerMessage.UnknownError;
				Logs.WriteLog(this.filepath + "error.log", "Unknow server answer on upload .arc file: " + httpstatus[0].ToString());
				break;
			}
			this.OnMessageEmit(EventArgs.Empty);
			double mbtmp = Convert.ToDouble(httpstatus[1]) / 1048576;
			this.uptimestats[0] += mbtmp;
			this.overallconnectionstats[0] += mbtmp;
			mbtmp = Convert.ToDouble(httpstatus[2]) / 1048576;
			this.uptimestats[1] += mbtmp;
			this.overallconnectionstats[1] += mbtmp;
			if (httpstatus[0] == 200)
			{
				//Compress and send sitemap on server
				if ((this.createsitemaps) && (File.Exists(this.filepath + this.workunits[num].ArcName + ".xml.gz")))
				{
					this.uploadstage = 2;
					this.sitemap = new Sitemap(this.filepath + this.workunits[num].ArcName + ".xml.gz");
					httpstatus = this.sitemap.UploadSitemap(false);
					if (httpstatus[0] == 417)
					{
						httpstatus = this.sitemap.UploadSitemap(true);
					}
					switch (httpstatus[0])
					{
					case 0:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.NoResponse;
						break;
					case 200:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.SitemapUploaded;
						break;
					case 401:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.InvalidLogin;
						break;
					case 403:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.SitemapDisabled;
						break;
					case 408:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.TimeoutReached;
						this.timeout += 50;
						break;
					case 500:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.UploadSitemapServerError;
						break;
					case 503:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.CannotConnect;
						break;
					default:
						this.crawlerinfo[num][4] = (int)CrawlerMessage.UnknownError;
						Logs.WriteLog(this.filepath + "error.log", "Unknow server answer on upload sitemap: " + httpstatus[0].ToString());
						break;
					}
					this.OnMessageEmit(EventArgs.Empty);
					mbtmp = Convert.ToDouble(httpstatus[2]) / 1048576;
					this.uptimestats[0] += mbtmp;
					this.overallconnectionstats[0] += mbtmp;
					mbtmp = Convert.ToDouble(httpstatus[1]) / 1048576;
					this.uptimestats[1] += mbtmp;
					this.overallconnectionstats[1] += mbtmp;
					File.Delete(this.filepath + this.workunits[num].ArcName + ".xml.gz");
				}
				//Clear all crawler files and info
				File.Delete(this.filepath + num.ToString() + ".xml");
                File.Delete(this.filepath + "workunit" + num.ToString());
				File.Delete(filename);
				this.crawlerinfo[num][2] = (int)CrawlerStatus.Stopped;
				this.crawlerinfo[num][0] = 0;
				this.downloaddate[num] = new StringBuilder();
				this.OnMessageEmit(EventArgs.Empty);
				this.crawlerstats[4] ++;
				this.overallstats[4] ++;
			}
			else
			{
				if (httpstatus[0] == 401)
				{
					File.Delete(filename);
					if (this.createsitemaps)
					{
						File.Delete(this.filepath + this.workunits[num].ArcName + ".xml.gz");
					}
					Config.WriteConfig(String.Empty, "/work/workunit", this.filepath + num.ToString() + ".xml");
					this.crawlerstats[5] ++;
					this.overallstats[5] ++;
					this.crawlerstats[4] ++;
					this.overallstats[4] ++;
				}
				if ((httpstatus[0] == 409) || (httpstatus[0] == 402))
				{
					File.Delete(this.filepath + num.ToString() + ".xml");
					File.Delete(this.filepath + "workunit" + num.ToString());
					File.Delete(filename);
					this.crawlerinfo[num][0] = 0;
					this.downloaddate[num] = new StringBuilder();
					this.crawlerstats[5] ++;
					this.overallstats[5] ++;
					this.crawlerstats[4] ++;
					this.overallstats[4] ++;
				}
				this.crawlerinfo[num][2] = (int)CrawlerStatus.UploadError;
				this.OnMessageEmit(EventArgs.Empty);
				this.connectionerror = true;
			}
		}
				
		/// <summary>
		/// Function invoke event. Raised when crawler send a message.
		/// </summary>
		/// <param name="args">
		/// A <see cref="EventArgs"/> Arguments send by event.
		/// </param>
		public void OnMessageEmit(EventArgs args)
		{
			EventHandler handler = this.messageEmit;
			if (handler != null)
			{
				handler(this, args);
			}
		}
			
		//// <value>
		/// Property read private field crawlers.
		/// </value>
		public int Crawlers
		{
			get {return this.crawlers;}
		}
		
		//// <value>
		/// Property read private field maxcrawlers.
		/// </value>
		public int MaxCrawlers
		{
			get {return this.maxcrawlers;}
		}
		
		//// <value>
		/// Property read/write private field running.
		/// </value>
		public bool Running
		{
			get {return this.running;}
			set {this.running = value;}
		}
		
		//// <value>
		/// Property read/write private fields timeout.
		/// </value>
		public int TimeOut
		{
			get {return this.timeout;}
			set {this.timeout = value;}
		}
		
		/// <summary>
		/// Function read double fields.
		/// </summary>
		/// <param name="name">
		/// A <see cref="System.String"/> field name.
		/// </param>
		/// <returns>
		/// A <see cref="System.Double"/> value of field.
		/// </returns>
		public double GetDoubleField(string name)
		{
			this.DisposeCheck();
			switch (name)
			{
			case "uptimesend":
				return this.uptimestats[0];
			case "uptimereceived":
				return this.uptimestats[1];
			case "overallsent":
				return this.overallconnectionstats[0];
			case "overallreceived":
				return this.overallconnectionstats[1];
			default:
				return 0;
			}
		}
		
		/// <summary>
		/// Function read string fields
		/// </summary>
		/// <param name="index">
		/// A <see cref="System.Int32"/> Element number in list
		/// </param>
		/// <param name="name">
		/// A <see cref="System.String"/> Field name
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> Value of field
		/// </returns>
		public string GetStringField(int index, string name)
		{
			this.DisposeCheck();
			switch (name)
			{
			case "workfilename":
				return this.workunits[index].ArcName;
			case "downloaddate":
				return this.downloaddate[index].ToString();
			case "cururl":
				return this.cururl[index].ToString();
			case "servermessage":
				return this.servermessage[index];
			default:
				return String.Empty;
			}
		}
		
		/// <summary>
		/// Function set values for private fields
		/// </summary>
		/// <param name="index">
		/// A <see cref="System.Int32"/> Element number in list
		/// </param>
		/// <param name="name">
		/// A <see cref="System.String"/> Field name
		/// </param>
		/// <param name="val">
		/// A <see cref="System.Int32"/> New field value (used only for working field)
		/// </param>
		public void SetField(int index, string name, int val)
		{
			this.DisposeCheck();
			switch (name)
			{
			case "working":
				this.crawlerinfo[index][1] = val;
				break;
			case "curstatus":
				this.crawlerinfo[index][3] = 0;
				break;
			case "message":
				this.crawlerinfo[index][4] = -1;
				break;
			case "cururl":
				this.cururl[index] = new StringBuilder(null);
				break;
			case "servermessage":
				this.servermessage[index] = String.Empty;
				break;
			case "crawlerstats":
				for (int i = 0; i < 6; i++)
				{
					this.crawlerstats[i] = 0;
					this.overallstats[i] = 0;
				}
				for (int i = 0; i < 2; i++)
				{
					this.uptimestats[i] = 0;
					this.overallconnectionstats[i] = 0;
				}
				break;
			default:
				break;
			}
		}
		
		/// <summary>
		/// Function read private fields
		/// </summary>
		/// <param name="index">
		/// A <see cref="System.Int32"/> Element number in list
		/// </param>
		/// <param name="name">
		/// A <see cref="System.String"/> Field name
		/// </param>
		/// <returns>
		/// A <see cref="System.Int32"/> Value of private field
		/// </returns>
		public int GetField(int index, string name)
		{
			this.DisposeCheck();
			switch (name)
			{
			case "iter":
				return this.workunits[index].Amount;
			case "curiter":
				return this.crawlerinfo[index][0];
			case "working":
				return this.crawlerinfo[index][1];
			case "status":
				return this.crawlerinfo[index][2];
			case "curstatus":
				return this.crawlerinfo[index][3];
			case "message":
				return this.crawlerinfo[index][4];
			case "crawledurls":
				return this.crawlerstats[0];
			case "validurls":
				return this.crawlerstats[1];
			case "invalidurls":
				return this.crawlerstats[2];
			case "errorurls":
				return this.crawlerstats[3];
			case "workunitsend":
				return this.crawlerstats[4];
			case "invalidarc":
				return this.crawlerstats[5];
			case "overallcrawled":
				return this.overallstats[0];
			case "overallvalid":
				return this.overallstats[1];
			case "overallinvalid":
				return this.overallstats[2];
			case "overallerror":
				return this.overallstats[3];
			case "overallworkunit":
				return this.overallstats[4];
			case "overallinvalidworkunit":
				return this.overallstats[5];
			default:
				return 0;
			}
		}
			
		/// <summary>
		/// Function count progress of upload workunit on server.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Double"/> percent of uploaded workunit.
		/// </returns>
		public double UploadProgress()
		{
			this.DisposeCheck();
			double temp;
			if (this.uploadstage == 1)
			{
				temp = (double)this.grubng.BytesSend / (double)this.grubng.FileLength * 100;
			}
			else
			{
				temp = (double)this.sitemap.BytesSend / (double)this.sitemap.FileLength * 100;
			}
			return temp;
		}
		
		/// <summary>
		/// Function check status of crawlers.
		/// </summary>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if all crawlers stopped, otherwise false.
		/// </returns>
		public bool CrawlersStopped()
		{
			this.DisposeCheck();
			for (int i = 0; i < this.crawlers; i++)
			{
				if (this.crawlerthread[i] != null)
				{
					if (!this.crawlerthread[i].IsCompleted)
					{
						return false;
					}
				}
			}
			return true;
		}
		
		/// <summary>
		/// Check for dispose. If was called, throw exception
		/// </summary>
		void DisposeCheck()
		{
			if (this.disposedres)
			{
				throw new ObjectDisposedException(GetType().Name);
			}
		}
		
		/// <summary>
		/// Function dispose unmanaged resources.
		/// </summary>
		public void Dispose()
		{
			if (!disposedres)
			{
				disposedres = true;
			}
		}
		
		//// <value>
		/// Property read/write private field connectionerror.
		/// </value>
		public bool ConnectionError
		{
			get {return this.connectionerror;}
			set {this.connectionerror = value;}
		}
	}
}
