// Updater.cs
// 
// Copyright (C) 2008,2009,2010 Bartek Jasicki
//
// This file is part of Grubng.
//
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//   Grubng is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Grubng.  If not, see <http://www.gnu.org/licenses/>.
//

/**@file Updater.cs
 * Provide functions for automatic client upgrading.*/

using System;
using System.Net;
using System.IO;
using System.Xml;

namespace Grubng
{
	
	/// <summary>
	/// Provide functions for automatic client upgrading.
	/// </summary>
	public sealed class Updater
	{
		/// <summary>
		/// Version of program
		/// </summary>
		string version;
		
		/// <summary>
		/// Standard class constructor.
		/// </summary>
		/// <param name="userversion">
		/// A <see cref="System.String"/> version of client.
		/// </param>
		public Updater(string userversion)
		{
			this.version = userversion;
		}
		
		/// <summary>
		/// Function connect to server and get info about updates.
		/// </summary>
		/// <param name="unstable">
		/// A <see cref="System.String"/> if Y - check for unstable upgrades, if N - check for stable updates
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> none - no updates available, current - update by one version (example: from
		/// 0.4.6 to 0.4.7), older - update by few versions (example: from 0.4.6 to 0.4.8).
		/// </returns>
		public string CheckUpdate(string unstable)
		{
			string type;
			if (unstable == "Y")
			{
				type = "testing";
			}
			else
			{
				type = "stable";
			}
			Config conf = new Config();
			string updateserver = Config.ReadConfig("/configuration/other/updateserver", conf.FilePath);
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(updateserver + type + "/update.xml");
			request.Proxy = null;
			request.KeepAlive = false;
			HttpWebResponse response;
			try
			{
				response = (HttpWebResponse)request.GetResponse();
			}
			catch
			{
				return "failed";
			}
			string result;
			using (StreamReader streamr = new StreamReader(response.GetResponseStream()))
			{
				result = streamr.ReadToEnd();
			}
			using (StreamWriter streamw = new StreamWriter("update.xml"))
			{
				streamw.Write(result);
			}
			response.Close();
			XmlDocument doc = new XmlDocument();
			doc.Load("update.xml");
			XmlNode noder = doc.SelectSingleNode("/update/current");
			string answer = noder.Attributes["ver"].InnerText;
			noder = doc.SelectSingleNode("/update/older");
			string answer2 = noder.Attributes["ver"].InnerText;
			doc = null;
			if (this.version == answer)
			{
				File.Delete("update.xml");
				return "none";
			}
			if (this.version == answer2)
			{
				return "current";
			}
			else
			{
				return "older";
			}
		}
		
		/// <summary>
		/// Function update program from earlier version.
		/// </summary>
		/// <param name="type">
		/// A <see cref="System.String"/> type of upgrade: none - no updates available, current - update 
		/// by one version (example: from 0.4.6 to 0.4.7), older - update by few versions (example: from 0.4.6 to 0.4.8).
		/// </param>
		/// <param name="unstable">
		/// A <see cref="System.String"/> check for unstable upgrades, if N - check for stable updates.
		/// </param>
		/// <param name="rootpass">
		/// A <see cref="System.String"/> password for sudo command - needed only on wide system installation on Unix systems.
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> array with names of upgraded files.
		/// </returns>
		public static string[] Update(string type, string unstable, string rootpass)
		{
			if (type == "none")
			{
				return new string[0];
			}
			string path;
			if (unstable == "Y")
			{
				path = "testing";
			}
			else
			{
				path = "stable";
			}
			XmlDocument doc = new XmlDocument();
			doc.Load("update.xml");
			XmlNode filesList;
			string filename;
			XmlElement fileatr;
			string updatedfiles = String.Empty;
			string system = null;
			string platform = null;
			if (Environment.OSVersion.Platform.ToString() == "Unix")
			{
				platform = "Unix";
			}
			else
			{
				platform = "Windows";
			}
			if (type == "older")
			{
				filesList  = doc.SelectSingleNode("/update/older");
				fileatr = (XmlElement)filesList;
				//Update files
				for (int i = 0; i < fileatr.GetElementsByTagName("file").Count; i++)
				{
					filename = fileatr.GetElementsByTagName("file")[i].InnerText;
					system = fileatr.GetElementsByTagName("file")[i].Attributes["system"].InnerText;
					if ((system == "all") || (system == platform))
					{
						Updater.UpdateFile(filename, path, rootpass, system);
						if (i == 0)
						{
							updatedfiles = filename;
						}
						else
						{
							updatedfiles += ";" + filename;
						}
					}
				}
			}
			filesList  = doc.SelectSingleNode("/update/current");
			fileatr = (XmlElement)filesList;
			//Update files
			for (int i = 0; i < fileatr.GetElementsByTagName("file").Count; i++)
			{
				filename = fileatr.GetElementsByTagName("file")[i].InnerText;
				system = fileatr.GetElementsByTagName("file")[i].Attributes["system"].InnerText;
				if ((system == "all") || (system == platform))
				{
					Updater.UpdateFile(filename, path, rootpass, system);
					if (i == 0)
					{
						updatedfiles = filename;
					}
					else
					{
						updatedfiles += ";" + filename;
					}
				}
			}
			doc = null;
			File.Delete("update.xml");
			return updatedfiles.Split(';');
		}
		
		/// <summary>
		/// Function update files. It download files to work directory and move they to
		/// directory with program
		/// </summary>
		/// <param name="fname">
		/// A <see cref="System.String"/> full path to file to update
		/// </param>
		/// <param name="wwwpath">
		/// A <see cref="System.String"/> type of update - stable or unstable
		/// </param>
		/// <param name="pass">
		/// A <see cref="System.String"/> neede only on wide install on Unix systems
		/// </param>
		/// <param name="system">
		/// A <see cref="System.String"/> system name for which is updating file (all, Unix, Windows)
		/// </param>
		static void UpdateFile(string fname, string wwwpath, string pass, string system)
		{
			Config conf = new Config();
			string[] nodes = new string[] {
				"/configuration/work/workdir",
				"/configuration/other/updateserver",
				"/configuration/work/proxystatus"};
			string[] values = Config.ReadFew(nodes, conf.FilePath);
			if (system != "all")
			{
				wwwpath += "/" + system;
			}
			string proxyaddress, proxyport, proxylogin, proxypass;
			if (values[2] == "custom")
			{
				nodes = new string[] {
					"/configuration/work/proxyaddress",
					"/configuration/work/proxyport",
					"/configuration/work/proxylogin",
					"/configuration/work/proxypass"};
				string[] proxyvals = Config.ReadFew(nodes, conf.FilePath);
				proxyaddress = proxyvals[0];
				proxyport = proxyvals[1];
				proxylogin = proxyvals[2];
				proxypass = proxyvals[3];
				proxyvals = null;
			}
			else
			{
				proxyaddress = null;
				proxyport = null;
				proxylogin = null;
				proxypass = null;
			}
			nodes = null;
			conf = null;
			HttpWebRequest request = (HttpWebRequest)WebRequest.Create(values[1] + wwwpath + "/" + fname);
			if (values[2] == "custom")
			{
				Uri proxyuri = new Uri(proxyaddress + ":" + proxyport);
				request.Proxy = new WebProxy(proxyuri);
				if (!String.IsNullOrEmpty(proxylogin))
				{
					request.Proxy.Credentials = new NetworkCredential(proxylogin, proxypass);
				}
			}
			if (values[2] == "none")
			{
				request.Proxy = null;
			}
			request.KeepAlive = false;
			HttpWebResponse response = (HttpWebResponse)request.GetResponse();
			string[] upgradefile = fname.Split('/');
			string filename = values[0] + upgradefile[upgradefile.Length - 1];
			byte[] buffer = new byte[4096];
			int bytesRead = 0;
			using (Stream uploader = response.GetResponseStream())
			{
				using (FileStream file = File.Create(filename))
				{
					while ((bytesRead = uploader.Read(buffer, 0, buffer.Length)) != 0)
					{
						file.Write(buffer, 0, bytesRead);
					}
					file.Close();
				}
				uploader.Close();
			}
			response.Close();
			response = null;
			System.Text.StringBuilder tmpdir = new System.Text.StringBuilder(System.AppDomain.CurrentDomain.BaseDirectory);
			for (int j = 0; j < upgradefile.Length - 1; j++)
			{
				tmpdir.Append(Path.DirectorySeparatorChar);
				tmpdir.Append(upgradefile[j]);
			}
			string directory = tmpdir.ToString();
			tmpdir = null;
			string fname2 = directory + Path.DirectorySeparatorChar + upgradefile[upgradefile.Length - 1];
			if (String.IsNullOrEmpty(pass))
			{
				Directory.CreateDirectory(directory);
				if (filename != fname2)
				{
					try
					{
						File.Copy(filename, fname2, true);
					}
					catch (IOException)
					{
						System.Threading.Thread.Sleep(500);
						File.Copy(filename, fname2, true);				
					}
				}
			}
			else
			{
				StreamWriter streamw;
				System.Diagnostics.ProcessStartInfo procinfo = new System.Diagnostics.ProcessStartInfo();
				procinfo.Arguments = "mkdir -p " + directory;
				procinfo.FileName = "sudo";
				procinfo.RedirectStandardInput = true;
				procinfo.UseShellExecute = false;
				using (System.Diagnostics.Process proc = System.Diagnostics.Process.Start(procinfo))
				{
					streamw = proc.StandardInput;
					try
					{
						streamw.WriteLine(pass);
						streamw.Close();
					}
					catch (Exception e)
					{
						Logs.WriteLog(values[0] + "error.log", "Updater.UpdateFile: " + e.ToString());
					}
					proc.WaitForExit();
				}
				System.Diagnostics.ProcessStartInfo procinfo2 = new System.Diagnostics.ProcessStartInfo();
				procinfo2.Arguments = "mv -f " + filename + " " + fname2;
				procinfo2.FileName = "sudo";
				procinfo2.RedirectStandardInput = true;
				procinfo2.UseShellExecute = false;
				using (System.Diagnostics.Process proc2 = System.Diagnostics.Process.Start(procinfo2))
				{
					streamw = proc2.StandardInput;
					try
					{
						streamw.WriteLine(pass);
						streamw.Close();
					}
					catch
					{
					}
					proc2.WaitForExit();
				}
			}
			values = null;
			if (filename != fname2)
			{
				File.Delete(filename);
			}
		}
	}
}
