// Workunit.cs
//  
//  Copyright (C) 2009,2010,2011 Bartek thindil Jasicki
// 
//  This file is part of Grubng
// 
//  Grubng is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 

/**@file Workunits.cs
 * Provide functions for manage workunits files.*/

using System;
using System.Collections.Generic;

namespace Grubng
{
	
	/// <summary>
	/// Provide functions for manage workunits files.
	/// </summary>
	internal sealed class Workunit
	{
		/// <summary>
		/// List of URL to crawl
		/// </summary>
		List<string> urls;
		/// <summary>
		/// List of hosts to crawl
		/// </summary>
		List<string> hosts;
		/// <summary>
		/// List of HTTP method
		/// </summary>
		List<string> methods;
		/// <summary>
		/// List of HTTP version
		/// </summary>
		List<string> versions;
		/// <summary>
		/// List of headers
		/// </summary>
		List<List<string>> headers;
		/// <summary>
		/// Amount of links to crawl
		/// </summary>
		int amount;
		/// <summary>
		/// Name of .arc file
		/// </summary>
		string arcname;
		/// <summary>
		/// URL to upload compressed .arc file
		/// </summary>
		string uploadurl;
		
		/// <summary>
		/// Standard class constructor
		/// </summary>
		public Workunit()
		{
			this.urls = new List<string>();
			this.hosts = new List<string>();
			this.methods = new List<string>();
			this.versions = new List<string>();
			this.headers = new List<List<string>>();
			this.arcname = String.Empty;
		}
		
		/// <summary>
		/// Function read selected workunit file and fill all crawler informations.
		/// </summary>
		/// <param name="path">
		/// A <see cref="System.String"/> full path (including filename) to workunits file.
		/// </param>
		/// <param name="username">
		/// A <see cref="System.String"/> user name.
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/> true if workunit is correct, otherwise false.
		/// </returns>
		public bool Read(string path, string username)
		{
			bool valid = false;
			using (System.IO.StreamReader file = new System.IO.StreamReader(path))
			{
				System.Text.StringBuilder fileline = new System.Text.StringBuilder();
				string[] urli;
				int index;
				bool isfirst = true;
				while (!file.EndOfStream)
				{
					fileline.Append(file.ReadLine());
					if (fileline.Length == 0)
					{
						break;
					}
					urli = fileline.ToString().Split(' ');
					if (urli.Length < 3)
					{
						break;
					}
					if (urli[0] == "GET")
					{
						isfirst = false;
						this.methods.Add(urli[0]);
						this.versions.Add(urli[2]);
						fileline.Remove(0, fileline.Length);
						fileline.Append(file.ReadLine());
						if (!fileline.ToString().StartsWith("Host:"))
						{
							break;
						}
						this.hosts.Add(fileline.ToString().Remove(0, 6));
						this.urls.Add("http://" + fileline.ToString().Remove(0, 6) + urli[1]);
						this.headers.Add(new List<string>());
						while (fileline.Length > 0)
						{
							fileline.Remove(0, fileline.Length);
							fileline.Append(file.ReadLine());
							if (fileline.Length > 0)
							{
								this.headers[this.amount].Add(fileline.ToString());
							}
						}
						if (this.headers[this.amount].Count == 0)
						{
							break;
						}
					}
					if (urli[0] == "PUT")
					{
						if (isfirst)
						{
							break;
						}
						this.methods.Add(urli[0]);
						if (urli.Length > 3)
						{
							for (int i = 2; i < (urli.Length - 1); i++)
							{
								urli[1] += " " + urli[i];
							}
						}
						index = urli[1].IndexOf(username);
						if (index == -1)
						{
							break;
						}
						this.arcname = urli[1].Substring(index);
						this.arcname = this.arcname.Remove(this.arcname.Length - 3);
						fileline.Remove(0, fileline.Length);
						fileline.Append(file.ReadLine());
						if (!fileline.ToString().StartsWith("Host:"))
						{
							break;
						}
						this.uploadurl = "http://" + fileline.ToString().Substring(6) + "/";
						valid = true;
					}
					fileline.Remove(0, fileline.Length);
					this.amount ++;
				}
			}
			if (!valid)
			{
				this.Clear();
			}
			return valid;
		}
		
		/// <summary>
		/// Function read only upload URL and filename from workunit file
		/// </summary>
		/// <param name="path">
		/// A <see cref="System.String"/> full path (including filename) to workunits file.
		/// </param>
		/// <param name="username">
		/// A <see cref="System.String"/> user name.
		/// </param>
		public void FastRead(string path, string username)
		{
			using (System.IO.StreamReader file = new System.IO.StreamReader(path))
			{
				System.Text.StringBuilder fileline = new System.Text.StringBuilder();
				string[] urli;
				int index;
				while (!file.EndOfStream)
				{
					fileline.Append(file.ReadLine());
					if (fileline.ToString().StartsWith("PUT"))
					{
						urli = fileline.ToString().Split(' ');
						if (urli.Length > 3)
						{
							for (int i = 2; i < (urli.Length - 1); i++)
							{
								urli[1] += " " + urli[i];
							}
						}
						index = urli[1].IndexOf(username);
						this.arcname = urli[1].Substring(index);
						this.arcname = this.arcname.Remove(this.arcname.Length - 3);
						fileline.Remove(0, fileline.Length);
						fileline.Append(file.ReadLine());
						this.uploadurl = "http://" + fileline.ToString().Substring(6) + "/";
					}
					fileline.Remove(0, fileline.Length);
				}
			}
		}
		
		/// <summary>
		/// Function clear all data.
		/// </summary>
		public void Clear()
		{
			this.urls.Clear();
			this.urls.TrimExcess();
			this.hosts.Clear();
			this.hosts.TrimExcess();
			this.methods.Clear();
			this.methods.TrimExcess();
			this.versions.Clear();
			this.versions.TrimExcess();
			this.headers.Clear();
			this.headers.TrimExcess();
			this.amount = 0;
			this.arcname = String.Empty;
			this.uploadurl = String.Empty;
		}
		
		/// <summary>
		/// Function return value of selected private string field
		/// </summary>
		/// <param name="name">
		/// A <see cref="System.String"/> field name.
		/// </param>
		/// <param name="index">
		/// A <see cref="System.Int32"/> element number in list.
		/// </param>
		/// <returns>
		/// A <see cref="System.String"/> value of selected private field.
		/// </returns>
		public string GetField(string name, int index)
		{
			switch(name)
			{
			case "urls":
				return this.urls[index];
			case "hosts":
				return this.hosts[index];
			case "methods":
				return this.methods[index];
			case "versions":
				return this.versions[index];
			default:
				return String.Empty;
			}
		}
		
		/// <summary>
		/// Function return URL HTTP headers
		/// </summary>
		/// <param name="index">
		/// A <see cref="System.Int32"/> number of URL for which we get HTTP headers
		/// </param>
		/// <returns>
		/// A <see cref="List"/> of HTTP headers
		/// </returns>
		public List<string> Headers(int index)
		{
			return this.headers[index];
		}
		
		/// <value>
		/// Property return value of private field amount
		/// </value>
		public int Amount
		{
			get {return this.amount;}
		}
		
		/// <value>
		/// Property return value of private field arcname
		/// </value>
		public string ArcName
		{
			get {return this.arcname;}
		}
		
		/// <value>
		/// Property return value of private field uploadurl
		/// </value>
		public string UploadURL
		{
			get {return this.uploadurl;}
		}
	}
}
